; 6502bench SourceGen v1.5.0
;
; This is based on a disassembly of the Titan //e disk and then refactored into
; a SOS Interpreter to allow lauching of the //e emulation from a Selector menu.
; The Interpreter is loaded, and then the contents are moved around in memory
; to setup the environment that the boot loader did on the original //e disk
;
;
; Interpreter code layout
; 2000 - 20FF  - memory move and card init routines
; 2100 - 21FF  - Slot 5 rom image
; 2200 - 22FF  - Slot 6 rom image
; 2300 - 23FF  - Slot 7 rom image
; 2400 - 53FF  - TWOE rom image (basic, etc)
; 5400 - 57FF  - font data 
; 5800 - 63FF  - 
; 6400 - 6BFF  - alternate Cxxx rom image
; 6C00 - 83FF  -
; 8400 - 97FF  - actual emulation init code and menus
;
;
; Memory map (after code is moved into correct locations)
; 0C00 - 0DFF  - copy slot and applesoft roms into correct memory locations
; 0E00 - 13FF  - start menu and config menu things
; 1A00 - 1FFF  - menu stuff continued here
;
; 9000 - 93FF  - this is the IIe font that is loaded. Sits in bank0
; 9400 - 9FFF  - this is in bank 0, not sure what this is
; A000 - BFFF  - continues up to bfff. we copy it into place
;
;
;
;    (1.I.S.R:W_P.R.R) ::= ENVIRONMENT REGISTER FLAGS.  FROM LEFT TO RIGHT BITS 7..0 
;                          (1MHZ, I/O ENABLE, SCREEN ENABLE, RESET ENABLE, 
;                           WRITE PROTECT, PRIMARY STACK, ROM1, ROM ENABLE)
;


            .setcpu "6502"
SRCPTR      =       $00
DSTPTR      =       $02
STARTBLK    =       $04
ENDBLK      =       $05
IBBUFP      =       $85
IBCMD       =       $87
KBD         =       $c000      ;last key pressed
CLR80VID    =       $c00c      ;W disable 80-column display mode
KBDSTRB     =       $c010      ;RW keyboard strobe
SETTEXTGR   =       $c051      ;Set TEXT/GR mode
CLRPAGE2    =       $c054      ;Clear PAGE2 mode
CLOCK       =       $c070      ;clock
CLRENCWRT   =       $c0da      ;Clear Char Set writing
SETENCWRT   =       $c0db      ;Set Char Set writing
BLOCKIO     =       $f479
A3RESET     =       $f4ee
A2RESET     =       $fa62
Z_REG       =       $ffd0      ;zero page register
ENV_REG     =       $ffdf      ;environment register
E_PCR       =       $ffec
E_IFR       =       $ffed
E_IER       =       $ffee
BNK_REG     =       $ffef      ;bank switch register

L1400       =       $1400
L1401       =       $1401
L1403       =       $1403
L1800       =       $1800

KBDFLG      =       $c008
DRV1EN      =       $c08b


            .segment "RAM"
            .export  SLOT5ROM
            .export  SLOT6ROM
            .export  SLOT7ROM
            .export  TWOEROM            
 
            .org     $2000 - 14
; sos interp header
            .byte    "SOS NTRP"
            .word    0000
            .word    CODEST
            .word    CODELEN


CODEST:     lda     #$18       ;set zero page to $18
            sta     Z_REG
            ldx     #$00       ;disable extended addressing for SRCPTR & DSTPTR
            stx     L1401
            stx     L1403

; copy the code from the interp at CODE0C00 to $0C00 - $13FF
            lda     #$00
            sta     DSTPTR
            lda     #$0C
            sta     DSTPTR+1
            lda     #<CODE0C00
            sta     SRCPTR
            lda     #>CODE0C00
            sta     SRCPTR+1
            ldx     #$13
            jsr     MOVEBLK
            
; copy the code from the interp at CODE0C00+$E00 to $1A00 - $1FFF
            lda     #$00
            sta     DSTPTR
            lda     #$1A
            sta     DSTPTR+1
            lda     #<(CODE0C00+$E00)
            sta     SRCPTR
            lda     #>(CODE0C00+$E00)
            sta     SRCPTR+1
            ldx     #$1F
            jsr     MOVEBLK

; copy 9000 - bfff
; this is copied using extended addressing
; 9000-9fff to bank0  (from current bank)
; A000-bfff in system bank
            lda     #$00
            sta     DSTPTR
            lda     #$90
            sta     DSTPTR+1
            lda     #<CODE9000
            sta     SRCPTR
            lda     #>CODE9000
            sta     SRCPTR+1
            lda     #$8f         ;store using special 8f case extended addressing
            sta     L1403
            ldx     #$bf
            jsr     MOVEBLK

; some init/test code from the original boot sector
; -need to work out what its doing, something with the language card
;
@LA07F:     lda     ENV_REG
            pha                ;save env_reg
            lda     #$fc       ;(1.I.S.R:W_P.R.R)
            sta     ENV_REG
            lda     Z_REG
            ldx     #$18
            stx     Z_REG
            pha                ;save zeropage reg
            lda     #$8f       ;enable special 8f ext addr
            sta     $1401
            lda     #$00       ;set 00 to $D000
            sta     $00
            lda     #$d0
            sta     $01
            ldy     #$00
            ldx     #$df
            lda     $c300      ;enable 3plus2 card (its always in slot3)
            lda     $c083
            lda     #$00
@LA0AA:     sta     KBDFLG
            sta     $c006
            sta     $c083
            sta     ($00),y
            sta     $c009
            sta     ($00),y
            sta     KBDFLG-1
            sta     ($00),y
            sta     KBDFLG
            sta     ($00),y
            sta     $c006
            cpx     $01
            bcc     @LA0DF
            sta     DRV1EN
            sta     ($00),y
            sta     $c009
            sta     ($00),y
            sta     KBDFLG-1
            sta     ($00),y
            sta     KBDFLG
            sta     ($00),y
@LA0DF:     iny
            bne     @LA0AA
            inc     $01
            bne     @LA0AA
            sta     $c082
            pla                ;restore zero page
            sta     Z_REG
            pla                ;restore env reg
            sta     ENV_REG


            jmp     $0C00

; copy memory from SRCPTR to DSTPTR
MOVEBLK:    ldy     #0
CC1:        lda     (SRCPTR),Y
            sta     (DSTPTR),Y
            iny
            bne     CC1
            inc     DSTPTR+1
            inc     SRCPTR+1
            txa
            cmp     DSTPTR+1
            bcs     CC1
            rts

            .res    $2100 - *,0
 
;emulation rom images

; copied to c500
SLOT5ROM:   .byte   $00,$20,$00,$00,$00,$03,$AC,$7D,$04,$B9,$83,$C0,$B9,$83,$C0,$60
            .byte   $20,$D3,$C5,$A5,$44,$85,$3E,$A5,$45,$85,$3F,$E6,$3F,$A0,$00,$60
            .byte   $AE,$AA,$C7,$AC,$AB,$C7,$A9,$00,$18,$60,$A5,$42,$F0,$F2,$C9,$03
            .byte   $F0,$F4,$A0,$00,$AD,$11,$C0,$10,$02,$A0,$08,$8C,$7D,$04,$A5,$3E
            .byte   $48,$A5,$3F,$48,$08,$78,$AD,$FF,$CF,$A5,$42,$C9,$02,$F0,$13,$A5
            .byte   $46,$A6,$47,$8D,$A1,$C0,$20,$5A,$A0,$99,$A9,$C0,$20,$8A,$C5,$4C
            .byte   $6B,$C5,$20,$AA,$C5,$8D,$A1,$C0,$20,$42,$A0,$20,$CD,$C5,$20,$06
            .byte   $C5,$AC,$A0,$C7,$99,$AD,$C0,$A0,$00,$90,$02,$A0,$27,$28,$68,$85
            .byte   $3F,$68,$85,$3E,$98,$18,$F0,$01,$38,$60,$2C,$FF,$CF,$B0,$18,$20
            .byte   $10,$C5,$20,$06,$C5,$A0,$00,$B9,$00,$C8,$91,$44,$B9,$00,$C9,$91
            .byte   $3E,$C8,$D0,$F3,$AD,$82,$C0,$4C,$CC,$C2,$2C,$FF,$CF,$AC,$A0,$C7
            .byte   $99,$A9,$C0,$AC,$7D,$04,$B9,$80,$C0,$20,$10,$C5,$B1,$44,$99,$00
            .byte   $C8,$B1,$3E,$99,$00,$C9,$C8,$D0,$F3,$A5,$46,$A6,$47,$A0,$FC,$8C
            .byte   $DF,$FF,$60,$A0,$B4,$30,$F8,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$4F,$2A

; copied to c600
SLOT6ROM:   .byte   $A2,$20,$A0,$00,$A2,$03,$86,$3C,$8A,$0A,$24,$3C,$F0,$10,$05,$3C
            .byte   $49,$FF,$29,$7E,$B0,$08,$4A,$D0,$FB,$98,$9D,$56,$03,$C8,$E8,$10
            .byte   $E5,$20,$58,$FF,$BA,$BD,$00,$01,$0A,$0A,$0A,$0A,$85,$2B,$AA,$BD
            .byte   $8E,$C0,$BD,$8C,$C0,$BD,$8A,$C0,$BD,$89,$C0,$A0,$50,$BD,$80,$C0
            .byte   $98,$29,$03,$0A,$05,$2B,$AA,$BD,$81,$C0,$A9,$56,$20,$A8,$FC,$88
            .byte   $10,$EB,$85,$26,$85,$3D,$85,$41,$A9,$08,$85,$27,$18,$08,$BD,$8C
            .byte   $C0,$10,$FB,$49,$D5,$D0,$F7,$BD,$8C,$C0,$10,$FB,$C9,$AA,$D0,$F3
            .byte   $EA,$BD,$8C,$C0,$10,$FB,$C9,$96,$F0,$09,$28,$90,$DF,$49,$AD,$F0
            .byte   $25,$D0,$D9,$A0,$03,$85,$40,$BD,$8C,$C0,$10,$FB,$2A,$85,$3C,$BD
            .byte   $8C,$C0,$10,$FB,$25,$3C,$88,$D0,$EC,$28,$C5,$3D,$D0,$BE,$A5,$40
            .byte   $C5,$41,$D0,$B8,$B0,$B7,$A0,$56,$84,$3C,$BC,$8C,$C0,$10,$FB,$59
            .byte   $D6,$02,$A4,$3C,$88,$99,$00,$03,$D0,$EE,$84,$3C,$BC,$8C,$C0,$10
            .byte   $FB,$59,$D6,$02,$A4,$3C,$91,$26,$C8,$D0,$EF,$BC,$8C,$C0,$10,$FB
            .byte   $59,$D6,$02,$D0,$87,$A0,$00,$A2,$56,$CA,$30,$FB,$B1,$26,$5E,$00
            .byte   $03,$2A,$5E,$00,$03,$2A,$91,$26,$C8,$D0,$EE,$E6,$27,$E6,$3D,$A5
            .byte   $3D,$CD,$00,$08,$A6,$2B,$90,$DB,$4C,$01,$08,$00,$00,$00,$00,$00

; copied to c700
SLOT7ROM:   .byte   $08,$EA,$28,$2C,$58,$FF,$70,$03,$60,$EA,$EA,$08,$78,$A0,$00,$AD
            .byte   $11,$C0,$10,$02,$A0,$08,$8C,$7D,$04,$AC,$A0,$C7,$99,$A9,$C0,$AD
            .byte   $82,$C0,$A9,$11,$8D,$7F,$04,$A0,$14,$CE,$7F,$04,$F0,$41,$A2,$10
            .byte   $BD,$83,$C7,$30,$25,$8D,$D0,$FF,$AD,$70,$C0,$9D,$FF,$01,$8C,$D0
            .byte   $FF,$AD,$70,$C0,$D0,$E3,$BD,$FF,$01,$29,$0F,$09,$B0,$9D,$00,$02
            .byte   $CA,$BD,$00,$02,$4A,$4A,$4A,$4A,$09,$B0,$9D,$00,$02,$CA,$10,$D0
            .byte   $AD,$04,$02,$38,$E9,$01,$C9,$B0,$D0,$02,$A9,$B7,$8D,$04,$02,$28
            .byte   $A2,$11,$A9,$00,$8D,$D0,$FF,$20,$06,$C5,$AC,$A0,$C7,$99,$AD,$C0
            .byte   $A9,$8D,$60,$07,$07,$AC,$05,$05,$AC,$06,$06,$AC,$04,$04,$AC,$03
            .byte   $03,$AC,$02,$02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$18,$0B,$00,$00,$08,$1E,$0A,$00,$26,$80,$01,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$2C,$FF,$CF,$8D,$A1,$C0,$20,$28,$95,$99,$A9
            .byte   $C0,$4C,$8A,$C5,$99,$A9,$C0,$4C,$CC,$C2,$8D,$A1,$C0,$4C,$00,$0E
            .byte   $A0,$0E,$99,$00,$C0,$88,$88,$10,$F9,$20,$84,$FE,$20,$2F,$FB,$20
            .byte   $93,$FE,$4C,$89,$FE,$20,$AA,$C5,$8D,$A1,$C0,$20,$53,$94,$10,$D4

; copied to d000 - ffff
TWOEROM:    .byte   $6F,$D8,$65,$D7,$F8,$DC,$94,$D9,$B1,$DB,$30,$F3,$D8,$DF,$E1,$DB
            .byte   $8F,$F3,$98,$F3,$E4,$F1,$DD,$F1,$D4,$F1,$24,$F2,$31,$F2,$40,$F2
            .byte   $D7,$F3,$E1,$F3,$E8,$F6,$FD,$F6,$68,$F7,$6E,$F7,$E6,$F7,$57,$FC
            .byte   $20,$F7,$26,$F7,$74,$F7,$6C,$F2,$6E,$F2,$72,$F2,$76,$F2,$7F,$F2
            .byte   $4E,$F2,$6A,$D9,$55,$F2,$85,$F2,$A5,$F2,$CA,$F2,$17,$F3,$BB,$F3
            .byte   $9E,$F3,$61,$F2,$45,$DA,$3D,$D9,$11,$D9,$C8,$D9,$48,$D8,$F4,$03
            .byte   $20,$D9,$6A,$D9,$DB,$D9,$6D,$D8,$EB,$D9,$83,$E7,$C8,$D8,$AF,$D8
            .byte   $12,$E3,$7A,$E7,$D4,$DA,$95,$D8,$A4,$D6,$69,$D6,$9F,$DB,$48,$D6
            .byte   $90,$EB,$23,$EC,$AF,$EB,$0A,$00,$DE,$E2,$12,$D4,$CD,$DF,$FF,$E2
            .byte   $8D,$EE,$AE,$EF,$41,$E9,$09,$EF,$EA,$EF,$F1,$EF,$3A,$F0,$9E,$F0
            .byte   $64,$E7,$D6,$E6,$C5,$E3,$07,$E7,$E5,$E6,$46,$E6,$5A,$E6,$86,$E6
            .byte   $91,$E6,$79,$C0,$E7,$79,$A9,$E7,$7B,$81,$E9,$7B,$68,$EA,$7D,$96
            .byte   $EE,$50,$54,$DF,$46,$4E,$DF,$7F,$CF,$EE,$7F,$97,$DE,$64,$64,$DF
            .byte   $45,$4E,$C4,$46,$4F,$D2,$4E,$45,$58,$D4,$44,$41,$54,$C1,$49,$4E
            .byte   $50,$55,$D4,$44,$45,$CC,$44,$49,$CD,$52,$45,$41,$C4,$47,$D2,$54
            .byte   $45,$58,$D4,$50,$52,$A3,$49,$4E,$A3,$43,$41,$4C,$CC,$50,$4C,$4F
            .byte   $D4,$48,$4C,$49,$CE,$56,$4C,$49,$CE,$48,$47,$52,$B2,$48,$47,$D2
            .byte   $48,$43,$4F,$4C,$4F,$52,$BD,$48,$50,$4C,$4F,$D4,$44,$52,$41,$D7
            .byte   $58,$44,$52,$41,$D7,$48,$54,$41,$C2,$48,$4F,$4D,$C5,$52,$4F,$54
            .byte   $BD,$53,$43,$41,$4C,$45,$BD,$53,$48,$4C,$4F,$41,$C4,$54,$52,$41
            .byte   $43,$C5,$4E,$4F,$54,$52,$41,$43,$C5,$4E,$4F,$52,$4D,$41,$CC,$49
            .byte   $4E,$56,$45,$52,$53,$C5,$46,$4C,$41,$53,$C8,$43,$4F,$4C,$4F,$52
            .byte   $BD,$50,$4F,$D0,$56,$54,$41,$C2,$48,$49,$4D,$45,$4D,$BA,$4C,$4F
            .byte   $4D,$45,$4D,$BA,$4F,$4E,$45,$52,$D2,$52,$45,$53,$55,$4D,$C5,$52
            .byte   $45,$43,$41,$4C,$CC,$53,$54,$4F,$52,$C5,$53,$50,$45,$45,$44,$BD
            .byte   $4C,$45,$D4,$47,$4F,$54,$CF,$52,$55,$CE,$49,$C6,$52,$45,$53,$54
            .byte   $4F,$52,$C5,$A6,$47,$4F,$53,$55,$C2,$52,$45,$54,$55,$52,$CE,$52
            .byte   $45,$CD,$53,$54,$4F,$D0,$4F,$CE,$57,$41,$49,$D4,$4C,$4F,$41,$C4
            .byte   $53,$41,$56,$C5,$44,$45,$C6,$50,$4F,$4B,$C5,$50,$52,$49,$4E,$D4
            .byte   $43,$4F,$4E,$D4,$4C,$49,$53,$D4,$43,$4C,$45,$41,$D2,$47,$45,$D4
            .byte   $4E,$45,$D7,$54,$41,$42,$A8,$54,$CF,$46,$CE,$53,$50,$43,$A8,$54
            .byte   $48,$45,$CE,$41,$D4,$4E,$4F,$D4,$53,$54,$45,$D0,$AB,$AD,$AA,$AF
            .byte   $DE,$41,$4E,$C4,$4F,$D2,$BE,$BD,$BC,$53,$47,$CE,$49,$4E,$D4,$41
            .byte   $42,$D3,$55,$53,$D2,$46,$52,$C5,$53,$43,$52,$4E,$A8,$50,$44,$CC
            .byte   $50,$4F,$D3,$53,$51,$D2,$52,$4E,$C4,$4C,$4F,$C7,$45,$58,$D0,$43
            .byte   $4F,$D3,$53,$49,$CE,$54,$41,$CE,$41,$54,$CE,$50,$45,$45,$CB,$4C
            .byte   $45,$CE,$53,$54,$52,$A4,$56,$41,$CC,$41,$53,$C3,$43,$48,$52,$A4
            .byte   $4C,$45,$46,$54,$A4,$52,$49,$47,$48,$54,$A4,$4D,$49,$44,$A4,$00
            .byte   $4E,$45,$58,$54,$20,$57,$49,$54,$48,$4F,$55,$54,$20,$46,$4F,$D2
            .byte   $53,$59,$4E,$54,$41,$D8,$52,$45,$54,$55,$52,$4E,$20,$57,$49,$54
            .byte   $48,$4F,$55,$54,$20,$47,$4F,$53,$55,$C2,$4F,$55,$54,$20,$4F,$46
            .byte   $20,$44,$41,$54,$C1,$49,$4C,$4C,$45,$47,$41,$4C,$20,$51,$55,$41
            .byte   $4E,$54,$49,$54,$D9,$4F,$56,$45,$52,$46,$4C,$4F,$D7,$4F,$55,$54
            .byte   $20,$4F,$46,$20,$4D,$45,$4D,$4F,$52,$D9,$55,$4E,$44,$45,$46,$27
            .byte   $44,$20,$53,$54,$41,$54,$45,$4D,$45,$4E,$D4,$42,$41,$44,$20,$53
            .byte   $55,$42,$53,$43,$52,$49,$50,$D4,$52,$45,$44,$49,$4D,$27,$44,$20
            .byte   $41,$52,$52,$41,$D9,$44,$49,$56,$49,$53,$49,$4F,$4E,$20,$42,$59
            .byte   $20,$5A,$45,$52,$CF,$49,$4C,$4C,$45,$47,$41,$4C,$20,$44,$49,$52
            .byte   $45,$43,$D4,$54,$59,$50,$45,$20,$4D,$49,$53,$4D,$41,$54,$43,$C8
            .byte   $53,$54,$52,$49,$4E,$47,$20,$54,$4F,$4F,$20,$4C,$4F,$4E,$C7,$46
            .byte   $4F,$52,$4D,$55,$4C,$41,$20,$54,$4F,$4F,$20,$43,$4F,$4D,$50,$4C
            .byte   $45,$D8,$43,$41,$4E,$27,$54,$20,$43,$4F,$4E,$54,$49,$4E,$55,$C5
            .byte   $55,$4E,$44,$45,$46,$27,$44,$20,$46,$55,$4E,$43,$54,$49,$4F,$CE
            .byte   $20,$45,$52,$52,$4F,$52,$07,$00,$20,$49,$4E,$20,$00,$0D,$42,$52
            .byte   $45,$41,$4B,$07,$00,$BA,$E8,$E8,$E8,$E8,$BD,$01,$01,$C9,$81,$D0
            .byte   $21,$A5,$86,$D0,$0A,$BD,$02,$01,$85,$85,$BD,$03,$01,$85,$86,$DD
            .byte   $03,$01,$D0,$07,$A5,$85,$DD,$02,$01,$F0,$07,$8A,$18,$69,$12,$AA
            .byte   $D0,$D8,$60,$20,$E3,$D3,$85,$6D,$84,$6E,$38,$A5,$96,$E5,$9B,$85
            .byte   $5E,$A8,$A5,$97,$E5,$9C,$AA,$E8,$98,$F0,$23,$A5,$96,$38,$E5,$5E
            .byte   $85,$96,$B0,$03,$C6,$97,$38,$A5,$94,$E5,$5E,$85,$94,$B0,$08,$C6
            .byte   $95,$90,$04,$B1,$96,$91,$94,$88,$D0,$F9,$B1,$96,$91,$94,$C6,$97
            .byte   $C6,$95,$CA,$D0,$F2,$60,$0A,$69,$36,$B0,$35,$85,$5E,$BA,$E4,$5E
            .byte   $90,$2E,$60,$C4,$70,$90,$28,$D0,$04,$C5,$6F,$90,$22,$48,$A2,$09
            .byte   $98,$48,$B5,$93,$CA,$10,$FA,$20,$84,$E4,$A2,$F7,$68,$95,$9D,$E8
            .byte   $30,$FA,$68,$A8,$68,$C4,$70,$90,$06,$D0,$05,$C5,$6F,$B0,$01,$60
            .byte   $A2,$4D,$24,$D8,$10,$03,$4C,$E9,$F2,$20,$FB,$DA,$20,$5A,$DB,$BD
            .byte   $60,$D2,$48,$20,$5C,$DB,$E8,$68,$10,$F5,$20,$83,$D6,$A9,$50,$A0
            .byte   $D3,$20,$3A,$DB,$A4,$76,$C8,$F0,$03,$20,$19,$ED,$20,$FB,$DA,$A2
            .byte   $DD,$20,$2E,$D5,$86,$B8,$84,$B9,$46,$D8,$20,$B1,$00,$AA,$F0,$EC
            .byte   $A2,$FF,$86,$76,$90,$06,$20,$59,$D5,$4C,$05,$D8,$A6,$AF,$86,$69
            .byte   $A6,$B0,$86,$6A,$20,$0C,$DA,$20,$59,$D5,$84,$0F,$20,$1A,$D6,$90
            .byte   $44,$A0,$01,$B1,$9B,$85,$5F,$A5,$69,$85,$5E,$A5,$9C,$85,$61,$A5
            .byte   $9B,$88,$F1,$9B,$18,$65,$69,$85,$69,$85,$60,$A5,$6A,$69,$FF,$85
            .byte   $6A,$E5,$9C,$AA,$38,$A5,$9B,$E5,$69,$A8,$B0,$03,$E8,$C6,$61,$18
            .byte   $65,$5E,$90,$03,$C6,$5F,$18,$B1,$5E,$91,$60,$C8,$D0,$F9,$E6,$5F
            .byte   $E6,$61,$CA,$D0,$F2,$AD,$00,$02,$F0,$38,$A5,$73,$A4,$74,$85,$6F
            .byte   $84,$70,$A5,$69,$85,$96,$65,$0F,$85,$94,$A4,$6A,$84,$97,$90,$01
            .byte   $C8,$84,$95,$20,$93,$D3,$A5,$50,$A4,$51,$8D,$FE,$01,$8C,$FF,$01
            .byte   $A5,$6D,$A4,$6E,$85,$69,$84,$6A,$A4,$0F,$B9,$FB,$01,$88,$91,$9B
            .byte   $D0,$F8,$20,$65,$D6,$A5,$67,$A4,$68,$85,$5E,$84,$5F,$18,$A0,$01
            .byte   $B1,$5E,$D0,$0B,$A5,$69,$85,$AF,$A5,$6A,$85,$B0,$4C,$3C,$D4,$A0
            .byte   $04,$C8,$B1,$5E,$D0,$FB,$C8,$98,$65,$5E,$AA,$A0,$00,$91,$5E,$A5
            .byte   $5F,$69,$00,$C8,$91,$5E,$86,$5E,$85,$5F,$90,$D2,$A2,$80,$86,$33
            .byte   $20,$6A,$FD,$E0,$EF,$90,$02,$A2,$EF,$A9,$00,$9D,$00,$02,$8A,$F0
            .byte   $0B,$BD,$FF,$01,$29,$7F,$9D,$FF,$01,$CA,$D0,$F5,$A9,$00,$A2,$FF
            .byte   $A0,$01,$60,$20,$0C,$FD,$29,$7F,$60,$A6,$B8,$CA,$A0,$04,$84,$13
            .byte   $24,$D6,$10,$08,$68,$68,$20,$65,$D6,$4C,$D2,$D7,$E8,$20,$8C,$F7
            .byte   $24,$13,$70,$04,$C9,$20,$F0,$F4,$85,$0E,$C9,$22,$F0,$74,$70,$4D
            .byte   $C9,$3F,$D0,$04,$A9,$BA,$D0,$45,$C9,$30,$90,$04,$C9,$3C,$90,$3D
            .byte   $84,$AD,$A9,$D0,$85,$9D,$A9,$CF,$85,$9E,$A0,$00,$84,$0F,$88,$86
            .byte   $B8,$CA,$C8,$D0,$02,$E6,$9E,$E8,$20,$8C,$F7,$C9,$20,$F0,$F8,$38
            .byte   $F1,$9D,$F0,$EE,$C9,$80,$D0,$41,$05,$0F,$C9,$C5,$D0,$0D,$20,$87
            .byte   $F7,$C9,$4E,$F0,$34,$C9,$4F,$F0,$30,$A9,$C5,$A4,$AD,$E8,$C8,$99
            .byte   $FB,$01,$B9,$FB,$01,$F0,$39,$38,$E9,$3A,$F0,$04,$C9,$49,$D0,$02
            .byte   $85,$13,$38,$E9,$78,$D0,$86,$85,$0E,$20,$8C,$F7,$F0,$DF,$C5,$0E
            .byte   $F0,$DB,$C8,$99,$FB,$01,$E8,$D0,$F0,$A6,$B8,$E6,$0F,$B1,$9D,$C8
            .byte   $D0,$02,$E6,$9E,$0A,$90,$F6,$B1,$9D,$D0,$9D,$20,$9A,$F7,$10,$BB
            .byte   $99,$FD,$01,$C6,$B9,$A9,$FF,$85,$B8,$60,$A5,$67,$A6,$68,$A0,$01
            .byte   $85,$9B,$86,$9C,$B1,$9B,$F0,$1F,$C8,$C8,$A5,$51,$D1,$9B,$90,$18
            .byte   $F0,$03,$88,$D0,$09,$A5,$50,$88,$D1,$9B,$90,$0C,$F0,$0A,$88,$B1
            .byte   $9B,$AA,$88,$B1,$9B,$B0,$D7,$18,$60,$D0,$FD,$A9,$00,$85,$D6,$A8
            .byte   $91,$67,$C8,$91,$67,$A5,$67,$69,$02,$85,$69,$85,$AF,$A5,$68,$69
            .byte   $00,$85,$6A,$85,$B0,$20,$97,$D6,$A9,$00,$D0,$2A,$A5,$73,$A4,$74
            .byte   $85,$6F,$84,$70,$A5,$69,$A4,$6A,$85,$6B,$84,$6C,$85,$6D,$84,$6E
            .byte   $20,$49,$D8,$A2,$55,$86,$52,$68,$A8,$68,$A2,$F8,$9A,$48,$98,$48
            .byte   $A9,$00,$85,$7A,$85,$14,$60,$18,$A5,$67,$69,$FF,$85,$B8,$A5,$68
            .byte   $69,$FF,$85,$B9,$60,$90,$0A,$F0,$08,$C9,$C9,$F0,$04,$C9,$2C,$D0
            .byte   $E5,$20,$0C,$DA,$20,$1A,$D6,$20,$B7,$00,$F0,$10,$C9,$C9,$F0,$04
            .byte   $C9,$2C,$D0,$84,$20,$B1,$00,$20,$0C,$DA,$D0,$CA,$68,$68,$A5,$50
            .byte   $05,$51,$D0,$06,$A9,$FF,$85,$50,$85,$51,$A0,$01,$B1,$9B,$F0,$44
            .byte   $20,$58,$D8,$20,$FB,$DA,$C8,$B1,$9B,$AA,$C8,$B1,$9B,$C5,$51,$D0
            .byte   $04,$E4,$50,$F0,$02,$B0,$2D,$84,$85,$20,$AA,$F7,$A9,$20,$A4,$85
            .byte   $29,$7F,$20,$5C,$DB,$20,$B4,$F7,$EA,$90,$07,$20,$FB,$DA,$A9,$05
            .byte   $85,$24,$C8,$B1,$9B,$D0,$1D,$A8,$B1,$9B,$AA,$C8,$B1,$9B,$86,$9B
            .byte   $85,$9C,$D0,$B6,$A9,$0D,$20,$5C,$DB,$4C,$D2,$D7,$C8,$D0,$02,$E6
            .byte   $9E,$B1,$9D,$60,$10,$CC,$38,$E9,$7F,$AA,$84,$85,$A0,$D0,$84,$9D
            .byte   $A0,$CF,$84,$9E,$A0,$FF,$CA,$F0,$07,$20,$2C,$D7,$10,$FB,$30,$F6
            .byte   $A9,$20,$20,$5C,$DB,$20,$2C,$D7,$30,$05,$20,$5C,$DB,$D0,$F6,$20
            .byte   $5C,$DB,$A9,$20,$D0,$98,$A9,$80,$85,$14,$20,$46,$DA,$20,$65,$D3
            .byte   $D0,$05,$8A,$69,$0F,$AA,$9A,$68,$68,$A9,$09,$20,$D6,$D3,$20,$A3
            .byte   $D9,$18,$98,$65,$B8,$48,$A5,$B9,$69,$00,$48,$A5,$76,$48,$A5,$75
            .byte   $48,$A9,$C1,$20,$C0,$DE,$20,$6A,$DD,$20,$67,$DD,$A5,$A2,$09,$7F
            .byte   $25,$9E,$85,$9E,$A9,$AF,$A0,$D7,$85,$5E,$84,$5F,$4C,$20,$DE,$A9
            .byte   $13,$A0,$E9,$20,$F9,$EA,$20,$B7,$00,$C9,$C7,$D0,$06,$20,$B1,$00
            .byte   $20,$67,$DD,$20,$82,$EB,$20,$15,$DE,$A5,$86,$48,$A5,$85,$48,$A9
            .byte   $81,$48,$BA,$86,$F8,$20,$58,$D8,$A5,$B8,$A4,$B9,$A6,$76,$E8,$F0
            .byte   $04,$85,$79,$84,$7A,$A0,$00,$B1,$B8,$D0,$57,$A0,$02,$B1,$B8,$18
            .byte   $F0,$34,$C8,$B1,$B8,$85,$75,$C8,$B1,$B8,$85,$76,$98,$65,$B8,$85
            .byte   $B8,$90,$02,$E6,$B9,$24,$F2,$10,$14,$A6,$76,$E8,$F0,$0F,$A9,$23
            .byte   $20,$5C,$DB,$A6,$75,$A5,$76,$20,$24,$ED,$20,$57,$DB,$20,$B1,$00
            .byte   $20,$28,$D8,$4C,$D2,$D7,$F0,$62,$F0,$2D,$E9,$80,$90,$11,$C9,$40
            .byte   $B0,$14,$0A,$A8,$B9,$01,$D0,$48,$B9,$00,$D0,$48,$4C,$B1,$00,$4C
            .byte   $46,$DA,$C9,$3A,$F0,$BF,$4C,$C9,$DE,$38,$A5,$67,$E9,$01,$A4,$68
            .byte   $B0,$01,$88,$85,$7D,$84,$7E,$60,$AD,$00,$C0,$C9,$83,$F0,$01,$60
            .byte   $20,$53,$D5,$A2,$FF,$24,$D8,$10,$03,$4C,$E9,$F2,$C9,$03,$B0,$01
            .byte   $18,$D0,$3C,$A5,$B8,$A4,$B9,$A6,$76,$E8,$F0,$0C,$85,$79,$84,$7A
            .byte   $A5,$75,$A4,$76,$85,$77,$84,$78,$68,$68,$A9,$5D,$A0,$D3,$90,$03
            .byte   $4C,$31,$D4,$4C,$3C,$D4,$D0,$17,$A2,$D2,$A4,$7A,$D0,$03,$4C,$12
            .byte   $D4,$A5,$79,$85,$B8,$84,$B9,$A5,$77,$A4,$78,$85,$75,$84,$76,$60
            .byte   $38,$A5,$AF,$E5,$67,$85,$50,$A5,$B0,$E5,$68,$85,$51,$20,$F0,$D8
            .byte   $20,$CD,$FE,$20,$01,$D9,$4C,$CD,$FE,$20,$F0,$D8,$20,$FD,$FE,$18
            .byte   $A5,$67,$65,$50,$85,$69,$A5,$68,$65,$51,$85,$6A,$A5,$52,$85,$D6
            .byte   $20,$01,$D9,$20,$FD,$FE,$24,$D6,$10,$03,$4C,$65,$D6,$4C,$F2,$D4
            .byte   $A9,$50,$A0,$00,$85,$3C,$84,$3D,$A9,$52,$85,$3E,$84,$3F,$84,$D6
            .byte   $60,$A5,$67,$A4,$68,$85,$3C,$84,$3D,$A5,$69,$A4,$6A,$85,$3E,$84
            .byte   $3F,$60,$08,$C6,$76,$28,$D0,$03,$4C,$65,$D6,$20,$6C,$D6,$4C,$35
            .byte   $D9,$A9,$03,$20,$D6,$D3,$A5,$B9,$48,$A5,$B8,$48,$A5,$76,$48,$A5
            .byte   $75,$48,$A9,$B0,$48,$20,$B7,$00,$20,$3E,$D9,$4C,$D2,$D7,$20,$0C
            .byte   $DA,$20,$A6,$D9,$A5,$76,$C5,$51,$B0,$0B,$98,$38,$65,$B8,$A6,$B9
            .byte   $90,$07,$E8,$B0,$04,$A5,$67,$A6,$68,$20,$1E,$D6,$90,$1E,$A5,$9B
            .byte   $E9,$01,$85,$B8,$A5,$9C,$E9,$00,$85,$B9,$60,$D0,$FD,$A9,$FF,$85
            .byte   $85,$20,$65,$D3,$9A,$C9,$B0,$F0,$0B,$A2,$16,$2C,$A2,$5A,$4C,$12
            .byte   $D4,$4C,$C9,$DE,$68,$68,$C0,$42,$F0,$3B,$85,$75,$68,$85,$76,$68
            .byte   $85,$B8,$68,$85,$B9,$20,$A3,$D9,$98,$18,$65,$B8,$85,$B8,$90,$02
            .byte   $E6,$B9,$60,$A2,$3A,$2C,$A2,$00,$86,$0D,$A0,$00,$84,$0E,$A5,$0E
            .byte   $A6,$0D,$85,$0D,$86,$0E,$B1,$B8,$F0,$E8,$C5,$0E,$F0,$E4,$C8,$C9
            .byte   $22,$D0,$F3,$F0,$E9,$68,$68,$68,$60,$20,$7B,$DD,$20,$B7,$00,$C9
            .byte   $AB,$F0,$05,$A9,$C4,$20,$C0,$DE,$A5,$9D,$D0,$05,$20,$A6,$D9,$F0
            .byte   $B7,$20,$B7,$00,$B0,$03,$4C,$3E,$D9,$4C,$28,$D8,$20,$F8,$E6,$48
            .byte   $C9,$B0,$F0,$04,$C9,$AB,$D0,$89,$C6,$A1,$D0,$04,$68,$4C,$2A,$D8
            .byte   $20,$B1,$00,$20,$0C,$DA,$C9,$2C,$F0,$EE,$68,$60,$A2,$00,$86,$50
            .byte   $86,$51,$B0,$F7,$E9,$2F,$85,$0D,$A5,$51,$85,$5E,$C9,$19,$B0,$D4
            .byte   $A5,$50,$0A,$26,$5E,$0A,$26,$5E,$65,$50,$85,$50,$A5,$5E,$65,$51
            .byte   $85,$51,$06,$50,$26,$51,$A5,$50,$65,$0D,$85,$50,$90,$02,$E6,$51
            .byte   $20,$B1,$00,$4C,$12,$DA,$20,$E3,$DF,$85,$85,$84,$86,$A9,$D0,$20
            .byte   $C0,$DE,$A5,$12,$48,$A5,$11,$48,$20,$7B,$DD,$68,$2A,$20,$6D,$DD
            .byte   $D0,$18,$68,$10,$12,$20,$72,$EB,$20,$0C,$E1,$A0,$00,$A5,$A0,$91
            .byte   $85,$C8,$A5,$A1,$91,$85,$60,$4C,$27,$EB,$68,$A0,$02,$B1,$A0,$C5
            .byte   $70,$90,$17,$D0,$07,$88,$B1,$A0,$C5,$6F,$90,$0E,$A4,$A1,$C4,$6A
            .byte   $90,$08,$D0,$0D,$A5,$A0,$C5,$69,$B0,$07,$A5,$A0,$A4,$A1,$4C,$B7
            .byte   $DA,$A0,$00,$B1,$A0,$20,$D5,$E3,$A5,$8C,$A4,$8D,$85,$AB,$84,$AC
            .byte   $20,$D4,$E5,$A9,$9D,$A0,$00,$85,$8C,$84,$8D,$20,$35,$E6,$A0,$00
            .byte   $B1,$8C,$91,$85,$C8,$B1,$8C,$91,$85,$C8,$B1,$8C,$91,$85,$60,$20
            .byte   $3D,$DB,$20,$B7,$00,$F0,$24,$F0,$29,$C9,$C0,$F0,$3C,$C9,$C3,$18
            .byte   $F0,$37,$C9,$2C,$18,$F0,$1C,$C9,$3B,$F0,$44,$20,$7B,$DD,$24,$11
            .byte   $30,$DD,$20,$34,$ED,$20,$E7,$E3,$4C,$CF,$DA,$A9,$0D,$20,$5C,$DB
            .byte   $49,$FF,$60,$20,$B4,$F7,$30,$09,$C9,$18,$90,$05,$20,$FB,$DA,$D0
            .byte   $1E,$69,$10,$29,$F0,$AA,$38,$B0,$0C,$08,$20,$F5,$E6,$C9,$29,$D0
            .byte   $62,$28,$90,$07,$CA,$20,$C3,$F7,$90,$05,$AA,$E8,$CA,$D0,$06,$20
            .byte   $B1,$00,$4C,$D7,$DA,$20,$57,$DB,$D0,$F2,$20,$E7,$E3,$20,$00,$E6
            .byte   $AA,$A0,$00,$E8,$CA,$F0,$BB,$B1,$5E,$20,$5C,$DB,$C8,$C9,$0D,$D0
            .byte   $F3,$20,$00,$DB,$4C,$44,$DB,$A9,$20,$2C,$A9,$3F,$09,$80,$C9,$A0
            .byte   $90,$02,$05,$F3,$20,$ED,$FD,$29,$7F,$48,$A5,$F1,$20,$A8,$FC,$68
            .byte   $60,$A5,$15,$F0,$12,$30,$04,$A0,$FF,$D0,$04,$A5,$7B,$A4,$7C,$85
            .byte   $75,$84,$76,$4C,$C9,$DE,$68,$24,$D8,$10,$05,$A2,$FE,$4C,$E9,$F2
            .byte   $A9,$EF,$A0,$DC,$20,$3A,$DB,$A5,$79,$A4,$7A,$85,$B8,$84,$B9,$60
            .byte   $20,$06,$E3,$A2,$01,$A0,$02,$A9,$00,$8D,$01,$02,$A9,$40,$20,$EB
            .byte   $DB,$60,$C9,$22,$D0,$0E,$20,$81,$DE,$A9,$3B,$20,$C0,$DE,$20,$3D
            .byte   $DB,$4C,$C7,$DB,$20,$5A,$DB,$20,$06,$E3,$A9,$2C,$8D,$FF,$01,$20
            .byte   $2C,$D5,$AD,$00,$02,$C9,$03,$D0,$10,$4C,$63,$D8,$20,$5A,$DB,$4C
            .byte   $2C,$D5,$A6,$7D,$A4,$7E,$A9,$98,$2C,$A9,$00,$85,$15,$86,$7F,$84
            .byte   $80,$20,$E3,$DF,$85,$85,$84,$86,$A5,$B8,$A4,$B9,$85,$87,$84,$88
            .byte   $A6,$7F,$A4,$80,$86,$B8,$84,$B9,$20,$B7,$00,$D0,$1E,$24,$15,$50
            .byte   $0E,$20,$0C,$FD,$29,$7F,$8D,$00,$02,$A2,$FF,$A0,$01,$D0,$08,$30
            .byte   $7F,$20,$5A,$DB,$20,$DC,$DB,$86,$B8,$84,$B9,$20,$B1,$00,$24,$11
            .byte   $10,$31,$24,$15,$50,$09,$E8,$86,$B8,$A9,$00,$85,$0D,$F0,$0C,$85
            .byte   $0D,$C9,$22,$F0,$07,$A9,$3A,$85,$0D,$A9,$2C,$18,$85,$0E,$A5,$B8
            .byte   $A4,$B9,$69,$00,$90,$01,$C8,$20,$ED,$E3,$20,$3D,$E7,$20,$7B,$DA
            .byte   $4C,$72,$DC,$48,$AD,$00,$02,$F0,$30,$68,$20,$4A,$EC,$A5,$12,$20
            .byte   $63,$DA,$20,$B7,$00,$F0,$07,$C9,$2C,$F0,$03,$4C,$71,$DB,$A5,$B8
            .byte   $A4,$B9,$85,$7F,$84,$80,$A5,$87,$A4,$88,$85,$B8,$84,$B9,$20,$B7
            .byte   $00,$F0,$33,$20,$BE,$DE,$4C,$F1,$DB,$A5,$15,$D0,$CC,$4C,$86,$DB
            .byte   $20,$A3,$D9,$C8,$AA,$D0,$12,$A2,$2A,$C8,$B1,$B8,$F0,$5F,$C8,$B1
            .byte   $B8,$85,$7B,$C8,$B1,$B8,$C8,$85,$7C,$B1,$B8,$AA,$20,$98,$D9,$E0
            .byte   $83,$D0,$DD,$4C,$2B,$DC,$A5,$7F,$A4,$80,$A6,$15,$10,$03,$4C,$53
            .byte   $D8,$A0,$00,$B1,$7F,$F0,$07,$A9,$DF,$A0,$DC,$4C,$3A,$DB,$60,$3F
            .byte   $45,$58,$54,$52,$41,$20,$49,$47,$4E,$4F,$52,$45,$44,$0D,$00,$3F
            .byte   $52,$45,$45,$4E,$54,$45,$52,$0D,$00,$D0,$04,$A0,$00,$F0,$03,$20
            .byte   $E3,$DF,$85,$85,$84,$86,$20,$65,$D3,$F0,$04,$A2,$00,$F0,$69,$9A
            .byte   $E8,$E8,$E8,$E8,$8A,$E8,$E8,$E8,$E8,$E8,$E8,$86,$60,$A0,$01,$20
            .byte   $F9,$EA,$BA,$BD,$09,$01,$85,$A2,$A5,$85,$A4,$86,$20,$BE,$E7,$20
            .byte   $27,$EB,$A0,$01,$20,$B4,$EB,$BA,$38,$FD,$09,$01,$F0,$17,$BD,$0F
            .byte   $01,$85,$75,$BD,$10,$01,$85,$76,$BD,$12,$01,$85,$B8,$BD,$11,$01
            .byte   $85,$B9,$4C,$D2,$D7,$8A,$69,$11,$AA,$9A,$20,$B7,$00,$C9,$2C,$D0
            .byte   $F1,$20,$B1,$00,$20,$FF,$DC,$20,$7B,$DD,$18,$24,$38,$24,$11,$30
            .byte   $03,$B0,$03,$60,$B0,$FD,$A2,$A3,$4C,$12,$D4,$A6,$B8,$D0,$02,$C6
            .byte   $B9,$C6,$B8,$A2,$00,$24,$48,$8A,$48,$A9,$01,$20,$D6,$D3,$20,$60
            .byte   $DE,$A9,$00,$85,$89,$20,$B7,$00,$38,$E9,$CF,$90,$17,$C9,$03,$B0
            .byte   $13,$C9,$01,$2A,$49,$01,$45,$89,$C5,$89,$90,$61,$85,$89,$20,$B1
            .byte   $00,$4C,$98,$DD,$A6,$89,$D0,$2C,$B0,$7B,$69,$07,$90,$77,$65,$11
            .byte   $D0,$03,$4C,$97,$E5,$69,$FF,$85,$5E,$0A,$65,$5E,$A8,$68,$D9,$B2
            .byte   $D0,$B0,$67,$20,$6A,$DD,$48,$20,$FD,$DD,$68,$A4,$87,$10,$17,$AA
            .byte   $F0,$56,$D0,$5F,$46,$11,$8A,$2A,$A6,$B8,$D0,$02,$C6,$B9,$C6,$B8
            .byte   $A0,$1B,$85,$89,$D0,$D7,$D9,$B2,$D0,$B0,$48,$90,$D9,$B9,$B4,$D0
            .byte   $48,$B9,$B3,$D0,$48,$20,$10,$DE,$A5,$89,$4C,$86,$DD,$4C,$C9,$DE
            .byte   $A5,$A2,$BE,$B2,$D0,$A8,$68,$85,$5E,$E6,$5E,$68,$85,$5F,$98,$48
            .byte   $20,$72,$EB,$A5,$A1,$48,$A5,$A0,$48,$A5,$9F,$48,$A5,$9E,$48,$A5
            .byte   $9D,$48,$6C,$5E,$00,$A0,$FF,$68,$F0,$23,$C9,$64,$F0,$03,$20,$6A
            .byte   $DD,$84,$87,$68,$4A,$85,$16,$68,$85,$A5,$68,$85,$A6,$68,$85,$A7
            .byte   $68,$85,$A8,$68,$85,$A9,$68,$85,$AA,$45,$A2,$85,$AB,$A5,$9D,$60
            .byte   $A9,$00,$85,$11,$20,$B1,$00,$B0,$03,$4C,$4A,$EC,$20,$7D,$E0,$B0
            .byte   $64,$C9,$2E,$F0,$F4,$C9,$C9,$F0,$55,$C9,$C8,$F0,$E7,$C9,$22,$D0
            .byte   $0F,$A5,$B8,$A4,$B9,$69,$00,$90,$01,$C8,$20,$E7,$E3,$4C,$3D,$E7
            .byte   $C9,$C6,$D0,$10,$A0,$18,$D0,$38,$A5,$9D,$D0,$03,$A0,$01,$2C,$A0
            .byte   $00,$4C,$01,$E3,$C9,$C2,$D0,$03,$4C,$54,$E3,$C9,$D2,$90,$03,$4C
            .byte   $0C,$DF,$20,$BB,$DE,$20,$7B,$DD,$A9,$29,$2C,$A9,$28,$2C,$A9,$2C
            .byte   $A0,$00,$D1,$B8,$D0,$03,$4C,$B1,$00,$A2,$10,$4C,$12,$D4,$A0,$15
            .byte   $68,$68,$4C,$D7,$DD,$20,$E3,$DF,$85,$A0,$84,$A1,$A6,$11,$F0,$05
            .byte   $A2,$00,$86,$AC,$60,$A6,$12,$10,$0D,$A0,$00,$B1,$A0,$AA,$C8,$B1
            .byte   $A0,$A8,$8A,$4C,$F2,$E2,$4C,$F9,$EA,$20,$B1,$00,$20,$EC,$F1,$8A
            .byte   $A4,$F0,$20,$71,$F8,$A8,$20,$01,$E3,$4C,$B8,$DE,$C9,$D7,$F0,$E9
            .byte   $0A,$48,$AA,$20,$B1,$00,$E0,$CF,$90,$20,$20,$BB,$DE,$20,$7B,$DD
            .byte   $20,$BE,$DE,$20,$6C,$DD,$68,$AA,$A5,$A1,$48,$A5,$A0,$48,$8A,$48
            .byte   $20,$F8,$E6,$68,$A8,$8A,$48,$4C,$3F,$DF,$20,$B2,$DE,$68,$A8,$B9
            .byte   $DC,$CF,$85,$91,$B9,$DD,$CF,$85,$92,$20,$90,$00,$4C,$6A,$DD,$A5
            .byte   $A5,$05,$9D,$D0,$0B,$A5,$A5,$F0,$04,$A5,$9D,$D0,$03,$A0,$00,$2C
            .byte   $A0,$01,$4C,$01,$E3,$20,$6D,$DD,$B0,$13,$A5,$AA,$09,$7F,$25,$A6
            .byte   $85,$A6,$A9,$A5,$A0,$00,$20,$B2,$EB,$AA,$4C,$B0,$DF,$A9,$00,$85
            .byte   $11,$C6,$89,$20,$00,$E6,$85,$9D,$86,$9E,$84,$9F,$A5,$A8,$A4,$A9
            .byte   $20,$04,$E6,$86,$A8,$84,$A9,$AA,$38,$E5,$9D,$F0,$08,$A9,$01,$90
            .byte   $04,$A6,$9D,$A9,$FF,$85,$A2,$A0,$FF,$E8,$C8,$CA,$D0,$07,$A6,$A2
            .byte   $30,$0F,$18,$90,$0C,$B1,$A8,$D1,$9E,$F0,$EF,$A2,$FF,$B0,$02,$A2
            .byte   $01,$E8,$8A,$2A,$25,$16,$F0,$02,$A9,$01,$4C,$93,$EB,$20,$FB,$E6
            .byte   $20,$1E,$FB,$4C,$01,$E3,$20,$BE,$DE,$AA,$20,$E8,$DF,$20,$B7,$00
            .byte   $D0,$F4,$60,$A2,$00,$20,$B7,$00,$86,$10,$85,$81,$20,$B7,$00,$20
            .byte   $7D,$E0,$B0,$03,$4C,$C9,$DE,$A2,$00,$86,$11,$86,$12,$4C,$07,$E0
            .byte   $4C,$28,$F1,$4C,$3C,$D4,$C4,$20,$B1,$00,$90,$05,$20,$7D,$E0,$90
            .byte   $0B,$AA,$20,$B1,$00,$90,$FB,$20,$7D,$E0,$B0,$F6,$C9,$24,$D0,$06
            .byte   $A9,$FF,$85,$11,$D0,$10,$C9,$25,$D0,$13,$A5,$14,$30,$C6,$A9,$80
            .byte   $85,$12,$05,$81,$85,$81,$8A,$09,$80,$AA,$20,$B1,$00,$86,$82,$38
            .byte   $05,$14,$E9,$28,$D0,$03,$4C,$1E,$E1,$24,$14,$30,$02,$70,$F7,$A9
            .byte   $00,$85,$14,$A5,$69,$A6,$6A,$A0,$00,$86,$9C,$85,$9B,$E4,$6C,$D0
            .byte   $04,$C5,$6B,$F0,$22,$A5,$81,$D1,$9B,$D0,$08,$A5,$82,$C8,$D1,$9B
            .byte   $F0,$6C,$88,$18,$A5,$9B,$69,$07,$90,$E1,$E8,$D0,$DC,$C9,$41,$90
            .byte   $05,$E9,$5B,$38,$E9,$A5,$60,$68,$48,$C9,$D7,$D0,$0F,$BA,$BD,$02
            .byte   $01,$C9,$DE,$D0,$07,$A9,$9A,$A0,$E0,$60,$00,$00,$A5,$6B,$A4,$6C
            .byte   $85,$9B,$84,$9C,$A5,$6D,$A4,$6E,$85,$96,$84,$97,$18,$69,$07,$90
            .byte   $01,$C8,$85,$94,$84,$95,$20,$93,$D3,$A5,$94,$A4,$95,$C8,$85,$6B
            .byte   $84,$6C,$A0,$00,$A5,$81,$91,$9B,$C8,$A5,$82,$91,$9B,$A9,$00,$C8
            .byte   $91,$9B,$C8,$91,$9B,$C8,$91,$9B,$C8,$91,$9B,$C8,$91,$9B,$A5,$9B
            .byte   $18,$69,$02,$A4,$9C,$90,$01,$C8,$85,$83,$84,$84,$60,$A5,$0F,$0A
            .byte   $69,$05,$65,$9B,$A4,$9C,$90,$01,$C8,$85,$94,$84,$95,$60,$90,$80
            .byte   $00,$00,$20,$B1,$00,$20,$67,$DD,$A5,$A2,$30,$0D,$A5,$9D,$C9,$90
            .byte   $90,$09,$A9,$FE,$A0,$E0,$20,$B2,$EB,$D0,$7E,$4C,$F2,$EB,$A5,$14
            .byte   $D0,$47,$A5,$10,$05,$12,$48,$A5,$11,$48,$A0,$00,$98,$48,$A5,$82
            .byte   $48,$A5,$81,$48,$20,$02,$E1,$68,$85,$81,$68,$85,$82,$68,$A8,$BA
            .byte   $BD,$02,$01,$48,$BD,$01,$01,$48,$A5,$A0,$9D,$02,$01,$A5,$A1,$9D
            .byte   $01,$01,$C8,$20,$B7,$00,$C9,$2C,$F0,$D2,$84,$0F,$20,$B8,$DE,$68
            .byte   $85,$11,$68,$85,$12,$29,$7F,$85,$10,$A6,$6B,$A5,$6C,$86,$9B,$85
            .byte   $9C,$C5,$6E,$D0,$04,$E4,$6D,$F0,$3F,$A0,$00,$B1,$9B,$C8,$C5,$81
            .byte   $D0,$06,$A5,$82,$D1,$9B,$F0,$16,$C8,$B1,$9B,$18,$65,$9B,$AA,$C8
            .byte   $B1,$9B,$65,$9C,$90,$D7,$A2,$6B,$2C,$A2,$35,$4C,$12,$D4,$A2,$78
            .byte   $A5,$10,$D0,$F7,$A5,$14,$F0,$02,$38,$60,$20,$ED,$E0,$A5,$0F,$A0
            .byte   $04,$D1,$9B,$D0,$E1,$4C,$4B,$E2,$A5,$14,$F0,$05,$A2,$2A,$4C,$12
            .byte   $D4,$20,$ED,$E0,$20,$E3,$D3,$A9,$00,$A8,$85,$AE,$A2,$05,$A5,$81
            .byte   $91,$9B,$10,$01,$CA,$C8,$A5,$82,$91,$9B,$10,$02,$CA,$CA,$86,$AD
            .byte   $A5,$0F,$C8,$C8,$C8,$91,$9B,$A2,$0B,$A9,$00,$24,$10,$50,$08,$68
            .byte   $18,$69,$01,$AA,$68,$69,$00,$C8,$91,$9B,$C8,$8A,$91,$9B,$20,$AD
            .byte   $E2,$86,$AD,$85,$AE,$A4,$5E,$C6,$0F,$D0,$DC,$65,$95,$B0,$5D,$85
            .byte   $95,$A8,$8A,$65,$94,$90,$03,$C8,$F0,$52,$20,$E3,$D3,$85,$6D,$84
            .byte   $6E,$A9,$00,$E6,$AE,$A4,$AD,$F0,$05,$88,$91,$94,$D0,$FB,$C6,$95
            .byte   $C6,$AE,$D0,$F5,$E6,$95,$38,$A5,$6D,$E5,$9B,$A0,$02,$91,$9B,$A5
            .byte   $6E,$C8,$E5,$9C,$91,$9B,$A5,$10,$D0,$62,$C8,$B1,$9B,$85,$0F,$A9
            .byte   $00,$85,$AD,$85,$AE,$C8,$68,$AA,$85,$A0,$68,$85,$A1,$D1,$9B,$90
            .byte   $0E,$D0,$06,$C8,$8A,$D1,$9B,$90,$07,$4C,$96,$E1,$4C,$10,$D4,$C8
            .byte   $A5,$AE,$05,$AD,$18,$F0,$0A,$20,$AD,$E2,$8A,$65,$A0,$AA,$98,$A4
            .byte   $5E,$65,$A1,$86,$AD,$C6,$0F,$D0,$CA,$85,$AE,$A2,$05,$A5,$81,$10
            .byte   $01,$CA,$A5,$82,$10,$02,$CA,$CA,$86,$64,$A9,$00,$20,$B6,$E2,$8A
            .byte   $65,$94,$85,$83,$98,$65,$95,$85,$84,$A8,$A5,$83,$60,$84,$5E,$B1
            .byte   $9B,$85,$64,$88,$B1,$9B,$85,$65,$A9,$10,$85,$99,$A2,$00,$A0,$00
            .byte   $8A,$0A,$AA,$98,$2A,$A8,$B0,$A4,$06,$AD,$26,$AE,$90,$0B,$18,$8A
            .byte   $65,$64,$AA,$98,$65,$65,$A8,$B0,$93,$C6,$99,$D0,$E3,$60,$A5,$11
            .byte   $F0,$03,$20,$00,$E6,$20,$84,$E4,$38,$A5,$6F,$E5,$6D,$A8,$A5,$70
            .byte   $E5,$6E,$A2,$00,$86,$11,$85,$9E,$84,$9F,$A2,$90,$4C,$9B,$EB,$A4
            .byte   $24,$A9,$00,$38,$F0,$EC,$A6,$76,$E8,$D0,$A1,$A2,$95,$2C,$A2,$E0
            .byte   $4C,$12,$D4,$20,$41,$E3,$20,$06,$E3,$20,$BB,$DE,$A9,$80,$85,$14
            .byte   $20,$E3,$DF,$20,$6A,$DD,$20,$B8,$DE,$A9,$D0,$20,$C0,$DE,$48,$A5
            .byte   $84,$48,$A5,$83,$48,$A5,$B9,$48,$A5,$B8,$48,$20,$95,$D9,$4C,$AF
            .byte   $E3,$A9,$C2,$20,$C0,$DE,$09,$80,$85,$14,$20,$EA,$DF,$85,$8A,$84
            .byte   $8B,$4C,$6A,$DD,$20,$41,$E3,$A5,$8B,$48,$A5,$8A,$48,$20,$B2,$DE
            .byte   $20,$6A,$DD,$68,$85,$8A,$68,$85,$8B,$A0,$02,$B1,$8A,$85,$83,$AA
            .byte   $C8,$B1,$8A,$F0,$99,$85,$84,$C8,$B1,$83,$48,$88,$10,$FA,$A4,$84
            .byte   $20,$2B,$EB,$A5,$B9,$48,$A5,$B8,$48,$B1,$8A,$85,$B8,$C8,$B1,$8A
            .byte   $85,$B9,$A5,$84,$48,$A5,$83,$48,$20,$67,$DD,$68,$85,$8A,$68,$85
            .byte   $8B,$20,$B7,$00,$F0,$03,$4C,$C9,$DE,$68,$85,$B8,$68,$85,$B9,$A0
            .byte   $00,$68,$91,$8A,$68,$C8,$91,$8A,$68,$C8,$91,$8A,$68,$C8,$91,$8A
            .byte   $68,$C8,$91,$8A,$60,$20,$6A,$DD,$A0,$00,$20,$36,$ED,$68,$68,$A9
            .byte   $FF,$A0,$00,$F0,$12,$A6,$A0,$A4,$A1,$86,$8C,$84,$8D,$20,$52,$E4
            .byte   $86,$9E,$84,$9F,$85,$9D,$60,$A2,$22,$86,$0D,$86,$0E,$85,$AB,$84
            .byte   $AC,$85,$9E,$84,$9F,$A0,$FF,$C8,$B1,$AB,$F0,$0C,$C5,$0D,$F0,$04
            .byte   $C5,$0E,$D0,$F3,$C9,$22,$F0,$01,$18,$84,$9D,$98,$65,$AB,$85,$AD
            .byte   $A6,$AC,$90,$01,$E8,$86,$AE,$A5,$AC,$F0,$04,$C9,$02,$D0,$0B,$98
            .byte   $20,$D5,$E3,$A6,$AB,$A4,$AC,$20,$E2,$E5,$A6,$52,$E0,$5E,$D0,$05
            .byte   $A2,$BF,$4C,$12,$D4,$A5,$9D,$95,$00,$A5,$9E,$95,$01,$A5,$9F,$95
            .byte   $02,$A0,$00,$86,$A0,$84,$A1,$88,$84,$11,$86,$53,$E8,$E8,$E8,$86
            .byte   $52,$60,$46,$13,$48,$49,$FF,$38,$65,$6F,$A4,$70,$B0,$01,$88,$C4
            .byte   $6E,$90,$11,$D0,$04,$C5,$6D,$90,$0B,$85,$6F,$84,$70,$85,$71,$84
            .byte   $72,$AA,$68,$60,$A2,$4D,$A5,$13,$30,$B8,$20,$84,$E4,$A9,$80,$85
            .byte   $13,$68,$D0,$D0,$A6,$73,$A5,$74,$86,$6F,$85,$70,$A0,$00,$84,$8B
            .byte   $A5,$6D,$A6,$6E,$85,$9B,$86,$9C,$A9,$55,$A2,$00,$85,$5E,$86,$5F
            .byte   $C5,$52,$F0,$05,$20,$23,$E5,$F0,$F7,$A9,$07,$85,$8F,$A5,$69,$A6
            .byte   $6A,$85,$5E,$86,$5F,$E4,$6C,$D0,$04,$C5,$6B,$F0,$05,$20,$19,$E5
            .byte   $F0,$F3,$85,$94,$86,$95,$A9,$03,$85,$8F,$A5,$94,$A6,$95,$E4,$6E
            .byte   $D0,$07,$C5,$6D,$D0,$03,$4C,$62,$E5,$85,$5E,$86,$5F,$A0,$00,$B1
            .byte   $5E,$AA,$C8,$B1,$5E,$08,$C8,$B1,$5E,$65,$94,$85,$94,$C8,$B1,$5E
            .byte   $65,$95,$85,$95,$28,$10,$D3,$8A,$30,$D0,$C8,$B1,$5E,$A0,$00,$0A
            .byte   $69,$05,$65,$5E,$85,$5E,$90,$02,$E6,$5F,$A6,$5F,$E4,$95,$D0,$04
            .byte   $C5,$94,$F0,$BA,$20,$23,$E5,$F0,$F3,$B1,$5E,$30,$35,$C8,$B1,$5E
            .byte   $10,$30,$C8,$B1,$5E,$F0,$2B,$C8,$B1,$5E,$AA,$C8,$B1,$5E,$C5,$70
            .byte   $90,$06,$D0,$1E,$E4,$6F,$B0,$1A,$C5,$9C,$90,$16,$D0,$04,$E4,$9B
            .byte   $90,$10,$86,$9B,$85,$9C,$A5,$5E,$A6,$5F,$85,$8A,$86,$8B,$A5,$8F
            .byte   $85,$91,$A5,$8F,$18,$65,$5E,$85,$5E,$90,$02,$E6,$5F,$A6,$5F,$A0
            .byte   $00,$60,$A6,$8B,$F0,$F7,$A5,$91,$29,$04,$4A,$A8,$85,$91,$B1,$8A
            .byte   $65,$9B,$85,$96,$A5,$9C,$69,$00,$85,$97,$A5,$6F,$A6,$70,$85,$94
            .byte   $86,$95,$20,$9A,$D3,$A4,$91,$C8,$A5,$94,$91,$8A,$AA,$E6,$95,$A5
            .byte   $95,$C8,$91,$8A,$4C,$88,$E4,$A5,$A1,$48,$A5,$A0,$48,$20,$60,$DE
            .byte   $20,$6C,$DD,$68,$85,$AB,$68,$85,$AC,$A0,$00,$B1,$AB,$18,$71,$A0
            .byte   $90,$05,$A2,$B0,$4C,$12,$D4,$20,$D5,$E3,$20,$D4,$E5,$A5,$8C,$A4
            .byte   $8D,$20,$04,$E6,$20,$E6,$E5,$A5,$AB,$A4,$AC,$20,$04,$E6,$20,$2A
            .byte   $E4,$4C,$95,$DD,$A0,$00,$B1,$AB,$48,$C8,$B1,$AB,$AA,$C8,$B1,$AB
            .byte   $A8,$68,$86,$5E,$84,$5F,$A8,$F0,$0A,$48,$88,$B1,$5E,$91,$71,$98
            .byte   $D0,$F8,$68,$18,$65,$71,$85,$71,$90,$02,$E6,$72,$60,$20,$6C,$DD
            .byte   $A5,$A0,$A4,$A1,$85,$5E,$84,$5F,$20,$35,$E6,$08,$A0,$00,$B1,$5E
            .byte   $48,$C8,$B1,$5E,$AA,$C8,$B1,$5E,$A8,$68,$28,$D0,$13,$C4,$70,$D0
            .byte   $0F,$E4,$6F,$D0,$0B,$48,$18,$65,$6F,$85,$6F,$90,$02,$E6,$70,$68
            .byte   $86,$5E,$84,$5F,$60,$C4,$54,$D0,$0C,$C5,$53,$D0,$08,$85,$52,$E9
            .byte   $03,$85,$53,$A0,$00,$60,$20,$FB,$E6,$8A,$48,$A9,$01,$20,$DD,$E3
            .byte   $68,$A0,$00,$91,$9E,$68,$68,$4C,$2A,$E4,$20,$B9,$E6,$D1,$8C,$98
            .byte   $90,$04,$B1,$8C,$AA,$98,$48,$8A,$48,$20,$DD,$E3,$A5,$8C,$A4,$8D
            .byte   $20,$04,$E6,$68,$A8,$68,$18,$65,$5E,$85,$5E,$90,$02,$E6,$5F,$98
            .byte   $20,$E6,$E5,$4C,$2A,$E4,$20,$B9,$E6,$18,$F1,$8C,$49,$FF,$4C,$60
            .byte   $E6,$A9,$FF,$85,$A1,$20,$B7,$00,$C9,$29,$F0,$06,$20,$BE,$DE,$20
            .byte   $F8,$E6,$20,$B9,$E6,$CA,$8A,$48,$18,$A2,$00,$F1,$8C,$B0,$B8,$49
            .byte   $FF,$C5,$A1,$90,$B3,$A5,$A1,$B0,$AF,$20,$B8,$DE,$68,$A8,$68,$85
            .byte   $91,$68,$68,$68,$AA,$68,$85,$8C,$68,$85,$8D,$A5,$91,$48,$98,$48
            .byte   $A0,$00,$8A,$F0,$1D,$60,$20,$DC,$E6,$4C,$01,$E3,$20,$FD,$E5,$A2
            .byte   $00,$86,$11,$A8,$60,$20,$DC,$E6,$F0,$08,$A0,$00,$B1,$5E,$A8,$4C
            .byte   $01,$E3,$4C,$99,$E1,$20,$B1,$00,$20,$67,$DD,$20,$08,$E1,$A6,$A0
            .byte   $D0,$F0,$A6,$A1,$4C,$B7,$00,$20,$DC,$E6,$D0,$03,$4C,$4E,$E8,$A6
            .byte   $B8,$A4,$B9,$86,$AD,$84,$AE,$A6,$5E,$86,$B8,$18,$65,$5E,$85,$60
            .byte   $A6,$5F,$86,$B9,$90,$01,$E8,$86,$61,$A0,$00,$B1,$60,$48,$A9,$00
            .byte   $91,$60,$20,$B7,$00,$20,$4A,$EC,$68,$A0,$00,$91,$60,$A6,$AD,$A4
            .byte   $AE,$86,$B8,$84,$B9,$60,$20,$67,$DD,$20,$52,$E7,$20,$BE,$DE,$4C
            .byte   $F8,$E6,$A5,$9D,$C9,$91,$B0,$9A,$20,$F2,$EB,$A5,$A0,$A4,$A1,$84
            .byte   $50,$85,$51,$60,$A5,$50,$48,$A5,$51,$48,$20,$52,$E7,$A0,$00,$B1
            .byte   $50,$A8,$68,$85,$51,$68,$85,$50,$4C,$01,$E3,$20,$46,$E7,$8A,$A0
            .byte   $00,$91,$50,$60,$20,$46,$E7,$86,$85,$A2,$00,$20,$B7,$00,$F0,$03
            .byte   $20,$4C,$E7,$86,$86,$A0,$00,$B1,$50,$45,$86,$25,$85,$F0,$F8,$60
            .byte   $A9,$64,$A0,$EE,$4C,$BE,$E7,$20,$E3,$E9,$A5,$A2,$49,$FF,$85,$A2
            .byte   $45,$AA,$85,$AB,$A5,$9D,$4C,$C1,$E7,$20,$F0,$E8,$90,$3C,$20,$E3
            .byte   $E9,$D0,$03,$4C,$53,$EB,$A6,$AC,$86,$92,$A2,$A5,$A5,$A5,$A8,$F0
            .byte   $CE,$38,$E5,$9D,$F0,$24,$90,$12,$84,$9D,$A4,$AA,$84,$A2,$49,$FF
            .byte   $69,$00,$A0,$00,$84,$92,$A2,$9D,$D0,$04,$A0,$00,$84,$AC,$C9,$F9
            .byte   $30,$C7,$A8,$A5,$AC,$56,$01,$20,$07,$E9,$24,$AB,$10,$57,$A0,$9D
            .byte   $E0,$A5,$F0,$02,$A0,$A5,$38,$49,$FF,$65,$92,$85,$AC,$B9,$04,$00
            .byte   $F5,$04,$85,$A1,$B9,$03,$00,$F5,$03,$85,$A0,$B9,$02,$00,$F5,$02
            .byte   $85,$9F,$B9,$01,$00,$F5,$01,$85,$9E,$B0,$03,$20,$9E,$E8,$A0,$00
            .byte   $98,$18,$A6,$9E,$D0,$4A,$A6,$9F,$86,$9E,$A6,$A0,$86,$9F,$A6,$A1
            .byte   $86,$A0,$A6,$AC,$86,$A1,$84,$AC,$69,$08,$C9,$20,$D0,$E4,$A9,$00
            .byte   $85,$9D,$85,$A2,$60,$65,$92,$85,$AC,$A5,$A1,$65,$A9,$85,$A1,$A5
            .byte   $A0,$65,$A8,$85,$A0,$A5,$9F,$65,$A7,$85,$9F,$A5,$9E,$65,$A6,$85
            .byte   $9E,$4C,$8D,$E8,$69,$01,$06,$AC,$26,$A1,$26,$A0,$26,$9F,$26,$9E
            .byte   $10,$F2,$38,$E5,$9D,$B0,$C7,$49,$FF,$69,$01,$85,$9D,$90,$0E,$E6
            .byte   $9D,$F0,$42,$66,$9E,$66,$9F,$66,$A0,$66,$A1,$66,$AC,$60,$A5,$A2
            .byte   $49,$FF,$85,$A2,$A5,$9E,$49,$FF,$85,$9E,$A5,$9F,$49,$FF,$85,$9F
            .byte   $A5,$A0,$49,$FF,$85,$A0,$A5,$A1,$49,$FF,$85,$A1,$A5,$AC,$49,$FF
            .byte   $85,$AC,$E6,$AC,$D0,$0E,$E6,$A1,$D0,$0A,$E6,$A0,$D0,$06,$E6,$9F
            .byte   $D0,$02,$E6,$9E,$60,$A2,$45,$4C,$12,$D4,$A2,$61,$B4,$04,$84,$AC
            .byte   $B4,$03,$94,$04,$B4,$02,$94,$03,$B4,$01,$94,$02,$A4,$A4,$94,$01
            .byte   $69,$08,$30,$E8,$F0,$E6,$E9,$08,$A8,$A5,$AC,$B0,$14,$16,$01,$90
            .byte   $02,$F6,$01,$76,$01,$76,$01,$76,$02,$76,$03,$76,$04,$6A,$C8,$D0
            .byte   $EC,$18,$60,$81,$00,$00,$00,$00,$03,$7F,$5E,$56,$CB,$79,$80,$13
            .byte   $9B,$0B,$64,$80,$76,$38,$93,$16,$82,$38,$AA,$3B,$20,$80,$35,$04
            .byte   $F3,$34,$81,$35,$04,$F3,$34,$80,$80,$00,$00,$00,$80,$31,$72,$17
            .byte   $F8,$20,$82,$EB,$F0,$02,$10,$03,$4C,$99,$E1,$A5,$9D,$E9,$7F,$48
            .byte   $A9,$80,$85,$9D,$A9,$2D,$A0,$E9,$20,$BE,$E7,$A9,$32,$A0,$E9,$20
            .byte   $66,$EA,$A9,$13,$A0,$E9,$20,$A7,$E7,$A9,$18,$A0,$E9,$20,$5C,$EF
            .byte   $A9,$37,$A0,$E9,$20,$BE,$E7,$68,$20,$D5,$EC,$A9,$3C,$A0,$E9,$20
            .byte   $E3,$E9,$D0,$03,$4C,$E2,$E9,$20,$0E,$EA,$A9,$00,$85,$62,$85,$63
            .byte   $85,$64,$85,$65,$A5,$AC,$20,$B0,$E9,$A5,$A1,$20,$B0,$E9,$A5,$A0
            .byte   $20,$B0,$E9,$A5,$9F,$20,$B0,$E9,$A5,$9E,$20,$B5,$E9,$4C,$E6,$EA
            .byte   $D0,$03,$4C,$DA,$E8,$4A,$09,$80,$A8,$90,$19,$18,$A5,$65,$65,$A9
            .byte   $85,$65,$A5,$64,$65,$A8,$85,$64,$A5,$63,$65,$A7,$85,$63,$A5,$62
            .byte   $65,$A6,$85,$62,$66,$62,$66,$63,$66,$64,$66,$65,$66,$AC,$98,$4A
            .byte   $D0,$D6,$60,$85,$5E,$84,$5F,$A0,$04,$B1,$5E,$85,$A9,$88,$B1,$5E
            .byte   $85,$A8,$88,$B1,$5E,$85,$A7,$88,$B1,$5E,$85,$AA,$45,$A2,$85,$AB
            .byte   $A5,$AA,$09,$80,$85,$A6,$88,$B1,$5E,$85,$A5,$A5,$9D,$60,$A5,$A5
            .byte   $F0,$1F,$18,$65,$9D,$90,$04,$30,$1D,$18,$2C,$10,$14,$69,$80,$85
            .byte   $9D,$D0,$03,$4C,$52,$E8,$A5,$AB,$85,$A2,$60,$A5,$A2,$49,$FF,$30
            .byte   $05,$68,$68,$4C,$4E,$E8,$4C,$D5,$E8,$20,$63,$EB,$AA,$F0,$10,$18
            .byte   $69,$02,$B0,$F2,$A2,$00,$86,$AB,$20,$CE,$E7,$E6,$9D,$F0,$E7,$60
            .byte   $84,$20,$00,$00,$00,$20,$63,$EB,$A9,$50,$A0,$EA,$A2,$00,$86,$AB
            .byte   $20,$F9,$EA,$4C,$69,$EA,$20,$E3,$E9,$F0,$76,$20,$72,$EB,$A9,$00
            .byte   $38,$E5,$9D,$85,$9D,$20,$0E,$EA,$E6,$9D,$F0,$BA,$A2,$FC,$A9,$01
            .byte   $A4,$A6,$C4,$9E,$D0,$10,$A4,$A7,$C4,$9F,$D0,$0A,$A4,$A8,$C4,$A0
            .byte   $D0,$04,$A4,$A9,$C4,$A1,$08,$2A,$90,$09,$E8,$95,$65,$F0,$32,$10
            .byte   $34,$A9,$01,$28,$B0,$0E,$06,$A9,$26,$A8,$26,$A7,$26,$A6,$B0,$E6
            .byte   $30,$CE,$10,$E2,$A8,$A5,$A9,$E5,$A1,$85,$A9,$A5,$A8,$E5,$A0,$85
            .byte   $A8,$A5,$A7,$E5,$9F,$85,$A7,$A5,$A6,$E5,$9E,$85,$A6,$98,$4C,$A6
            .byte   $EA,$A9,$40,$D0,$CE,$0A,$0A,$0A,$0A,$0A,$0A,$85,$AC,$28,$4C,$E6
            .byte   $EA,$A2,$85,$4C,$12,$D4,$A5,$62,$85,$9E,$A5,$63,$85,$9F,$A5,$64
            .byte   $85,$A0,$A5,$65,$85,$A1,$4C,$2E,$E8,$85,$5E,$84,$5F,$A0,$04,$B1
            .byte   $5E,$85,$A1,$88,$B1,$5E,$85,$A0,$88,$B1,$5E,$85,$9F,$88,$B1,$5E
            .byte   $85,$A2,$09,$80,$85,$9E,$88,$B1,$5E,$85,$9D,$84,$AC,$60,$A2,$98
            .byte   $2C,$A2,$93,$A0,$00,$F0,$04,$A6,$85,$A4,$86,$20,$72,$EB,$86,$5E
            .byte   $84,$5F,$A0,$04,$A5,$A1,$91,$5E,$88,$A5,$A0,$91,$5E,$88,$A5,$9F
            .byte   $91,$5E,$88,$A5,$A2,$09,$7F,$25,$9E,$91,$5E,$88,$A5,$9D,$91,$5E
            .byte   $84,$AC,$60,$A5,$AA,$85,$A2,$A2,$05,$B5,$A4,$95,$9C,$CA,$D0,$F9
            .byte   $86,$AC,$60,$20,$72,$EB,$A2,$06,$B5,$9C,$95,$A4,$CA,$D0,$F9,$86
            .byte   $AC,$60,$A5,$9D,$F0,$FB,$06,$AC,$90,$F7,$20,$C6,$E8,$D0,$F2,$4C
            .byte   $8F,$E8,$A5,$9D,$F0,$09,$A5,$A2,$2A,$A9,$FF,$B0,$02,$A9,$01,$60
            .byte   $20,$82,$EB,$85,$9E,$A9,$00,$85,$9F,$A2,$88,$A5,$9E,$49,$FF,$2A
            .byte   $A9,$00,$85,$A1,$85,$A0,$86,$9D,$85,$AC,$85,$A2,$4C,$29,$E8,$46
            .byte   $A2,$60,$85,$60,$84,$61,$A0,$00,$B1,$60,$C8,$AA,$F0,$C4,$B1,$60
            .byte   $45,$A2,$30,$C2,$E4,$9D,$D0,$21,$B1,$60,$09,$80,$C5,$9E,$D0,$19
            .byte   $C8,$B1,$60,$C5,$9F,$D0,$12,$C8,$B1,$60,$C5,$A0,$D0,$0B,$C8,$A9
            .byte   $7F,$C5,$AC,$B1,$60,$E5,$A1,$F0,$28,$A5,$A2,$90,$02,$49,$FF,$4C
            .byte   $88,$EB,$A5,$9D,$F0,$4A,$38,$E9,$A0,$24,$A2,$10,$09,$AA,$A9,$FF
            .byte   $85,$A4,$20,$A4,$E8,$8A,$A2,$9D,$C9,$F9,$10,$06,$20,$F0,$E8,$84
            .byte   $A4,$60,$A8,$A5,$A2,$29,$80,$46,$9E,$05,$9E,$85,$9E,$20,$07,$E9
            .byte   $84,$A4,$60,$A5,$9D,$C9,$A0,$B0,$20,$20,$F2,$EB,$84,$AC,$A5,$A2
            .byte   $84,$A2,$49,$80,$2A,$A9,$A0,$85,$9D,$A5,$A1,$85,$0D,$4C,$29,$E8
            .byte   $85,$9E,$85,$9F,$85,$A0,$85,$A1,$A8,$60,$A0,$00,$A2,$0A,$94,$99
            .byte   $CA,$10,$FB,$90,$0F,$C9,$2D,$D0,$04,$86,$A3,$F0,$04,$C9,$2B,$D0
            .byte   $05,$20,$B1,$00,$90,$5B,$C9,$2E,$F0,$2E,$C9,$45,$D0,$30,$20,$B1
            .byte   $00,$90,$17,$C9,$C9,$F0,$0E,$C9,$2D,$F0,$0A,$C9,$C8,$F0,$08,$C9
            .byte   $2B,$F0,$04,$D0,$07,$66,$9C,$20,$B1,$00,$90,$5C,$24,$9C,$10,$0E
            .byte   $A9,$00,$38,$E5,$9A,$4C,$A0,$EC,$66,$9B,$24,$9B,$50,$C3,$A5,$9A
            .byte   $38,$E5,$99,$85,$9A,$F0,$12,$10,$09,$20,$55,$EA,$E6,$9A,$D0,$F9
            .byte   $F0,$07,$20,$39,$EA,$C6,$9A,$D0,$F9,$A5,$A3,$30,$01,$60,$4C,$D0
            .byte   $EE,$48,$24,$9B,$10,$02,$E6,$99,$20,$39,$EA,$68,$38,$E9,$30,$20
            .byte   $D5,$EC,$4C,$61,$EC,$48,$20,$63,$EB,$68,$20,$93,$EB,$A5,$AA,$45
            .byte   $A2,$85,$AB,$A6,$9D,$4C,$C1,$E7,$A5,$9A,$C9,$0A,$90,$09,$A9,$64
            .byte   $24,$9C,$30,$11,$4C,$D5,$E8,$0A,$0A,$18,$65,$9A,$0A,$18,$A0,$00
            .byte   $71,$B8,$38,$E9,$30,$85,$9A,$4C,$87,$EC,$9B,$3E,$BC,$1F,$FD,$9E
            .byte   $6E,$6B,$27,$FD,$9E,$6E,$6B,$28,$00,$A9,$58,$A0,$D3,$20,$31,$ED
            .byte   $A5,$76,$A6,$75,$85,$9E,$86,$9F,$A2,$90,$38,$20,$A0,$EB,$20,$34
            .byte   $ED,$4C,$3A,$DB,$A0,$01,$A9,$2D,$88,$24,$A2,$10,$04,$C8,$99,$FF
            .byte   $00,$85,$A2,$84,$AD,$C8,$A9,$30,$A6,$9D,$D0,$03,$4C,$57,$EE,$A9
            .byte   $00,$E0,$80,$F0,$02,$B0,$09,$A9,$14,$A0,$ED,$20,$7F,$E9,$A9,$F7
            .byte   $85,$99,$A9,$0F,$A0,$ED,$20,$B2,$EB,$F0,$1E,$10,$12,$A9,$0A,$A0
            .byte   $ED,$20,$B2,$EB,$F0,$02,$10,$0E,$20,$39,$EA,$C6,$99,$D0,$EE,$20
            .byte   $55,$EA,$E6,$99,$D0,$DC,$20,$A0,$E7,$20,$F2,$EB,$A2,$01,$A5,$99
            .byte   $18,$69,$0A,$30,$09,$C9,$0B,$B0,$06,$69,$FF,$AA,$A9,$02,$38,$E9
            .byte   $02,$85,$9A,$86,$99,$8A,$F0,$02,$10,$13,$A4,$AD,$A9,$2E,$C8,$99
            .byte   $FF,$00,$8A,$F0,$06,$A9,$30,$C8,$99,$FF,$00,$84,$AD,$A0,$00,$A2
            .byte   $80,$A5,$A1,$18,$79,$6C,$EE,$85,$A1,$A5,$A0,$79,$6B,$EE,$85,$A0
            .byte   $A5,$9F,$79,$6A,$EE,$85,$9F,$A5,$9E,$79,$69,$EE,$85,$9E,$E8,$B0
            .byte   $04,$10,$DE,$30,$02,$30,$DA,$8A,$90,$04,$49,$FF,$69,$0A,$69,$2F
            .byte   $C8,$C8,$C8,$C8,$84,$83,$A4,$AD,$C8,$AA,$29,$7F,$99,$FF,$00,$C6
            .byte   $99,$D0,$06,$A9,$2E,$C8,$99,$FF,$00,$84,$AD,$A4,$83,$8A,$49,$FF
            .byte   $29,$80,$AA,$C0,$24,$D0,$AA,$A4,$AD,$B9,$FF,$00,$88,$C9,$30,$F0
            .byte   $F8,$C9,$2E,$F0,$01,$C8,$A9,$2B,$A6,$9A,$F0,$2E,$10,$08,$A9,$00
            .byte   $38,$E5,$9A,$AA,$A9,$2D,$99,$01,$01,$A9,$45,$99,$00,$01,$8A,$A2
            .byte   $2F,$38,$E8,$E9,$0A,$B0,$FB,$69,$3A,$99,$03,$01,$8A,$99,$02,$01
            .byte   $A9,$00,$99,$04,$01,$F0,$08,$99,$FF,$00,$A9,$00,$99,$00,$01,$A9
            .byte   $00,$A0,$01,$60,$80,$00,$00,$00,$00,$FA,$0A,$1F,$00,$00,$98,$96
            .byte   $80,$FF,$F0,$BD,$C0,$00,$01,$86,$A0,$FF,$FF,$D8,$F0,$00,$00,$03
            .byte   $E8,$FF,$FF,$FF,$9C,$00,$00,$00,$0A,$FF,$FF,$FF,$FF,$20,$63,$EB
            .byte   $A9,$64,$A0,$EE,$20,$F9,$EA,$F0,$70,$A5,$A5,$D0,$03,$4C,$50,$E8
            .byte   $A2,$8A,$A0,$00,$20,$2B,$EB,$A5,$AA,$10,$0F,$20,$23,$EC,$A9,$8A
            .byte   $A0,$00,$20,$B2,$EB,$D0,$03,$98,$A4,$0D,$20,$55,$EB,$98,$48,$20
            .byte   $41,$E9,$A9,$8A,$A0,$00,$20,$7F,$E9,$20,$09,$EF,$68,$4A,$90,$0A
            .byte   $A5,$9D,$F0,$06,$A5,$A2,$49,$FF,$85,$A2,$60,$81,$38,$AA,$3B,$29
            .byte   $07,$71,$34,$58,$3E,$56,$74,$16,$7E,$B3,$1B,$77,$2F,$EE,$E3,$85
            .byte   $7A,$1D,$84,$1C,$2A,$7C,$63,$59,$58,$0A,$7E,$75,$FD,$E7,$C6,$80
            .byte   $31,$72,$18,$10,$81,$00,$00,$00,$00,$A9,$DB,$A0,$EE,$20,$7F,$E9
            .byte   $A5,$AC,$69,$50,$90,$03,$20,$7A,$EB,$85,$92,$20,$66,$EB,$A5,$9D
            .byte   $C9,$88,$90,$03,$20,$2B,$EA,$20,$23,$EC,$A5,$0D,$18,$69,$81,$F0
            .byte   $F3,$38,$E9,$01,$48,$A2,$05,$B5,$A5,$B4,$9D,$95,$9D,$94,$A5,$CA
            .byte   $10,$F5,$A5,$92,$85,$AC,$20,$AA,$E7,$20,$D0,$EE,$A9,$E0,$A0,$EE
            .byte   $20,$72,$EF,$A9,$00,$85,$AB,$68,$20,$10,$EA,$60,$85,$AD,$84,$AE
            .byte   $20,$21,$EB,$A9,$93,$20,$7F,$E9,$20,$76,$EF,$A9,$93,$A0,$00,$4C
            .byte   $7F,$E9,$85,$AD,$84,$AE,$20,$1E,$EB,$B1,$AD,$85,$A3,$A4,$AD,$C8
            .byte   $98,$D0,$02,$E6,$AE,$85,$AD,$A4,$AE,$20,$7F,$E9,$A5,$AD,$A4,$AE
            .byte   $18,$69,$05,$90,$01,$C8,$85,$AD,$84,$AE,$20,$BE,$E7,$A9,$98,$A0
            .byte   $00,$C6,$A3,$D0,$E4,$60,$98,$35,$44,$7A,$68,$28,$B1,$46,$20,$82
            .byte   $EB,$AA,$30,$18,$A9,$C9,$A0,$00,$20,$F9,$EA,$8A,$F0,$E7,$A9,$A6
            .byte   $A0,$EF,$20,$7F,$E9,$A9,$AA,$A0,$EF,$20,$BE,$E7,$A6,$A1,$A5,$9E
            .byte   $85,$A1,$86,$9E,$A9,$00,$85,$A2,$A5,$9D,$85,$AC,$A9,$80,$85,$9D
            .byte   $20,$2E,$E8,$A2,$C9,$A0,$00,$4C,$2B,$EB,$A9,$66,$A0,$F0,$20,$BE
            .byte   $E7,$20,$63,$EB,$A9,$6B,$A0,$F0,$A6,$AA,$20,$5E,$EA,$20,$63,$EB
            .byte   $20,$23,$EC,$A9,$00,$85,$AB,$20,$AA,$E7,$A9,$70,$A0,$F0,$20,$A7
            .byte   $E7,$A5,$A2,$48,$10,$0D,$20,$A0,$E7,$A5,$A2,$30,$09,$A5,$16,$49
            .byte   $FF,$85,$16,$20,$D0,$EE,$A9,$70,$A0,$F0,$20,$BE,$E7,$68,$10,$03
            .byte   $20,$D0,$EE,$A9,$75,$A0,$F0,$4C,$5C,$EF,$20,$21,$EB,$A9,$00,$85
            .byte   $16,$20,$F1,$EF,$A2,$8A,$A0,$00,$20,$E7,$EF,$A9,$93,$A0,$00,$20
            .byte   $F9,$EA,$A9,$00,$85,$A2,$A5,$16,$20,$62,$F0,$A9,$8A,$A0,$00,$4C
            .byte   $66,$EA,$48,$4C,$23,$F0,$81,$49,$0F,$DA,$A2,$83,$49,$0F,$DA,$A2
            .byte   $7F,$00,$00,$00,$00,$05,$84,$E6,$1A,$2D,$1B,$86,$28,$07,$FB,$F8
            .byte   $87,$99,$68,$89,$01,$87,$23,$35,$DF,$E1,$86,$A5,$5D,$E7,$28,$83
            .byte   $49,$0F,$DA,$A2,$A6,$D3,$C1,$C8,$D4,$C8,$D5,$C4,$CE,$CA,$A5,$A2
            .byte   $48,$10,$03,$20,$D0,$EE,$A5,$9D,$48,$C9,$81,$90,$07,$A9,$13,$A0
            .byte   $E9,$20,$66,$EA,$A9,$CE,$A0,$F0,$20,$5C,$EF,$68,$C9,$81,$90,$07
            .byte   $A9,$66,$A0,$F0,$20,$A7,$E7,$68,$10,$03,$4C,$D0,$EE,$60,$0B,$76
            .byte   $B3,$83,$BD,$D3,$79,$1E,$F4,$A6,$F5,$7B,$83,$FC,$B0,$10,$7C,$0C
            .byte   $1F,$67,$CA,$7C,$DE,$53,$CB,$C1,$7D,$14,$64,$70,$4C,$7D,$B7,$EA
            .byte   $51,$7A,$7D,$63,$30,$88,$7E,$7E,$92,$44,$99,$3A,$7E,$4C,$CC,$91
            .byte   $C7,$7F,$AA,$AA,$AA,$13,$81,$00,$00,$00,$00,$E6,$B8,$D0,$02,$E6
            .byte   $B9,$AD,$60,$EA,$C9,$3A,$B0,$0A,$C9,$20,$F0,$EF,$38,$E9,$30,$38
            .byte   $E9,$D0,$60,$80,$4F,$C7,$52,$58,$A2,$FF,$86,$76,$A2,$FB,$9A,$A9
            .byte   $28,$A0,$F1,$85,$01,$84,$02,$85,$04,$84,$05,$20,$73,$F2,$A9,$4C
            .byte   $85,$00,$85,$03,$85,$90,$85,$0A,$A9,$99,$A0,$E1,$85,$0B,$84,$0C
            .byte   $A2,$1C,$BD,$0A,$F1,$95,$B0,$86,$F1,$CA,$D0,$F6,$86,$F2,$8A,$85
            .byte   $A4,$85,$54,$48,$A9,$03,$85,$8F,$20,$FB,$DA,$A9,$01,$8D,$FD,$01
            .byte   $8D,$FC,$01,$A2,$55,$86,$52,$A9,$00,$A0,$08,$85,$50,$84,$51,$A0
            .byte   $00,$E6,$51,$B1,$50,$49,$FF,$91,$50,$D1,$50,$D0,$08,$49,$FF,$91
            .byte   $50,$D1,$50,$F0,$EC,$A4,$50,$A5,$51,$29,$F0,$84,$73,$85,$74,$84
            .byte   $6F,$85,$70,$A2,$00,$A0,$08,$86,$67,$84,$68,$A0,$00,$84,$D6,$98
            .byte   $91,$67,$E6,$67,$D0,$02,$E6,$68,$A5,$67,$A4,$68,$20,$E3,$D3,$20
            .byte   $4B,$D6,$A9,$3A,$A0,$DB,$85,$04,$84,$05,$A9,$3C,$A0,$D4,$85,$01
            .byte   $84,$02,$6C,$01,$00,$20,$67,$DD,$20,$52,$E7,$6C,$50,$00,$20,$F8
            .byte   $E6,$8A,$4C,$8B,$FE,$20,$F8,$E6,$8A,$4C,$95,$FE,$20,$F8,$E6,$E0
            .byte   $30,$B0,$13,$86,$F0,$A9,$2C,$20,$C0,$DE,$20,$F8,$E6,$E0,$30,$B0
            .byte   $05,$86,$2C,$86,$2D,$60,$4C,$99,$E1,$20,$EC,$F1,$E4,$F0,$B0,$08
            .byte   $A5,$F0,$85,$2C,$85,$2D,$86,$F0,$A9,$C5,$20,$C0,$DE,$20,$F8,$E6
            .byte   $E0,$30,$B0,$E2,$60,$20,$EC,$F1,$8A,$A4,$F0,$C0,$28,$B0,$D7,$4C
            .byte   $00,$F8,$20,$09,$F2,$8A,$A4,$2C,$C0,$28,$B0,$CA,$A4,$F0,$4C,$19
            .byte   $F8,$20,$09,$F2,$8A,$A8,$C0,$28,$B0,$BC,$A5,$F0,$4C,$28,$F8,$20
            .byte   $F8,$E6,$8A,$4C,$64,$F8,$20,$F8,$E6,$CA,$8A,$C9,$18,$B0,$A7,$4C
            .byte   $5B,$FB,$20,$F8,$E6,$8A,$49,$FF,$AA,$E8,$86,$F1,$60,$38,$90,$18
            .byte   $66,$F2,$60,$A9,$FF,$D0,$02,$A9,$3F,$A2,$00,$85,$32,$86,$F3,$60
            .byte   $A9,$7F,$A2,$40,$D0,$F5,$20,$67,$DD,$20,$52,$E7,$A5,$50,$C5,$6D
            .byte   $A5,$51,$E5,$6E,$B0,$03,$4C,$10,$D4,$A5,$50,$85,$73,$85,$6F,$A5
            .byte   $51,$85,$74,$85,$70,$60,$20,$67,$DD,$20,$52,$E7,$A5,$50,$C5,$73
            .byte   $A5,$51,$E5,$74,$B0,$E0,$A5,$50,$C5,$69,$A5,$51,$E5,$6A,$90,$D6
            .byte   $A5,$50,$85,$69,$A5,$51,$85,$6A,$4C,$6C,$D6,$A9,$AB,$20,$C0,$DE
            .byte   $A5,$B8,$85,$F4,$A5,$B9,$85,$F5,$38,$66,$D8,$A5,$75,$85,$F6,$A5
            .byte   $76,$85,$F7,$20,$A6,$D9,$4C,$98,$D9,$86,$DE,$A6,$F8,$86,$DF,$A5
            .byte   $75,$85,$DA,$A5,$76,$85,$DB,$A5,$79,$85,$DC,$A5,$7A,$85,$DD,$A5
            .byte   $F4,$85,$B8,$A5,$F5,$85,$B9,$A5,$F6,$85,$75,$A5,$F7,$85,$76,$20
            .byte   $B7,$00,$20,$3E,$D9,$4C,$D2,$D7,$A5,$DA,$85,$75,$A5,$DB,$85,$76
            .byte   $A5,$DC,$85,$B8,$A5,$DD,$85,$B9,$A6,$DF,$9A,$4C,$D2,$D7,$4C,$C9
            .byte   $DE,$B0,$FB,$A6,$AF,$86,$69,$A6,$B0,$86,$6A,$20,$0C,$DA,$20,$1A
            .byte   $D6,$A5,$9B,$85,$60,$A5,$9C,$85,$61,$A9,$2C,$20,$C0,$DE,$20,$0C
            .byte   $DA,$E6,$50,$D0,$02,$E6,$51,$20,$1A,$D6,$A5,$9B,$C5,$60,$A5,$9C
            .byte   $E5,$61,$B0,$01,$60,$A0,$00,$B1,$9B,$91,$60,$E6,$9B,$D0,$02,$E6
            .byte   $9C,$E6,$60,$D0,$02,$E6,$61,$A5,$69,$C5,$9B,$A5,$6A,$E5,$9C,$B0
            .byte   $E6,$A6,$61,$A4,$60,$D0,$01,$CA,$88,$86,$6A,$84,$69,$4C,$F2,$D4
            .byte   $AD,$56,$C0,$AD,$53,$C0,$4C,$40,$FB,$AD,$54,$C0,$4C,$39,$FB,$20
            .byte   $D9,$F7,$A0,$03,$B1,$9B,$AA,$88,$B1,$9B,$E9,$01,$B0,$01,$CA,$85
            .byte   $50,$86,$51,$20,$CD,$FE,$20,$77,$F7,$4C,$CD,$FE,$20,$D9,$F7,$20
            .byte   $FD,$FE,$A0,$02,$B1,$9B,$C5,$50,$C8,$B1,$9B,$E5,$51,$B0,$03,$4C
            .byte   $10,$D4,$20,$77,$F7,$4C,$FD,$FE,$2C,$55,$C0,$2C,$52,$C0,$A9,$40
            .byte   $D0,$08,$A9,$20,$2C,$54,$C0,$2C,$53,$C0,$85,$E6,$AD,$57,$C0,$AD
            .byte   $50,$C0,$A9,$00,$85,$1C,$A5,$E6,$85,$1B,$A0,$00,$84,$1A,$A5,$1C
            .byte   $91,$1A,$20,$7E,$F4,$C8,$D0,$F6,$E6,$1B,$A5,$1B,$29,$1F,$D0,$EE
            .byte   $60,$85,$E2,$86,$E0,$84,$E1,$48,$29,$C0,$85,$26,$4A,$4A,$05,$26
            .byte   $85,$26,$68,$85,$27,$0A,$0A,$0A,$26,$27,$0A,$26,$27,$0A,$66,$26
            .byte   $A5,$27,$29,$1F,$05,$E6,$85,$27,$8A,$C0,$00,$F0,$05,$A0,$23,$69
            .byte   $04,$C8,$E9,$07,$B0,$FB,$84,$E5,$AA,$BD,$B9,$F4,$85,$30,$98,$4A
            .byte   $A5,$E4,$85,$1C,$B0,$28,$60,$20,$11,$F4,$A5,$1C,$51,$26,$25,$30
            .byte   $51,$26,$91,$26,$60,$10,$23,$A5,$30,$4A,$B0,$05,$49,$C0,$85,$30
            .byte   $60,$88,$10,$02,$A0,$27,$A9,$C0,$85,$30,$84,$E5,$A5,$1C,$0A,$C9
            .byte   $C0,$10,$06,$A5,$1C,$49,$7F,$85,$1C,$60,$A5,$30,$0A,$49,$80,$30
            .byte   $DD,$A9,$81,$C8,$C0,$28,$90,$E0,$A0,$00,$B0,$DC,$18,$A5,$D1,$29
            .byte   $04,$F0,$25,$A9,$7F,$25,$30,$31,$26,$D0,$19,$E6,$EA,$A9,$7F,$25
            .byte   $30,$10,$11,$18,$A5,$D1,$29,$04,$F0,$0E,$B1,$26,$45,$1C,$25,$30
            .byte   $D0,$02,$E6,$EA,$51,$26,$91,$26,$A5,$D1,$65,$D3,$29,$03,$C9,$02
            .byte   $6A,$B0,$92,$30,$30,$18,$A5,$27,$2C,$B9,$F5,$D0,$22,$06,$26,$B0
            .byte   $1A,$2C,$CD,$F4,$F0,$05,$69,$1F,$38,$B0,$12,$69,$23,$48,$A5,$26
            .byte   $69,$B0,$B0,$02,$69,$F0,$85,$26,$68,$B0,$02,$69,$1F,$66,$26,$69
            .byte   $FC,$85,$27,$60,$18,$A5,$27,$69,$04,$2C,$B9,$F5,$D0,$F3,$06,$26
            .byte   $90,$18,$69,$E0,$18,$2C,$08,$F5,$F0,$12,$A5,$26,$69,$50,$49,$F0
            .byte   $F0,$02,$49,$F0,$85,$26,$A5,$E6,$90,$02,$69,$E0,$66,$26,$90,$D1
            .byte   $48,$A9,$00,$85,$E0,$85,$E1,$85,$E2,$68,$48,$38,$E5,$E0,$48,$8A
            .byte   $E5,$E1,$85,$D3,$B0,$0A,$68,$49,$FF,$69,$01,$48,$A9,$00,$E5,$D3
            .byte   $85,$D1,$85,$D5,$68,$85,$D0,$85,$D4,$68,$85,$E0,$86,$E1,$98,$18
            .byte   $E5,$E2,$90,$04,$49,$FF,$69,$FE,$85,$D2,$84,$E2,$66,$D3,$38,$E5
            .byte   $D0,$AA,$A9,$FF,$E5,$D1,$85,$1D,$A4,$E5,$B0,$05,$0A,$20,$65,$F4
            .byte   $38,$A5,$D4,$65,$D2,$85,$D4,$A5,$D5,$E9,$00,$85,$D5,$B1,$26,$45
            .byte   $1C,$25,$30,$51,$26,$91,$26,$E8,$D0,$04,$E6,$1D,$F0,$62,$A5,$D3
            .byte   $B0,$DA,$20,$D3,$F4,$18,$A5,$D4,$65,$D0,$85,$D4,$A5,$D5,$65,$D1
            .byte   $50,$D9,$81,$82,$84,$88,$90,$A0,$C0,$1C,$FF,$FE,$FA,$F4,$EC,$E1
            .byte   $D4,$C5,$B4,$A1,$8D,$78,$61,$49,$31,$18,$FF,$A5,$26,$0A,$A5,$27
            .byte   $29,$03,$2A,$05,$26,$0A,$0A,$0A,$85,$E2,$A5,$27,$4A,$4A,$29,$07
            .byte   $05,$E2,$85,$E2,$A5,$E5,$0A,$65,$E5,$0A,$AA,$CA,$A5,$30,$29,$7F
            .byte   $E8,$4A,$D0,$FC,$85,$E1,$8A,$18,$65,$E5,$90,$02,$E6,$E1,$85,$E0
            .byte   $60,$86,$1A,$84,$1B,$AA,$4A,$4A,$4A,$4A,$85,$D3,$8A,$29,$0F,$AA
            .byte   $BC,$BA,$F5,$84,$D0,$49,$0F,$AA,$BC,$BB,$F5,$C8,$84,$D2,$A4,$E5
            .byte   $A2,$00,$86,$EA,$A1,$1A,$85,$D1,$A2,$80,$86,$D4,$86,$D5,$A6,$E7
            .byte   $A5,$D4,$38,$65,$D0,$85,$D4,$90,$04,$20,$B3,$F4,$18,$A5,$D5,$65
            .byte   $D2,$85,$D5,$90,$03,$20,$B4,$F4,$CA,$D0,$E5,$A5,$D1,$4A,$4A,$4A
            .byte   $D0,$D4,$E6,$1A,$D0,$02,$E6,$1B,$A1,$1A,$D0,$CA,$60,$86,$1A,$84
            .byte   $1B,$AA,$4A,$4A,$4A,$4A,$85,$D3,$8A,$29,$0F,$AA,$BC,$BA,$F5,$84
            .byte   $D0,$49,$0F,$AA,$BC,$BB,$F5,$C8,$84,$D2,$A4,$E5,$A2,$00,$86,$EA
            .byte   $A1,$1A,$85,$D1,$A2,$80,$86,$D4,$86,$D5,$A6,$E7,$A5,$D4,$38,$65
            .byte   $D0,$85,$D4,$90,$04,$20,$9C,$F4,$18,$A5,$D5,$65,$D2,$85,$D5,$90
            .byte   $03,$20,$9D,$F4,$CA,$D0,$E5,$A5,$D1,$4A,$4A,$4A,$D0,$D4,$E6,$1A
            .byte   $D0,$02,$E6,$1B,$A1,$1A,$D0,$CA,$60,$20,$67,$DD,$20,$52,$E7,$A4
            .byte   $51,$A6,$50,$C0,$01,$90,$06,$D0,$1D,$E0,$18,$B0,$19,$8A,$48,$98
            .byte   $48,$A9,$2C,$20,$C0,$DE,$20,$F8,$E6,$E0,$C0,$B0,$09,$86,$9D,$68
            .byte   $A8,$68,$AA,$A5,$9D,$60,$4C,$06,$F2,$20,$F8,$E6,$E0,$08,$B0,$F6
            .byte   $BD,$F6,$F6,$85,$E4,$60,$00,$2A,$55,$7F,$80,$AA,$D5,$FF,$C9,$C1
            .byte   $F0,$0D,$20,$B9,$F6,$20,$57,$F4,$20,$B7,$00,$C9,$C1,$D0,$E6,$20
            .byte   $C0,$DE,$20,$B9,$F6,$84,$9D,$A8,$8A,$A6,$9D,$20,$3A,$F5,$4C,$08
            .byte   $F7,$20,$F8,$E6,$86,$F9,$60,$20,$F8,$E6,$86,$E7,$60,$20,$F8,$E6
            .byte   $A5,$E8,$85,$1A,$A5,$E9,$85,$1B,$8A,$A2,$00,$C1,$1A,$F0,$02,$B0
            .byte   $A5,$0A,$90,$03,$E6,$1B,$18,$A8,$B1,$1A,$65,$1A,$AA,$C8,$B1,$1A
            .byte   $65,$E9,$85,$1B,$86,$1A,$20,$B7,$00,$C9,$C5,$D0,$09,$20,$C0,$DE
            .byte   $20,$B9,$F6,$20,$11,$F4,$A5,$F9,$60,$20,$2D,$F7,$4C,$05,$F6,$20
            .byte   $2D,$F7,$4C,$61,$F6,$38,$90,$18,$8D,$07,$C0,$20,$00,$C5,$8D,$06
            .byte   $C0,$B0,$01,$60,$4C,$10,$D4,$BD,$01,$02,$10,$11,$A5,$0E,$F0,$16
            .byte   $C9,$22,$F0,$12,$A5,$13,$C9,$49,$F0,$0C,$BD,$00,$02,$08,$C9,$61
            .byte   $90,$02,$29,$5F,$28,$60,$BD,$00,$02,$60,$48,$A9,$20,$20,$5C,$DB
            .byte   $68,$4C,$24,$ED,$A5,$24,$C9,$21,$2C,$1F,$C0,$10,$05,$AD,$7B,$05
            .byte   $C9,$49,$60,$8A,$2C,$1F,$C0,$30,$08,$2C,$85,$24,$38,$8A,$E5,$24
            .byte   $60,$ED,$7B,$05,$60,$00,$00,$00,$00,$A9,$40,$85,$14,$20,$E3,$DF
            .byte   $A9,$00,$85,$14,$4C,$F0,$D8,$20,$F8,$E6,$CA,$A9,$28,$C5,$21,$B0
            .byte   $02,$A5,$21,$20,$CA,$F7,$86,$24,$90,$D6,$AA,$20,$FB,$DA,$D0,$EB
            .byte   $4A,$08,$20,$47,$F8,$28,$A9,$0F,$90,$02,$69,$E0,$85,$2E,$B1,$26
            .byte   $45,$30,$25,$2E,$51,$26,$91,$26,$60,$20,$00,$F8,$C4,$2C,$B0,$11
            .byte   $C8,$20,$0E,$F8,$90,$F6,$69,$01,$48,$20,$00,$F8,$68,$C5,$2D,$90
            .byte   $F5,$60,$A0,$2F,$D0,$02,$A0,$27,$84,$2D,$A0,$27,$A9,$00,$85,$30
            .byte   $20,$28,$F8,$88,$10,$F6,$60,$48,$4A,$29,$03,$09,$04,$85,$27,$68
            .byte   $29,$18,$90,$02,$69,$7F,$85,$26,$0A,$0A,$05,$26,$85,$26,$60,$A5
            .byte   $30,$18,$69,$03,$29,$0F,$85,$30,$0A,$0A,$0A,$0A,$05,$30,$85,$30
            .byte   $60,$4A,$08,$20,$47,$F8,$B1,$26,$28,$90,$04,$4A,$4A,$4A,$4A,$29
            .byte   $0F,$60,$A6,$3A,$A4,$3B,$20,$96,$FD,$20,$48,$F9,$A1,$3A,$A8,$4A
            .byte   $90,$09,$6A,$B0,$10,$C9,$A2,$F0,$0C,$29,$87,$4A,$AA,$BD,$62,$F9
            .byte   $20,$79,$F8,$D0,$04,$A0,$80,$A9,$00,$AA,$BD,$A6,$F9,$85,$2E,$AA
            .byte   $84,$2A,$A0,$10,$4C,$B4,$FB,$A9,$F8,$85,$37,$84,$36,$B1,$36,$91
            .byte   $36,$C8,$D0,$F9,$E6,$37,$D0,$F5,$BD,$80,$C0,$BD,$80,$C0,$60,$00
            .byte   $20,$82,$F8,$48,$B1,$3A,$20,$DA,$FD,$A2,$01,$20,$4A,$F9,$C4,$2F
            .byte   $C8,$90,$F1,$A2,$03,$C0,$04,$90,$F2,$68,$A8,$B9,$C0,$F9,$85,$2C
            .byte   $B9,$00,$FA,$85,$2D,$A9,$00,$A0,$05,$06,$2D,$26,$2C,$2A,$88,$D0
            .byte   $F8,$69,$BF,$20,$ED,$FD,$CA,$D0,$EC,$20,$48,$F9,$A4,$2F,$A2,$06
            .byte   $E0,$03,$F0,$1C,$06,$2E,$90,$0E,$BD,$B3,$F9,$20,$ED,$FD,$BD,$B9
            .byte   $F9,$F0,$03,$20,$ED,$FD,$CA,$D0,$E7,$60,$88,$30,$E7,$20,$DA,$FD
            .byte   $A5,$2E,$C9,$E8,$B1,$3A,$90,$F2,$20,$56,$F9,$AA,$E8,$D0,$01,$C8
            .byte   $98,$20,$DA,$FD,$8A,$4C,$DA,$FD,$A2,$03,$A9,$A0,$20,$ED,$FD,$CA
            .byte   $D0,$F8,$60,$38,$A5,$2F,$A4,$3B,$AA,$10,$01,$88,$65,$3A,$90,$01
            .byte   $C8,$60,$04,$20,$54,$30,$0D,$80,$04,$90,$03,$22,$54,$33,$0D,$80
            .byte   $04,$90,$04,$20,$54,$33,$0D,$80,$04,$90,$04,$20,$54,$3B,$0D,$80
            .byte   $04,$90,$00,$22,$44,$33,$0D,$C8,$44,$00,$11,$22,$44,$33,$0D,$C8
            .byte   $44,$A9,$01,$22,$44,$33,$0D,$80,$04,$90,$01,$22,$44,$33,$0D,$80
            .byte   $04,$90,$26,$31,$87,$9A,$00,$21,$81,$82,$00,$00,$59,$4D,$91,$92
            .byte   $86,$4A,$85,$9D,$AC,$A9,$AC,$A3,$A8,$A4,$D9,$00,$D8,$A4,$A4,$00
            .byte   $1C,$8A,$1C,$23,$5D,$8B,$1B,$A1,$9D,$8A,$1D,$23,$9D,$8B,$1D,$A1
            .byte   $00,$29,$19,$AE,$69,$A8,$19,$23,$24,$53,$1B,$23,$24,$53,$19,$A1
            .byte   $00,$1A,$5B,$5B,$A5,$69,$24,$24,$AE,$AE,$A8,$AD,$29,$00,$7C,$00
            .byte   $15,$9C,$6D,$9C,$A5,$69,$29,$53,$84,$13,$34,$11,$A5,$69,$23,$A0
;FA00        
            .byte   $D8,$62,$5A,$48,$26,$62,$94,$88,$54,$44,$C8,$54,$68,$44,$E8,$94
            .byte   $00,$B4,$08,$84,$74,$B4,$28,$6E,$74,$F4,$CC,$4A,$72,$F2,$A4,$8A
            .byte   $00,$AA,$A2,$A2,$74,$74,$74,$72,$44,$68,$B2,$32,$B2,$00,$22,$00
            .byte   $1A,$1A,$26,$26,$72,$72,$88,$C8,$C4,$CA,$26,$48,$44,$44,$A2,$C8
            .byte   $85,$45,$A5,$45,$4C,$FA,$C3,$8D,$06,$C0,$85,$45,$28,$20,$4C,$FF
            .byte   $68,$85,$3A,$68,$85,$3B,$6C,$F0,$03,$20,$82,$F8,$20,$DA,$FA,$4C
            .byte   $65,$FF,$D8,$8D,$06,$C0,$AC,$A0,$C7,$99,$AD,$C0,$20,$E0,$C7,$AD
            .byte   $58,$C0,$AD,$5A,$C0,$A0,$09,$20,$B4,$FB,$EA,$AD,$FF,$CF,$2C,$10
            .byte   $C0,$D8,$20,$3A,$FF,$AD,$F3,$03,$49,$A5,$CD,$F4,$03,$D0,$17,$AD
            .byte   $F2,$03,$D0,$0F,$A9,$E0,$CD,$F3,$03,$D0,$08,$A0,$03,$8C,$F2,$03
            .byte   $4C,$00,$E0,$6C,$F2,$03,$20,$60,$FB,$A2,$05,$BD,$FC,$FA,$9D,$EF
;patch to swap slot search order
;faB5 c8 -> c0    - start slot (- or + one)
;fabc c6 -> e6    - c6=dec e6=inc
;fac1 c0 -> c7    - last slot (- or + one)

;FAB0
;            .byte   $03,$CA,$D0,$F7,$A9,$C8,$86,$00,$85,$01,$A0,$05,$c6,$01,$A5,$01  ;slot 7 to 1
            .byte   $03,$CA,$D0,$F7,$A9,$C0,$86,$00,$85,$01,$A0,$05,$e6,$01,$A5,$01  ;slot 1 to 6

;FAC0
;            .byte   $C9,$C0,$F0,$D7,$8D,$F8,$07,$B1,$00,$D9,$01,$FB,$D0,$EC,$88,$88  ;slot 7 to 1
            .byte   $C9,$C7,$F0,$D7,$8D,$F8,$07,$B1,$00,$D9,$01,$FB,$D0,$EC,$88,$88  ;slot 1 to 6

            .byte   $10,$F5,$6C,$00,$00,$00,$00,$20,$8E,$FD,$A9,$45,$85,$40,$A9,$00
            .byte   $85,$41,$A2,$FB,$A9,$A0,$20,$ED,$FD,$BD,$1E,$FA,$20,$ED,$FD,$A9
            .byte   $BD,$20,$ED,$FD,$B5,$4A,$20,$DA,$FD,$E8,$30,$E8,$60,$59,$FA,$00
;FB00     
            .byte   $E0,$45,$20,$FF,$00,$FF,$03,$FF,$3C,$C1,$F0,$F0,$EC,$E5,$A0,$DD
            .byte   $DB,$C4,$C2,$C1,$FF,$C3,$FF,$FF,$FF,$C1,$D8,$D9,$D0,$D3,$AD,$70
            .byte   $C0,$A0,$00,$EA,$EA,$BD,$64,$C0,$10,$04,$C8,$D0,$F8,$88,$60,$A9
            .byte   $00,$85,$48,$AD,$56,$C0,$AD,$54,$C0,$AD,$51,$C0,$A9,$00,$F0,$0B
            .byte   $AD,$50,$C0,$AD,$53,$C0,$20,$36,$F8,$A9,$14,$85,$22,$A9,$00,$85
            .byte   $20,$A0,$0C,$D0,$5F,$A9,$18,$85,$23,$A9,$17,$85,$25,$4C,$22,$FC
            .byte   $20,$58,$FC,$A0,$09,$B9,$09,$FF,$99,$0E,$04,$88,$D0,$F7,$60,$AD
            .byte   $F3,$03,$49,$A5,$8D,$F4,$03,$60,$C9,$8D,$D0,$18,$AC,$00,$C0,$10
            .byte   $13,$C0,$93,$D0,$0F,$2C,$10,$C0,$AC,$00,$C0,$10,$FB,$C0,$83,$F0
            .byte   $03,$2C,$10,$C0,$4C,$FD,$FB,$38,$4C,$2C,$FC,$A8,$B9,$48,$FA,$20
            .byte   $97,$FB,$20,$21,$FD,$C9,$CE,$B0,$EE,$C9,$C9,$90,$EA,$C9,$CC,$F0
            .byte   $E6,$D0,$E8,$06,$2C,$15,$C0,$08,$8D,$07,$C0,$4C,$00,$C1,$00,$00
            .byte   $E0,$48,$4A,$29,$03,$09,$04,$85,$29,$68,$29,$18,$90,$02,$69,$7F
            .byte   $85,$28,$0A,$0A,$05,$28,$85,$28,$60,$C9,$87,$D0,$12,$A9,$40,$20
            .byte   $A8,$FC,$A0,$C0,$A9,$0C,$20,$A8,$FC,$AD,$30,$C0,$88,$D0,$F5,$60
            .byte   $A4,$24,$91,$28,$E6,$24,$A5,$24,$C5,$21,$B0,$66,$60,$C9,$A0,$B0
;fC00         
            .byte   $EF,$A8,$10,$EC,$C9,$8D,$F0,$5A,$C9,$8A,$F0,$5A,$C9,$88,$D0,$C9
            .byte   $C6,$24,$10,$E8,$A5,$21,$85,$24,$C6,$24,$A5,$22,$C5,$25,$B0,$DC
            .byte   $C6,$25,$A5,$25,$85,$28,$98,$A0,$04,$D0,$89,$00,$49,$C0,$F0,$28
            .byte   $69,$FD,$90,$C0,$F0,$DA,$69,$FD,$90,$2C,$F0,$DE,$69,$FD,$90,$5C
            .byte   $D0,$BA,$A0,$0A,$D0,$E3,$2C,$1F,$C0,$10,$04,$A0,$00,$F0,$0B,$98
            .byte   $48,$20,$78,$FB,$68,$A4,$35,$60,$A0,$05,$4C,$B4,$FB,$53,$5C,$5C
            .byte   $00,$00,$A9,$00,$85,$24,$E6,$25,$A5,$25,$C5,$23,$90,$B6,$C6,$25
            .byte   $A0,$06,$D0,$B5,$8D,$06,$C0,$6C,$FE,$03,$68,$8D,$F8,$07,$C9,$C1
            .byte   $90,$0D,$8D,$FF,$CF,$A0,$00,$A6,$01,$85,$01,$B1,$00,$86,$01,$8D
            .byte   $07,$C0,$4C,$00,$C5,$90,$02,$25,$32,$4C,$F7,$FD,$38,$90,$18,$84
            .byte   $2A,$A0,$07,$B0,$78,$C8,$D0,$75,$38,$48,$E9,$01,$D0,$FC,$68,$E9
            .byte   $01,$D0,$F6,$60,$E6,$42,$D0,$02,$E6,$43,$A5,$3C,$C5,$3E,$A5,$3D
            .byte   $E5,$3F,$E6,$3C,$D0,$02,$E6,$3D,$60,$8D,$07,$C0,$20,$67,$C5,$4C
            .byte   $C5,$FE,$8D,$06,$C0,$20,$4A,$F9,$A9,$DE,$20,$ED,$FD,$20,$3A,$FF
            .byte   $4C,$F0,$FC,$8D,$06,$C0,$20,$D0,$F8,$20,$53,$F9,$84,$3B,$85,$3A
            .byte   $A9,$A1,$85,$33,$20,$67,$FD,$8D,$07,$C0,$4C,$9C,$CF,$B9,$00,$02
;FD00
            .byte   $C8,$C9,$E1,$90,$06,$C9,$FB,$B0,$02,$29,$DF,$60,$A0,$0B,$D0,$03
            .byte   $4C,$18,$FD,$20,$B4,$FB,$EA,$EA,$6C,$38,$00,$A0,$03,$4C,$B4,$FB
            .byte   $EA,$20,$0C,$FD,$A0,$01,$D0,$F5,$4E,$F8,$07,$4C,$0C,$FD,$EA,$20
            .byte   $21,$FD,$20,$A5,$FB,$20,$28,$FD,$C9,$9B,$F0,$F3,$60,$A0,$0F,$20
            .byte   $B4,$FB,$A4,$24,$9D,$00,$02,$20,$ED,$FD,$EA,$EA,$EA,$BD,$00,$02
            .byte   $C9,$88,$F0,$1D,$C9,$98,$F0,$0A,$E0,$F8,$90,$03,$20,$3A,$FF,$E8
            .byte   $D0,$13,$A9,$DC,$20,$ED,$FD,$20,$8E,$FD,$A5,$33,$20,$ED,$FD,$A2
            .byte   $01,$8A,$F0,$F3,$CA,$20,$35,$FD,$C9,$95,$D0,$08,$B1,$28,$2C,$1F
            .byte   $C0,$30,$BA,$EA,$9D,$00,$02,$C9,$8D,$D0,$BC,$20,$9C,$FC,$A9,$8D
            .byte   $D0,$5B,$A4,$3D,$A6,$3C,$20,$8E,$FD,$20,$40,$F9,$A0,$00,$A9,$AD
            .byte   $4C,$ED,$FD,$A5,$3C,$09,$07,$85,$3E,$A5,$3D,$85,$3F,$A5,$3C,$29
            .byte   $07,$D0,$03,$20,$92,$FD,$A9,$A0,$20,$ED,$FD,$B1,$3C,$20,$DA,$FD
            .byte   $20,$BA,$FC,$90,$E8,$60,$4A,$90,$EA,$4A,$4A,$A5,$3E,$90,$02,$49
            .byte   $FF,$65,$3C,$48,$A9,$BD,$20,$ED,$FD,$68,$48,$4A,$4A,$4A,$4A,$20
            .byte   $E5,$FD,$68,$29,$0F,$09,$B0,$C9,$BA,$90,$02,$69,$06,$6C,$36,$00
            .byte   $48,$C9,$A0,$4C,$95,$FC,$48,$84,$35,$A8,$68,$4C,$46,$FC,$EA,$EA
;FE00
            .byte   $C6,$34,$F0,$9F,$CA,$D0,$16,$C9,$BA,$D0,$BB,$85,$31,$A5,$3E,$91
            .byte   $40,$E6,$40,$D0,$02,$E6,$41,$60,$A4,$34,$B9,$FF,$01,$85,$31,$60
            .byte   $A2,$01,$B5,$3E,$95,$42,$95,$44,$CA,$10,$F7,$60,$B1,$3C,$91,$42
            .byte   $20,$B4,$FC,$90,$F7,$60,$B1,$3C,$D1,$42,$F0,$1C,$20,$92,$FD,$B1
            .byte   $3C,$20,$DA,$FD,$A9,$A0,$20,$ED,$FD,$A9,$A8,$20,$ED,$FD,$B1,$42
            .byte   $20,$DA,$FD,$A9,$A9,$20,$ED,$FD,$20,$B4,$FC,$90,$D9,$60,$20,$75
            .byte   $FE,$A9,$14,$48,$20,$D0,$F8,$20,$53,$F9,$85,$3A,$84,$3B,$68,$38
            .byte   $E9,$01,$D0,$EF,$60,$8A,$F0,$07,$B5,$3C,$95,$3A,$CA,$10,$F9,$60
            .byte   $A0,$3F,$D0,$02,$A0,$FF,$84,$32,$60,$A9,$00,$85,$3E,$A2,$38,$A0
            .byte   $1B,$D0,$08,$A9,$00,$85,$3E,$A2,$36,$A0,$F0,$A5,$3E,$29,$0F,$F0
            .byte   $04,$09,$C0,$A0,$00,$94,$00,$95,$01,$A0,$0E,$4C,$B4,$FB,$EA,$78
            .byte   $4C,$00,$E0,$4C,$03,$E0,$20,$75,$FE,$20,$3F,$FF,$6C,$3A,$00,$4C
            .byte   $D7,$FA,$60,$EA,$60,$8D,$06,$C0,$60,$EA,$4C,$F8,$03,$A9,$40,$8D
            .byte   $07,$C0,$20,$AA,$C5,$F0,$2C,$A0,$01,$A5,$43,$F0,$04,$D1,$3C,$D0
            .byte   $0A,$88,$A5,$42,$D1,$3C,$D0,$03,$20,$92,$FD,$20,$BA,$FC,$90,$E7
            .byte   $60,$A0,$0D,$20,$B4,$FB,$20,$00,$FE,$68,$68,$D0,$6C,$8D,$07,$C0
;FF00
            .byte   $20,$D1,$C5,$8D,$06,$C0,$F0,$32,$D0,$23,$C1,$F0,$F0,$EC,$E5,$A0
            .byte   $AF,$AF,$E5,$20,$FD,$FC,$C9,$A0,$F0,$F9,$60,$B0,$6D,$C9,$A0,$D0
            .byte   $28,$B9,$00,$02,$A2,$07,$C9,$8D,$F0,$7D,$C8,$D0,$63,$A9,$C5,$20
            .byte   $ED,$FD,$A9,$D2,$20,$ED,$FD,$20,$ED,$FD,$A9,$87,$4C,$ED,$FD,$A5
            .byte   $48,$48,$A5,$45,$A6,$46,$A4,$47,$28,$60,$85,$45,$86,$46,$84,$47
            .byte   $08,$68,$85,$48,$BA,$86,$49,$D8,$60,$20,$84,$FE,$20,$2F,$FB,$20
            .byte   $93,$FE,$20,$89,$FE,$D8,$20,$3A,$FF,$A9,$AA,$85,$33,$20,$67,$FD
            .byte   $20,$C7,$FF,$20,$A7,$FF,$84,$34,$A0,$17,$88,$30,$E8,$D9,$CC,$FF
            .byte   $D0,$F8,$20,$BE,$FF,$A4,$34,$4C,$73,$FF,$A2,$03,$0A,$0A,$0A,$0A
            .byte   $0A,$26,$3E,$26,$3F,$CA,$10,$F8,$A5,$31,$D0,$06,$B5,$3F,$95,$3D
            .byte   $95,$41,$E8,$F0,$F3,$D0,$06,$A2,$00,$86,$3E,$86,$3F,$20,$FD,$FC
            .byte   $EA,$49,$B0,$C9,$0A,$90,$D3,$69,$88,$C9,$FA,$4C,$1B,$FF,$A9,$FE
            .byte   $48,$B9,$E3,$FF,$48,$A5,$31,$A0,$00,$84,$31,$60,$BC,$B2,$BE,$9A
            .byte   $EF,$C4,$EC,$A9,$BB,$A6,$A4,$06,$95,$07,$02,$05,$F0,$00,$EB,$93
            .byte   $A7,$C6,$99,$B2,$C9,$BE,$F0,$35,$8C,$D6,$96,$AF,$17,$17,$2B,$1F
            .byte   $83,$7F,$5D,$CC,$B5,$FC,$17,$17,$F5,$03,$62,$FA,$62,$FA,$FA,$C3
 

CODE9000:   

;font
            .byte   $1C,$22,$2A,$3A,$1A,$02,$3C,$00,$08,$14,$22,$22,$3E,$22,$22,$00
            .byte   $1E,$22,$22,$1E,$22,$22,$1E,$00,$1C,$22,$02,$02,$02,$22,$1C,$00
            .byte   $1E,$22,$22,$22,$22,$22,$1E,$00,$3E,$02,$02,$1E,$02,$02,$3E,$00
            .byte   $3E,$02,$02,$1E,$02,$02,$02,$00,$3C,$02,$02,$02,$32,$22,$3C,$00
            .byte   $22,$22,$22,$3E,$22,$22,$22,$00,$1C,$08,$08,$08,$08,$08,$1C,$00
            .byte   $20,$20,$20,$20,$20,$22,$1C,$00,$22,$12,$0A,$06,$0A,$12,$22,$00
            .byte   $02,$02,$02,$02,$02,$02,$3E,$00,$22,$36,$2A,$2A,$22,$22,$22,$00
            .byte   $22,$22,$26,$2A,$32,$22,$22,$00,$1C,$22,$22,$22,$22,$22,$1C,$00
            .byte   $1E,$22,$22,$1E,$02,$02,$02,$00,$1C,$22,$22,$22,$2A,$12,$2C,$00
            .byte   $1E,$22,$22,$1E,$0A,$12,$22,$00,$1C,$22,$02,$1C,$20,$22,$1C,$00
            .byte   $3E,$08,$08,$08,$08,$08,$08,$00,$22,$22,$22,$22,$22,$22,$1C,$00
            .byte   $22,$22,$22,$22,$22,$14,$08,$00,$22,$22,$22,$2A,$2A,$36,$22,$00
            .byte   $22,$22,$14,$08,$14,$22,$22,$00,$22,$22,$14,$08,$08,$08,$08,$00
            .byte   $3E,$20,$10,$08,$04,$02,$3E,$00,$3E,$06,$06,$06,$06,$06,$3E,$00
            .byte   $00,$02,$04,$08,$10,$20,$00,$00,$3E,$30,$30,$30,$30,$30,$3E,$00
            .byte   $00,$00,$08,$14,$22,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$7F
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$08,$08,$08,$08,$08,$00,$08,$00
            .byte   $14,$14,$14,$00,$00,$00,$00,$00,$14,$14,$3E,$14,$3E,$14,$14,$00
            .byte   $08,$3C,$0A,$1C,$28,$1E,$08,$00,$06,$26,$10,$08,$04,$32,$30,$00
            .byte   $04,$0A,$0A,$04,$2A,$12,$2C,$00,$08,$08,$08,$00,$00,$00,$00,$00
            .byte   $08,$04,$02,$02,$02,$04,$08,$00,$08,$10,$20,$20,$20,$10,$08,$00
            .byte   $08,$2A,$1C,$08,$1C,$2A,$08,$00,$00,$08,$08,$3E,$08,$08,$00,$00
            .byte   $00,$00,$00,$00,$08,$08,$04,$00,$00,$00,$00,$3E,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$08,$00,$00,$20,$10,$08,$04,$02,$00,$00
            .byte   $1C,$22,$32,$2A,$26,$22,$1C,$00,$08,$0C,$08,$08,$08,$08,$1C,$00
            .byte   $1C,$22,$20,$18,$04,$02,$3E,$00,$3E,$20,$10,$18,$20,$22,$1C,$00
            .byte   $10,$18,$14,$12,$3E,$10,$10,$00,$3E,$02,$1E,$20,$20,$22,$1C,$00
            .byte   $38,$04,$02,$1E,$22,$22,$1C,$00,$3E,$20,$10,$08,$04,$04,$04,$00
            .byte   $1C,$22,$22,$1C,$22,$22,$1C,$00,$1C,$22,$22,$3C,$20,$10,$0E,$00
            .byte   $00,$00,$08,$00,$08,$00,$00,$00,$00,$00,$08,$00,$08,$08,$04,$00
            .byte   $10,$08,$04,$02,$04,$08,$10,$00,$00,$00,$3E,$00,$3E,$00,$00,$00
            .byte   $04,$08,$10,$20,$10,$08,$04,$00,$1C,$22,$10,$08,$08,$00,$08,$00
            .byte   $1C,$22,$2A,$3A,$1A,$02,$3C,$00,$08,$14,$22,$22,$3E,$22,$22,$00
            .byte   $1E,$22,$22,$1E,$22,$22,$1E,$00,$1C,$22,$02,$02,$02,$22,$1C,$00
            .byte   $1E,$22,$22,$22,$22,$22,$1E,$00,$3E,$02,$02,$1E,$02,$02,$3E,$00
            .byte   $3E,$02,$02,$1E,$02,$02,$02,$00,$3C,$02,$02,$02,$32,$22,$3C,$00
            .byte   $22,$22,$22,$3E,$22,$22,$22,$00,$1C,$08,$08,$08,$08,$08,$1C,$00
            .byte   $20,$20,$20,$20,$20,$22,$1C,$00,$22,$12,$0A,$06,$0A,$12,$22,$00
            .byte   $02,$02,$02,$02,$02,$02,$3E,$00,$22,$36,$2A,$2A,$22,$22,$22,$00
            .byte   $22,$22,$26,$2A,$32,$22,$22,$00,$1C,$22,$22,$22,$22,$22,$1C,$00
            .byte   $1E,$22,$22,$1E,$02,$02,$02,$00,$1C,$22,$22,$22,$2A,$12,$2C,$00
            .byte   $1E,$22,$22,$1E,$0A,$12,$22,$00,$1C,$22,$02,$1C,$20,$22,$1C,$00
            .byte   $3E,$08,$08,$08,$08,$08,$08,$00,$22,$22,$22,$22,$22,$22,$1C,$00
            .byte   $22,$22,$22,$22,$22,$14,$08,$00,$22,$22,$22,$2A,$2A,$36,$22,$00
            .byte   $22,$22,$14,$08,$14,$22,$22,$00,$22,$22,$14,$08,$08,$08,$08,$00
            .byte   $3E,$20,$10,$08,$04,$02,$3E,$00,$3E,$06,$06,$06,$06,$06,$3E,$00
            .byte   $00,$02,$04,$08,$10,$20,$00,$00,$3E,$30,$30,$30,$30,$30,$3E,$00
            .byte   $00,$00,$08,$14,$22,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$7F
            .byte   $04,$08,$10,$00,$00,$00,$00,$00,$00,$00,$1C,$20,$3C,$22,$3C,$00
            .byte   $02,$02,$1E,$22,$22,$22,$1E,$00,$00,$00,$3C,$02,$02,$02,$3C,$00
            .byte   $20,$20,$3C,$22,$22,$22,$3C,$00,$00,$00,$1C,$22,$3E,$02,$3C,$00
            .byte   $18,$24,$04,$1E,$04,$04,$04,$00,$00,$00,$1C,$22,$22,$3C,$20,$1C
            .byte   $02,$02,$1E,$22,$22,$22,$22,$00,$08,$00,$0C,$08,$08,$08,$1C,$00
            .byte   $10,$00,$18,$10,$10,$10,$12,$0C,$02,$02,$22,$12,$0E,$12,$22,$00
            .byte   $0C,$08,$08,$08,$08,$08,$1C,$00,$00,$00,$36,$2A,$2A,$2A,$22,$00
            .byte   $00,$00,$1E,$22,$22,$22,$22,$00,$00,$00,$1C,$22,$22,$22,$1C,$00
            .byte   $00,$00,$1E,$22,$22,$1E,$02,$02,$00,$00,$3C,$22,$22,$3C,$20,$20
            .byte   $00,$00,$3A,$06,$02,$02,$02,$00,$00,$00,$3C,$02,$1C,$20,$1E,$00
            .byte   $04,$04,$1E,$04,$04,$24,$18,$00,$00,$00,$22,$22,$22,$32,$2C,$00
            .byte   $00,$00,$22,$22,$22,$14,$08,$00,$00,$00,$22,$22,$2A,$2A,$36,$00
            .byte   $00,$00,$22,$14,$08,$14,$22,$00,$00,$00,$22,$22,$22,$3C,$20,$1C
            .byte   $00,$00,$3E,$10,$08,$04,$3E,$00,$38,$0C,$0C,$06,$0C,$0C,$38,$00
            .byte   $08,$08,$08,$08,$08,$08,$08,$08,$0E,$18,$18,$30,$18,$18,$0E,$00
            .byte   $2C,$1A,$00,$00,$00,$00,$00,$00,$00,$2A,$14,$2A,$14,$2A,$00,$00

;??
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$85,$46,$86,$47,$AD,$C3,$C0,$AD,$00,$C4,$AD,$01,$C4
            .byte   $AD,$02,$C4,$AD,$03,$C4,$38,$AD,$82,$C0,$AD,$C2,$C0,$29,$01,$D0
            .byte   $1F,$A9,$FF,$8D,$0C,$94,$C5,$47,$D0,$09,$A5,$46,$C9,$FE,$90,$03
            .byte   $18,$90,$0D,$EE,$0C,$94,$A5,$46,$CD,$A8,$C7,$A5,$47,$ED,$A9,$C7
            .byte   $60,$38,$60,$20,$13,$94,$90,$03,$4C,$E7,$94,$AD,$00,$C4,$A9,$00
            .byte   $8D,$12,$94,$8D,$06,$94,$8D,$07,$94,$A5,$46,$8D,$02,$94,$49,$FF
            .byte   $8D,$05,$94,$A5,$47,$8D,$01,$94,$49,$FF,$8D,$04,$94,$A9,$00,$8D
            .byte   $00,$94,$A9,$FF,$8D,$03,$94,$A9,$02,$8D,$0A,$94,$A9,$00,$8D,$0B
            .byte   $94,$8D,$0E,$94,$A9,$02,$8D,$08,$94,$A9,$01,$8D,$09,$94,$A9,$00
            .byte   $8D,$0F,$94,$20,$D0,$96,$B0,$48,$D0,$EA,$20,$5A,$96,$B0,$3C,$A9
            .byte   $04,$8D,$09,$94,$20,$D0,$96,$B0,$37,$D0,$D9,$20,$05,$95,$B0,$2B
            .byte   $A9,$06,$8D,$09,$94,$A9,$FF,$8D,$0F,$94,$20,$D0,$96,$B0,$21,$D0
            .byte   $C3,$20,$41,$96,$B0,$15,$30,$18,$29,$41,$D0,$26,$AD,$07,$94,$29
            .byte   $48,$D0,$1F,$20,$DB,$95,$18,$AC,$A0,$C7,$60,$CE,$0A,$94,$10,$9C
            .byte   $AD,$12,$94,$10,$03,$4C,$CF,$95,$20,$22,$96,$38,$6E,$12,$94,$4C
            .byte   $94,$94,$4C,$D5,$95,$AD,$01,$C4,$AD,$03,$C4,$A2,$C8,$20,$10,$97
            .byte   $A2,$C9,$20,$10,$97,$A0,$00,$B9,$00,$94,$8D,$C0,$C0,$C8,$C0,$06
            .byte   $D0,$F5,$20,$88,$96,$4C,$1B,$96,$20,$13,$94,$B0,$BA,$AD,$00,$C4
            .byte   $A9,$00,$8D,$12,$94,$8D,$06,$94,$8D,$07,$94,$A5,$46,$8D,$02,$94
            .byte   $49,$FF,$8D,$05,$94,$A5,$47,$8D,$01,$94,$49,$FF,$8D,$04,$94,$A9
            .byte   $00,$8D,$00,$94,$A9,$FF,$8D,$03,$94,$A9,$02,$8D,$0A,$94,$A9,$00
            .byte   $8D,$0B,$94,$A9,$0A,$8D,$0D,$94,$A9,$03,$8D,$0E,$94,$A9,$00,$8D
            .byte   $0F,$94,$A9,$00,$8D,$08,$94,$A9,$01,$8D,$09,$94,$20,$D0,$96,$B0
            .byte   $3F,$D0,$EA,$20,$5A,$96,$B0,$33,$A9,$02,$8D,$09,$94,$A9,$FF,$8D
            .byte   $0F,$94,$20,$D0,$96,$B0,$29,$D0,$D4,$20,$41,$96,$B0,$1D,$30,$20
            .byte   $20,$FB,$95,$B0,$16,$AD,$06,$94,$29,$01,$D0,$29,$AD,$07,$94,$29
            .byte   $58,$D0,$22,$20,$DB,$95,$18,$AC,$A0,$C7,$60,$CE,$0A,$94,$10,$9E
            .byte   $AD,$12,$94,$30,$0A,$20,$22,$96,$38,$6E,$12,$94,$4C,$6D,$95,$20
            .byte   $1B,$96,$AD,$00,$C4,$20,$DB,$95,$38,$B0,$DC,$A9,$01,$8D,$09,$94
            .byte   $20,$D0,$96,$A9,$FF,$8D,$08,$94,$20,$5A,$96,$20,$1B,$96,$AD,$04
            .byte   $C4,$20,$1B,$96,$AD,$04,$C4,$AD,$00,$C4,$60,$20,$1B,$96,$A2,$C8
            .byte   $20,$10,$97,$A2,$C9,$20,$10,$97,$A0,$00,$AD,$C1,$C0,$D9,$00,$94
            .byte   $38,$D0,$08,$C8,$C0,$06,$D0,$F2,$20,$88,$96,$AD,$05,$C4,$AD,$07
            .byte   $C4,$60,$AD,$0C,$C4,$A0,$27,$88,$D0,$FD,$AD,$08,$C4,$60,$8D,$C0
            .byte   $C0,$AD,$03,$C4,$AD,$00,$C4,$20,$95,$96,$AD,$05,$C4,$AD,$07,$C4
            .byte   $60,$AD,$C1,$C0,$8D,$06,$94,$AD,$C1,$C0,$8D,$07,$94,$AD,$C1,$C0
            .byte   $AD,$C1,$C0,$20,$88,$96,$AD,$06,$94,$60,$AD,$01,$C4,$AD,$03,$C4
            .byte   $AD,$08,$94,$8D,$C0,$C0,$AD,$0C,$94,$8D,$C0,$C0,$A5,$47,$8D,$C0
            .byte   $C0,$A5,$46,$8D,$C0,$C0,$AD,$0D,$94,$8D,$C0,$C0,$AD,$0E,$94,$8D
            .byte   $C0,$C0,$AD,$05,$C4,$AD,$07,$C4,$AD,$C2,$C0,$8D,$C3,$C0,$0A,$0A
            .byte   $90,$02,$A9,$01,$60,$A0,$01,$AD,$0F,$94,$F0,$02,$A0,$10,$8C,$10
            .byte   $94,$A0,$00,$8C,$11,$94,$18,$AD,$C2,$C0,$30,$0E,$88,$D0,$F8,$CE
            .byte   $11,$94,$D0,$F3,$CE,$10,$94,$D0,$EE,$38,$60,$A0,$00,$8C,$11,$94
            .byte   $18,$AD,$C2,$C0,$10,$09,$88,$D0,$F8,$CE,$11,$94,$D0,$F3,$38,$60
            .byte   $20,$95,$96,$B0,$3A,$AD,$05,$C4,$AD,$07,$C4,$AD,$04,$C4,$20,$BB
            .byte   $96,$B0,$2C,$AC,$C1,$C0,$CC,$09,$94,$F0,$1B,$AC,$09,$94,$C8,$D0
            .byte   $05,$A9,$AA,$4C,$2E,$96,$A9,$AA,$20,$2E,$96,$B0,$12,$EE,$0B,$94
            .byte   $AC,$0B,$94,$C0,$02,$60,$A9,$55,$20,$2E,$96,$B0,$02,$A9,$00,$60
            .byte   $AD,$DF,$FF,$48,$29,$A7,$09,$03,$8D,$DF,$FF,$8E,$D0,$FF,$A9,$FF
            .byte   $38,$20,$00,$F8,$68,$8D,$DF,$FF,$A9,$00,$8D,$D0,$FF,$60,$4E,$50
            .byte   $D5,$54,$00,$00,$00,$86,$C8,$01,$00,$00,$00,$00,$0A,$54,$45,$D3
            .byte   $54,$5F,$D3,$45,$52,$49,$41,$E5,$C8,$01,$00,$00,$00,$00,$0A,$50
            .byte   $45,$CE,$54,$52,$D9,$00,$00,$00,$00,$2D,$C9,$01,$00,$00,$00,$00
            .byte   $0A,$43,$48,$C5,$43,$4B,$DF,$54,$45,$52,$4D,$36,$C9,$01,$00,$00
            .byte   $00,$00,$0A,$52,$45,$D3,$54,$4F,$D2,$45,$00,$00,$00,$99,$C9,$01
            .byte   $C1,$D0,$D0,$CC,$C5,$A0,$AF,$AF,$AF,$A0,$CF,$D0,$C5,$D2,$C1,$D4
            .byte   $C9,$CF,$CE,$00,$A0,$A0,$A0,$B2,$A9,$A0,$A0,$A0,$C3,$C8,$C1,$CE
            .byte   $C7,$C5,$A0,$AF,$AF,$E5,$A0,$C5,$CD,$D5,$CC,$C1,$D4,$C9,$CF,$CE
            .byte   $00,$A0,$A0,$A0,$B3,$A9,$A0,$A0,$A0,$D3,$D4,$C1,$D2,$D4,$A0,$C1
            .byte   $D0,$D0,$CC,$C5,$A0,$AF,$AF,$E5,$A0,$C5,$CD,$D5,$CC,$C1,$D4,$C9
            .byte   $CF,$CE,$00,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$D3
            .byte   $C5,$CC,$C5,$C3,$D4,$A0,$C2,$D9,$A0,$CE,$D5,$CD,$C2,$C5,$D2,$00
            .byte   $A9,$00,$A8,$BE,$0F,$10,$30,$16,$48,$B9,$10,$10,$BE,$11,$10,$48
            .byte   $B9,$0F,$10,$A8,$68,$20,$48,$1F,$68,$18,$69,$03,$D0,$E4,$60,$0A
            .byte   $28,$10,$0E,$43,$10,$14,$62,$10,$18,$7B,$10,$1C,$9B,$10,$20,$B8
            .byte   $10,$26,$D2,$10,$28,$EF,$10,$FF,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$D4
            .byte   $C9,$D4,$C1,$CE,$A0,$A0,$AF,$AF,$AF,$A0,$F0,$EC,$F5,$F3,$A0,$AF
            .byte   $AF,$E5,$00,$A0,$A0,$C1,$D0,$D0,$CC,$C5,$A0,$AF,$AF,$E5,$A0,$C5
            .byte   $CD,$D5,$CC,$C1,$D4,$C9,$CF,$CE,$A0,$D3,$C5,$D4,$D4,$C9,$CE,$C7
            .byte   $D3,$00,$A0,$A0,$A0,$B1,$A9,$A0,$A0,$A0,$D0,$F2,$E9,$EE,$F4,$E5
            .byte   $F2,$A0,$D3,$E5,$F4,$F4,$E9,$EE,$E7,$F3,$00,$A0,$A0,$A0,$B2,$A9
            .byte   $A0,$A0,$A0,$C3,$EF,$ED,$ED,$F5,$EE,$E9,$E3,$E1,$F4,$E9,$EF,$EE
            .byte   $F3,$A0,$D3,$E5,$F4,$F4,$E9,$EE,$E7,$F3,$00,$A0,$A0,$A0,$B3,$A9
            .byte   $A0,$A0,$A0,$C4,$EF,$F5,$E2,$EC,$E5,$A0,$C8,$E9,$AD,$D2,$E5,$F3
            .byte   $A0,$E9,$F3,$A0,$CF,$EE,$A0,$00,$A0,$A0,$A0,$B4,$A9,$A0,$A0,$A0
            .byte   $D3,$E1,$F6,$E5,$A0,$CE,$E5,$F7,$A0,$D3,$E5,$F4,$F4,$E9,$EE,$E7
            .byte   $F3,$00,$A0,$A0,$A0,$A0,$A0,$A0,$C5,$CE,$D4,$C5,$D2,$A0,$C3,$C8
            .byte   $CF,$C9,$C3,$C5,$A0,$C2,$D9,$A0,$CE,$D5,$CD,$C2,$C5,$D2,$00,$A0
            .byte   $A0,$A0,$A0,$BC,$D2,$C5,$D4,$D5,$D2,$CE,$BE,$A0,$C6,$CF,$D2,$A0
            .byte   $D0,$D2,$C5,$D6,$C9,$CF,$D5,$D3,$A0,$CD,$C5,$CE,$D5,$00,$A9,$00
            .byte   $A8,$BE,$2D,$11,$30,$16,$48,$B9,$2E,$11,$BE,$2F,$11,$48,$B9,$2D
            .byte   $11,$A8,$68,$20,$48,$1F,$68,$18,$69,$03,$D0,$E4,$60,$02,$52,$11
            .byte   $06,$6D,$11,$0C,$8B,$11,$10,$A7,$11,$14,$BC,$11,$18,$DA,$11,$1C
            .byte   $EE,$11,$20,$09,$12,$24,$24,$12,$28,$41,$12,$2C,$5C,$12,$2E,$7C
            .byte   $12,$FF,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$D4,$C9,$D4,$C1,$CE,$A0,$A0
            .byte   $AF,$AF,$AF,$A0,$F0,$EC,$F5,$F3,$A0,$AF,$AF,$E5,$00,$A0,$A0,$A0
            .byte   $A0,$A0,$A0,$D3,$CC,$CF,$D4,$A0,$B1,$A0,$D0,$D2,$C9,$CE,$D4,$C5
            .byte   $D2,$A0,$D3,$C5,$D4,$D4,$C9,$CE,$C7,$D3,$00,$A0,$A0,$A0,$B1,$A9
            .byte   $A0,$A0,$A0,$C2,$E1,$F5,$E4,$A0,$F2,$E1,$F4,$E5,$A0,$E9,$F3,$A0
            .byte   $A0,$B1,$B9,$B2,$B0,$B0,$00,$A0,$A0,$A0,$B2,$A9,$A0,$A0,$A0,$D3
            .byte   $D0,$C1,$C3,$C5,$A0,$D0,$E1,$F2,$E9,$F4,$F9,$00,$A0,$A0,$A0,$B3
            .byte   $A9,$A0,$A0,$A0,$C4,$E1,$F4,$E1,$A0,$C6,$EF,$F2,$ED,$E1,$F4,$A0
            .byte   $E9,$F3,$A0,$B8,$A0,$C2,$E9,$F4,$F3,$00,$A0,$A0,$A0,$B4,$A9,$A0
            .byte   $A0,$A0,$B2,$A0,$D3,$F4,$EF,$F0,$A0,$C2,$E9,$F4,$F3,$00,$A0,$A0
            .byte   $A0,$B5,$A9,$A0,$A0,$A0,$CC,$E9,$EE,$E5,$A0,$CC,$E5,$EE,$E7,$F4
            .byte   $E8,$A0,$E9,$F3,$A0,$CF,$C6,$C6,$00,$A0,$A0,$A0,$B6,$A9,$A0,$A0
            .byte   $A0,$D0,$E1,$E7,$E5,$A0,$CC,$E5,$EE,$E7,$F4,$E8,$A0,$E9,$F3,$A0
            .byte   $CF,$C6,$C6,$00,$A0,$A0,$A0,$B7,$A9,$A0,$A0,$A0,$CC,$E9,$EE,$E5
            .byte   $A0,$C6,$E5,$E5,$E4,$F3,$A0,$E1,$F2,$E5,$A0,$C1,$C4,$C4,$C5,$C4
            .byte   $00,$A0,$A0,$A0,$B8,$A9,$A0,$A0,$A0,$C8,$C9,$AD,$C2,$C9,$D4,$A0
            .byte   $E9,$F3,$A0,$C4,$C9,$D3,$C1,$C2,$CC,$C5,$C4,$00,$A0,$A0,$A0,$A0
            .byte   $A0,$A0,$A0,$A0,$D4,$CF,$C7,$C7,$CC,$C5,$A0,$C3,$C8,$CF,$C9,$C3
            .byte   $C5,$A0,$C2,$D9,$A0,$CE,$D5,$CD,$C2,$C5,$D2,$00,$A0,$A0,$A0,$A0
            .byte   $A0,$A0,$A0,$BC,$D2,$C5,$D4,$D5,$D2,$CE,$BE,$A0,$C6,$CF,$D2,$A0
            .byte   $D0,$D2,$C5,$D6,$C9,$CF,$D5,$D3,$A0,$CD,$C5,$CE,$D5,$00,$20,$29
            .byte   $1F,$AD,$A6,$C7,$48,$29,$0F,$0A,$0A,$0A,$AA,$A0,$00,$BD,$6F,$1C
            .byte   $99,$A0,$11,$E8,$C8,$C0,$06,$D0,$F4,$68,$48,$29,$60,$4A,$4A,$4A
            .byte   $4A,$4A,$85,$00,$38,$A9,$B8,$E5,$00,$8D,$D3,$11,$68,$30,$0C,$A9
            .byte   $B1,$8D,$E2,$11,$A9,$A0,$8D,$EC,$11,$D0,$0A,$A9,$B2,$8D,$E2,$11
            .byte   $A9,$F3,$8D,$EC,$11,$AD,$A7,$C7,$A2,$20,$29,$E0,$F0,$06,$4A,$4A
            .byte   $4A,$29,$18,$AA,$A0,$00,$BD,$EF,$1C,$99,$AF,$11,$E8,$C8,$C0,$05
            .byte   $D0,$F4,$AD,$A3,$C7,$D0,$0F,$A9,$CF,$8D,$05,$12,$A9,$C6,$8D,$06
            .byte   $12,$8D,$07,$12,$D0,$0C,$20,$19,$1B,$8D,$05,$12,$8E,$06,$12,$8C
            .byte   $07,$12,$AD,$A4,$C7,$D0,$0F,$A9,$CF,$8D,$20,$12,$A9,$C6,$8D,$21
            .byte   $12,$8D,$22,$12,$D0,$0C,$20,$19,$1B,$8D,$20,$12,$8E,$21,$12,$8C
            .byte   $22,$12,$AD,$A5,$C7,$48,$29,$08,$D0,$17,$A9,$CF,$8D,$3B,$12,$A9
            .byte   $C6,$8D,$3C,$12,$8D,$3D,$12,$A9,$A0,$8D,$3E,$12,$8D,$3F,$12,$D0
            .byte   $15,$A9,$C1,$8D,$3B,$12,$A9,$C4,$8D,$3C,$12,$8D,$3D,$12,$8D,$3F
            .byte   $12,$A9,$C5,$8D,$3E,$12,$68,$29,$04,$D0,$11,$A9,$C4,$8D,$53,$12
            .byte   $A9,$C9,$8D,$54,$12,$A9,$D3,$8D,$55,$12,$D0,$0F,$A9,$A0,$8D,$53
            .byte   $12,$A9,$C5,$8D,$54,$12,$A9,$CE,$8D,$55,$12,$20,$0E,$11,$AD,$00
            .byte   $C0,$10,$FB,$8D,$10,$C0,$C9,$B1,$F0,$35,$C9,$B2,$F0,$37,$C9,$B3
            .byte   $F0,$39,$C9,$B4,$F0,$3B,$C9,$B5,$F0,$0D,$C9,$B6,$F0,$0F,$C9,$B7
            .byte   $F0,$11,$C9,$B8,$F0,$13,$60,$20,$00,$1A,$4C,$9E,$12,$20,$07,$1A
            .byte   $4C,$9E,$12,$20,$0E,$1A,$4C,$9E,$12,$20,$17,$1A,$4C,$9E,$12,$20
            .byte   $51,$1A,$4C,$9E,$12,$20,$34,$1A,$4C,$9E,$12,$20,$20,$1A,$4C,$9E
            .byte   $12,$AD,$A6,$C7,$18,$69,$80,$8D,$A6,$C7,$4C,$9E,$12,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

;loads into $a000, seems mapped to alternate c000 page in //e emulation
;
            .byte   $85,$46,$86,$47,$AD,$82,$C0,$20,$D3,$C5,$A5,$46,$CD,$AA,$C7,$A5
            .byte   $47,$85,$3F,$ED,$AB,$C7,$B0,$28,$A5,$46,$0A,$26,$3F,$48,$0A,$26
            .byte   $3F,$A5,$3F,$18,$69,$01,$29,$07,$8D,$EF,$FF,$68,$29,$7E,$18,$69
            .byte   $20,$85,$3F,$09,$01,$85,$3D,$A9,$00,$85,$3E,$85,$3C,$A8,$18,$60
            .byte   $38,$60,$20,$00,$A0,$B0,$12,$B9,$00,$C8,$91,$3E,$B9,$00,$C9,$91
            .byte   $3C,$C8,$D0,$F3,$A9,$00,$8D,$EF,$FF,$60,$20,$00,$A0,$B0,$FA,$B1
            .byte   $3E,$99,$00,$C8,$B1,$3C,$99,$00,$C9,$C8,$D0,$F3,$A9,$00,$8D,$EF
            .byte   $FF,$20,$CD,$C5,$AC,$A0,$C7,$60,$00,$00,$AC,$00,$00,$60,$00,$00
            .byte   $00,$94,$A9,$FF,$8D,$03,$94,$A9,$02,$8D,$0A,$94,$A9,$00,$8D,$0B
            .byte   $94,$8D,$0E,$94,$A9,$02,$8D,$08,$94,$A9,$01,$8D,$09,$94,$A9,$00
            .byte   $8D,$0F,$94,$20,$D0,$96,$B0,$48,$D0,$EA,$20,$5A,$96,$B0,$3C,$A9
            .byte   $04,$8D,$09,$94,$20,$D0,$96,$B0,$37,$D0,$D9,$20,$05,$95,$B0,$2B
            .byte   $A9,$06,$8D,$09,$94,$A9,$FF,$8D,$0F,$94,$20,$D0,$96,$B0,$21,$D0
            .byte   $C3,$20,$41,$96,$B0,$15,$30,$18,$29,$41,$D0,$26,$AD,$07,$94,$29
            .byte   $48,$D0,$1F,$20,$DB,$95,$18,$AC,$A0,$C7,$60,$CE,$0A,$94,$10,$9C
            .byte   $AD,$12,$94,$10,$03,$4C,$CF,$95,$20,$22,$96,$38,$6E,$12,$94,$4C
;c100 rom
            .byte   $4C,$13,$C2,$A4,$24,$A5,$25,$48,$20,$03,$CE,$20,$F4,$C1,$A0,$00
            .byte   $68,$69,$00,$C5,$23,$90,$F0,$B0,$34,$A5,$22,$85,$25,$A0,$00,$84
            .byte   $24,$F0,$E4,$A5,$22,$48,$20,$03,$CE,$A5,$28,$85,$2A,$A5,$29,$85
            .byte   $2B,$A4,$21,$88,$68,$69,$01,$C5,$23,$B0,$0D,$48,$20,$03,$CE,$B1
            .byte   $28,$91,$2A,$88,$10,$F9,$30,$E1,$A0,$00,$20,$F4,$C1,$A5,$25,$4C
            .byte   $03,$CE,$A9,$28,$85,$21,$A9,$18,$85,$23,$A9,$17,$85,$25,$D0,$EF
            .byte   $A4,$2A,$4C,$F4,$C1,$4C,$EB,$CB,$4C,$9A,$CC,$A4,$2A,$4C,$9D,$CC
            .byte   $4C,$74,$CC,$4C,$A0,$C2,$4C,$B0,$C2,$4C,$F2,$C2,$20,$90,$CC,$AD
            .byte   $7B,$05,$85,$24,$8D,$7B,$04,$4C,$FE,$CD,$B4,$00,$F0,$0F,$C0,$1B
            .byte   $F0,$0E,$20,$80,$CD,$B4,$00,$F0,$04,$A9,$FD,$95,$01,$B5,$01,$60
            .byte   $A5,$37,$C9,$C3,$D0,$F3,$4C,$32,$C8,$A4,$24,$B1,$28,$48,$29,$3F
            .byte   $09,$40,$91,$28,$68,$60,$A8,$A5,$28,$20,$BA,$CA,$90,$4C,$20,$14
            .byte   $CE,$A0,$03,$D9,$EE,$C2,$D0,$03,$B9,$A4,$C9,$88,$10,$F5,$30,$3A
            .byte   $20,$70,$C8,$4C,$0A,$C2,$8A,$29,$03,$85,$2F,$A5,$2A,$29,$8F,$4C
            .byte   $71,$CA,$20,$F0,$FC,$8A,$85,$34,$60,$AC,$7B,$05,$20,$44,$CE,$09
            .byte   $80,$60,$A4,$24,$A9,$A0,$2C,$1E,$C0,$10,$06,$24,$32,$30,$02,$A9
;c200
            .byte   $20,$4C,$A8,$CC,$A8,$A5,$28,$20,$03,$CE,$28,$30,$03,$4C,$C5,$FE
            .byte   $4C,$C8,$FE,$88,$30,$BA,$88,$30,$A5,$88,$30,$9A,$88,$30,$3D,$88
            .byte   $30,$E2,$A9,$C2,$48,$A9,$09,$48,$AD,$FB,$04,$29,$D6,$D0,$0D,$98
            .byte   $18,$69,$0C,$48,$20,$50,$C8,$20,$FE,$CD,$68,$A8,$A9,$C1,$48,$B9
            .byte   $44,$C2,$48,$60,$18,$22,$F1,$5F,$75,$02,$A8,$51,$E1,$94,$E8,$D5
            .byte   $7B,$64,$67,$6A,$75,$6F,$78,$72,$E1,$89,$E8,$D5,$2C,$1F,$C0,$10
            .byte   $06,$20,$74,$C8,$4C,$0A,$C2,$A8,$8A,$48,$98,$48,$48,$68,$C9,$FF
            .byte   $F0,$04,$A9,$FF,$D0,$02,$68,$48,$48,$A4,$24,$91,$28,$E6,$4E,$D0
            .byte   $0A,$A5,$4F,$E6,$4F,$45,$4F,$29,$40,$D0,$E2,$AD,$00,$C0,$10,$ED
            .byte   $68,$68,$A4,$24,$91,$28,$68,$AA,$AD,$00,$C0,$8D,$10,$C0,$30,$C4
            .byte   $20,$52,$C1,$2C,$1F,$C0,$10,$02,$06,$21,$A5,$25,$8D,$FB,$05,$60
            .byte   $AD,$82,$C0,$20,$03,$C5,$A9,$FF,$8D,$FB,$04,$AD,$5D,$C0,$AD,$5F
            .byte   $C0,$AD,$61,$C0,$10,$0B,$A9,$00,$8D,$F4,$03,$8D,$F5,$03,$4C,$D7
            .byte   $C7,$A0,$20,$AD,$19,$C0,$10,$FB,$AD,$19,$C0,$30,$FB,$88,$D0,$F3
            .byte   $60,$CA,$D0,$F0,$60,$20,$89,$CA,$D0,$03,$8D,$0A,$C0,$60,$88,$95
            .byte   $8A,$8B,$A4,$24,$B1,$28,$2C,$1F,$C0,$30,$F2,$4C,$26,$CE,$00,$00
;c300
            .byte   $2C,$43,$CE,$70,$12,$38,$90,$18,$B8,$50,$0C,$01,$88,$4A,$50,$56
            .byte   $5C,$4C,$76,$C3,$4C,$C3,$C3,$8D,$7B,$06,$98,$48,$8A,$48,$08,$AD
            .byte   $FB,$04,$2C,$F8,$07,$30,$05,$09,$08,$8D,$FB,$04,$20,$6D,$C3,$28
            .byte   $70,$15,$90,$10,$AA,$10,$0D,$20,$5B,$CD,$68,$AA,$68,$A8,$AD,$7B
            .byte   $06,$6C,$38,$00,$4C,$7C,$C8,$4C,$03,$C8,$20,$6D,$C3,$4C,$B4,$C9
            .byte   $20,$6D,$C3,$4C,$D6,$C9,$20,$6D,$C3,$4C,$F0,$C9,$AA,$F0,$08,$CA
            .byte   $D0,$07,$2C,$00,$C0,$10,$04,$38,$60,$A2,$03,$18,$60,$A2,$C3,$8E
            .byte   $F8,$07,$AE,$FF,$CF,$60,$48,$98,$48,$AD,$13,$C0,$48,$AD,$14,$C0
            .byte   $48,$90,$08,$8D,$02,$C0,$8D,$05,$C0,$B0,$06,$8D,$04,$C0,$8D,$03
            .byte   $C0,$A0,$00,$B1,$3C,$91,$42,$E6,$42,$D0,$02,$E6,$43,$A5,$3C,$C5
            .byte   $3E,$A5,$3D,$E5,$3F,$E6,$3C,$D0,$02,$E6,$3D,$90,$E6,$8D,$04,$C0
            .byte   $68,$10,$03,$8D,$05,$C0,$8D,$02,$C0,$68,$10,$03,$8D,$03,$C0,$68
            .byte   $A8,$68,$60,$48,$AD,$ED,$03,$48,$AD,$EE,$03,$48,$90,$08,$8D,$03
            .byte   $C0,$8D,$05,$C0,$B0,$06,$8D,$02,$C0,$8D,$04,$C0,$68,$8D,$EE,$03
            .byte   $68,$8D,$ED,$03,$68,$70,$05,$8D,$08,$C0,$50,$03,$8D,$09,$C0,$6C
            .byte   $ED,$03,$00,$00,$8D,$81,$C0,$4C,$7A,$FC,$2C,$15,$C0,$8D,$07,$C0
;
            .byte   $4C,$00,$C6,$01,$18,$48,$48,$48,$8A,$BA,$E8,$E8,$E8,$E8,$48,$98
            .byte   $48,$BD,$00,$01,$29,$10,$A8,$AD,$18,$C0,$2D,$1C,$C0,$29,$80,$F0
            .byte   $05,$A9,$20,$8D,$54,$C0,$2A,$2C,$13,$C0,$10,$05,$8D,$02,$C0,$09
            .byte   $20,$2C,$14,$C0,$10,$05,$8D,$04,$C0,$09,$10,$2C,$12,$C0,$10,$0C
            .byte   $09,$0C,$2C,$11,$C0,$10,$02,$49,$06,$8D,$81,$C0,$2C,$16,$C0,$10
            .byte   $0D,$BA,$8E,$01,$01,$AE,$00,$01,$9A,$8D,$08,$C0,$09,$80,$88,$30
            .byte   $0C,$85,$44,$68,$A8,$68,$AA,$68,$68,$68,$4C,$47,$FA,$48,$AD,$F8
            .byte   $07,$48,$A9,$C3,$48,$A9,$F4,$48,$08,$4C,$06,$C5,$AD,$81,$C0,$68
            .byte   $10,$07,$8D,$09,$C0,$AE,$01,$01,$9A,$A0,$06,$10,$06,$BE,$C1,$C4
            .byte   $FE,$00,$C0,$88,$30,$03,$0A,$D0,$F2,$0A,$0A,$68,$A8,$BA,$A9,$40
            .byte   $48,$A9,$C0,$48,$A9,$06,$69,$00,$48,$A9,$8D,$48,$9A,$8A,$69,$03
            .byte   $AA,$38,$E9,$07,$9D,$00,$01,$E8,$A9,$01,$9D,$00,$01,$68,$AA,$68
            .byte   $60,$83,$8B,$8B,$05,$03,$55,$00,$20,$13,$FF,$84,$34,$DD,$B4,$F9
            .byte   $D0,$13,$20,$13,$FF,$DD,$BA,$F9,$F0,$0D,$BD,$BA,$F9,$F0,$07,$C9
            .byte   $A4,$F0,$03,$A4,$34,$18,$88,$26,$44,$E0,$03,$D0,$0D,$20,$A7,$FF
            .byte   $A5,$3F,$F0,$01,$E8,$86,$35,$A2,$03,$88,$86,$3D,$CA,$10,$C9,$60
;
            .byte   $4C,$14,$C5,$4C,$26,$C5,$8D,$A9,$C0,$A9,$02,$8D,$DE,$FF,$20,$3E
            .byte   $C5,$4C,$74,$FC,$78,$8D,$A9,$C0,$A9,$82,$8D,$DD,$FF,$8D,$DE,$FF
            .byte   $20,$3E,$C5,$4C,$7C,$C6,$8D,$A9,$C0,$A9,$23,$8D,$DC,$FF,$A9,$7F
            .byte   $8D,$DE,$FF,$A9,$82,$8D,$DE,$FF,$AD,$DD,$FF,$8D,$DD,$FF,$AC,$A0
            .byte   $C7,$99,$AD,$C0,$60,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;
            .byte   $D8,$38,$30,$01,$18,$48,$48,$48,$8A,$BA,$E8,$E8,$E8,$E8,$48,$98
            .byte   $48,$BD,$00,$01,$29,$10,$A8,$AD,$18,$C0,$2D,$1C,$C0,$29,$80,$F0
            .byte   $05,$A9,$20,$8D,$54,$C0,$2A,$2C,$13,$C0,$10,$05,$8D,$02,$C0,$09
            .byte   $20,$2C,$14,$C0,$10,$05,$8D,$04,$C0,$09,$10,$2C,$12,$C0,$10,$0C
            .byte   $09,$0C,$2C,$11,$C0,$10,$02,$49,$06,$8D,$81,$C0,$2C,$16,$C0,$10
            .byte   $0D,$BA,$8E,$01,$01,$AE,$00,$01,$9A,$8D,$08,$C0,$09,$80,$88,$30
            .byte   $0C,$85,$44,$68,$A8,$68,$AA,$68,$68,$68,$4C,$47,$FA,$48,$AD,$F8
            .byte   $07,$48,$A9,$C3,$48,$A9,$F4,$48,$08,$4C,$06,$C5,$AD,$81,$C0,$68
            .byte   $10,$07,$8D,$09,$C0,$AE,$01,$01,$9A,$A0,$06,$10,$06,$BE,$C1,$C6
            .byte   $FE,$00,$C0,$88,$30,$03,$0A,$D0,$F2,$0A,$0A,$68,$A8,$BA,$A9,$40
            .byte   $48,$A9,$C0,$48,$A9,$06,$69,$00,$48,$A9,$8D,$48,$9A,$8A,$69,$03
            .byte   $AA,$38,$E9,$07,$9D,$00,$01,$E8,$A9,$01,$9D,$00,$01,$68,$AA,$68
            .byte   $60,$83,$8B,$8B,$05,$03,$55,$00,$20,$13,$FF,$84,$34,$DD,$B4,$F9
            .byte   $D0,$13,$20,$13,$FF,$DD,$BA,$F9,$F0,$0D,$BD,$BA,$F9,$F0,$07,$C9
            .byte   $A4,$F0,$03,$A4,$34,$18,$88,$26,$44,$E0,$03,$D0,$0D,$20,$A7,$FF
            .byte   $A5,$3F,$F0,$01,$E8,$86,$35,$A2,$03,$88,$86,$3D,$CA,$10,$C9,$60
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$8D,$06,$C0,$8D,$A1,$C0,$4C,$93,$0D
            .byte   $A0,$0E,$99,$00,$C0,$88,$88,$10,$F9,$20,$84,$FE,$20,$2F,$FB,$20
            .byte   $93,$FE,$4C,$89,$FE,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

; loads into A800
            .byte   $4C,$B0,$C9,$20,$F4,$CE,$20,$2A,$C8,$20,$2E,$CD,$A9,$01,$8D,$FB
            .byte   $04,$20,$89,$CA,$D0,$08,$06,$21,$8D,$01,$C0,$8D,$0D,$C0,$8D,$0F
            .byte   $C0,$20,$90,$CC,$AC,$7B,$05,$4C,$7E,$C8,$A9,$07,$85,$36,$A9,$C3
            .byte   $85,$37,$A9,$05,$85,$38,$A9,$C3,$85,$39,$60,$E6,$4E,$D0,$02,$E6
            .byte   $4F,$AD,$00,$C0,$10,$F5,$8D,$10,$C0,$60,$00,$00,$00,$4C,$50,$C3
            .byte   $A5,$25,$8D,$FB,$05,$A4,$24,$CC,$7B,$04,$F0,$03,$8C,$7B,$05,$A5
            .byte   $21,$18,$ED,$7B,$05,$B0,$05,$A0,$00,$8C,$7B,$05,$AC,$7B,$05,$60
            .byte   $A4,$35,$18,$B0,$38,$8D,$7B,$06,$98,$48,$8A,$48,$B0,$5E,$20,$50
            .byte   $C8,$AD,$7B,$06,$C9,$8D,$D0,$18,$AE,$00,$C0,$10,$13,$E0,$93,$D0
            .byte   $0F,$2C,$10,$C0,$AE,$00,$C0,$10,$FB,$E0,$83,$F0,$03,$2C,$10,$C0
            .byte   $29,$7F,$C9,$20,$B0,$06,$20,$D2,$CA,$4C,$BD,$C8,$AD,$7B,$06,$20
            .byte   $38,$CE,$C8,$8C,$7B,$05,$C4,$21,$90,$03,$20,$51,$CB,$AD,$FB,$04
            .byte   $29,$F7,$8D,$FB,$04,$AD,$7B,$05,$2C,$1F,$C0,$10,$02,$A9,$00,$85
            .byte   $24,$8D,$7B,$04,$68,$AA,$68,$A8,$AD,$7B,$06,$60,$A4,$24,$AD,$7B
            .byte   $06,$91,$28,$20,$50,$C8,$20,$26,$CE,$20,$3B,$C8,$8D,$7B,$06,$20
            .byte   $26,$CE,$A8,$AD,$FB,$04,$29,$08,$F0,$CB,$C0,$8D,$D0,$08,$AD,$FB
            .byte   $04,$29,$F7,$8D,$FB,$04,$C0,$9B,$F0,$11,$C0,$95,$D0,$B7,$AC,$7B
            .byte   $05,$20,$44,$CE,$09,$80,$8D,$7B,$06,$D0,$AA,$20,$B1,$CE,$20,$3B
            .byte   $C8,$20,$C4,$CE,$20,$14,$CE,$29,$7F,$A0,$10,$D9,$7C,$C9,$F0,$05
            .byte   $88,$10,$F8,$30,$0F,$B9,$6B,$C9,$29,$7F,$20,$D6,$CA,$B9,$6B,$C9
            .byte   $30,$D9,$10,$A2,$A8,$AD,$FB,$04,$C0,$11,$D0,$0B,$20,$4D,$CD,$A9
            .byte   $98,$8D,$7B,$06,$4C,$C5,$C8,$C0,$05,$D0,$08,$29,$DF,$8D,$FB,$04
            .byte   $4C,$E6,$C8,$C0,$04,$D0,$F9,$09,$20,$D0,$F2,$0C,$1C,$08,$0A,$1F
            .byte   $1D,$0B,$9F,$88,$9C,$8A,$11,$12,$88,$8A,$9F,$9C,$40,$41,$42,$43
            .byte   $44,$45,$46,$49,$4A,$4B,$4D,$34,$38,$08,$0A,$0B,$15,$2C,$13,$C0
            .byte   $30,$11,$A9,$EE,$8D,$05,$C0,$8D,$03,$C0,$8D,$00,$0C,$8D,$00,$08
            .byte   $CD,$00,$0C,$60,$CA,$CB,$CD,$C9,$00,$00,$AD,$7B,$06,$4C,$56,$C3
            .byte   $A9,$83,$D0,$02,$A9,$81,$48,$4C,$C0,$C9,$F0,$04,$68,$A2,$09,$60
            .byte   $68,$8D,$FB,$04,$8D,$01,$C0,$8D,$0D,$C0,$8D,$0F,$C0,$20,$D4,$CE
            .byte   $20,$90,$CC,$4C,$1F,$CA,$20,$D4,$CE,$20,$3B,$C8,$29,$7F,$8D,$7B
            .byte   $06,$A2,$00,$AD,$FB,$04,$29,$02,$F0,$02,$A2,$C3,$AD,$7B,$06,$60
            .byte   $29,$7F,$AA,$20,$D4,$CE,$A9,$08,$2C,$FB,$04,$D0,$32,$8A,$2C,$2E
            .byte   $CA,$F0,$50,$AC,$7B,$05,$24,$32,$10,$02,$09,$80,$20,$70,$CE,$C8
            .byte   $8C,$7B,$05,$C4,$21,$90,$08,$A9,$00,$8D,$7B,$05,$20,$D8,$CB,$A5
            .byte   $28,$8D,$7B,$07,$A5,$29,$8D,$FB,$07,$20,$1F,$CE,$A2,$00,$60,$20
            .byte   $1F,$CE,$8A,$38,$E9,$20,$2C,$FB,$06,$30,$30,$8D,$FB,$05,$85,$25
            .byte   $20,$BA,$CA,$AD,$FB,$06,$8D,$7B,$05,$A9,$F7,$2D,$FB,$04,$8D,$FB
            .byte   $04,$D0,$CC,$20,$1F,$CE,$8A,$C9,$1E,$F0,$06,$20,$D6,$CA,$4C,$1F
            .byte   $CA,$A9,$08,$0D,$FB,$04,$8D,$FB,$04,$A9,$FF,$8D,$FB,$06,$4C,$29
            .byte   $CA,$AA,$A5,$2A,$A0,$03,$E0,$8A,$F0,$0B,$4A,$90,$08,$4A,$4A,$09
            .byte   $20,$88,$D0,$FA,$C8,$88,$D0,$F2,$60,$A9,$00,$60,$2C,$12,$C0,$10
            .byte   $FA,$A9,$06,$CD,$B3,$FB,$F0,$F3,$A2,$03,$2C,$11,$C0,$30,$02,$A2
            .byte   $0B,$8D,$B3,$FB,$2C,$80,$C0,$AD,$B3,$FB,$C9,$06,$F0,$01,$E8,$2C
            .byte   $81,$C0,$2C,$81,$C0,$A0,$00,$4C,$B7,$F8,$48,$4A,$29,$03,$09,$04
            .byte   $85,$29,$68,$29,$18,$90,$02,$69,$7F,$85,$28,$0A,$0A,$05,$28,$85
            .byte   $28,$60,$2C,$06,$CB,$50,$B8,$8D,$7B,$07,$48,$98,$48,$AC,$7B,$07
            .byte   $C0,$05,$90,$13,$B9,$B4,$CB,$F0,$0E,$50,$12,$30,$10,$8D,$7B,$07
            .byte   $AD,$FB,$04,$29,$28,$F0,$03,$38,$B0,$09,$AD,$7B,$07,$09,$80,$20
            .byte   $07,$CB,$18,$68,$A8,$68,$60,$48,$B9,$99,$CB,$48,$60,$AD,$FB,$04
            .byte   $10,$05,$29,$EF,$8D,$FB,$04,$60,$AD,$FB,$04,$10,$FA,$09,$10,$D0
            .byte   $F3,$A9,$40,$20,$34,$CB,$A0,$C0,$A9,$0C,$20,$34,$CB,$AD,$30,$C0
            .byte   $88,$D0,$F5,$60,$38,$48,$E9,$01,$D0,$FC,$68,$E9,$01,$D0,$F6,$60
            .byte   $CE,$7B,$05,$10,$0B,$A5,$21,$8D,$7B,$05,$CE,$7B,$05,$20,$79,$CB
            .byte   $60,$A9,$00,$8D,$7B,$05,$AD,$FB,$04,$30,$03,$20,$D8,$CB,$60,$A5
            .byte   $22,$85,$25,$A9,$00,$8D,$7B,$05,$4C,$FE,$CD,$EE,$7B,$05,$AD,$7B
            .byte   $05,$C5,$21,$90,$03,$20,$51,$CB,$60,$A5,$22,$C5,$25,$B0,$1E,$C6
            .byte   $25,$4C,$FE,$CD,$AD,$FB,$04,$10,$02,$29,$FB,$A0,$FF,$D0,$09,$AD
            .byte   $FB,$04,$10,$02,$09,$04,$A0,$7F,$8D,$FB,$04,$84,$32,$60,$0C,$17
            .byte   $20,$3F,$00,$D7,$73,$8F,$50,$83,$8E,$00,$E9,$FB,$00,$00,$4C,$D3
            .byte   $EA,$3C,$5E,$95,$43,$6A,$99,$00,$78,$4B,$4B,$CB,$CB,$00,$CB,$4C
            .byte   $4C,$CB,$4B,$4B,$00,$4C,$4C,$00,$00,$4D,$4B,$4B,$4D,$4B,$4C,$4D
            .byte   $4B,$4C,$00,$4B,$A0,$00,$F0,$15,$E6,$25,$A5,$25,$8D,$FB,$05,$C5
            .byte   $23,$B0,$03,$4C,$03,$CE,$CE,$FB,$05,$C6,$25,$A0,$01,$8A,$48,$8C
            .byte   $7B,$07,$A5,$21,$48,$2C,$1F,$C0,$10,$1C,$8D,$01,$C0,$4A,$AA,$A5
            .byte   $20,$4A,$B8,$90,$03,$2C,$06,$CB,$2A,$45,$21,$4A,$70,$03,$B0,$01
            .byte   $CA,$86,$21,$AD,$1F,$C0,$08,$A6,$22,$98,$D0,$03,$A6,$23,$CA,$8A
            .byte   $20,$03,$CE,$A5,$28,$85,$2A,$A5,$29,$85,$2B,$AD,$7B,$07,$F0,$32
            .byte   $E8,$E4,$23,$B0,$32,$8A,$20,$03,$CE,$A4,$21,$28,$08,$10,$1E,$AD
            .byte   $55,$C0,$98,$F0,$07,$B1,$28,$91,$2A,$88,$D0,$F9,$70,$04,$B1,$28
            .byte   $91,$2A,$AD,$54,$C0,$A4,$21,$B0,$04,$B1,$28,$91,$2A,$88,$10,$F9
            .byte   $30,$C1,$CA,$E4,$22,$10,$CE,$28,$68,$85,$21,$20,$96,$CC,$20,$FE
            .byte   $CD,$68,$AA,$60,$20,$9A,$CC,$A5,$25,$48,$10,$06,$20,$03,$CE,$20
            .byte   $96,$CC,$E6,$25,$A5,$25,$C5,$23,$90,$F2,$68,$85,$25,$4C,$FE,$CD
            .byte   $20,$5F,$CB,$4C,$74,$CC,$A0,$00,$F0,$03,$AC,$7B,$05,$A5,$32,$29
            .byte   $80,$09,$20,$2C,$1F,$C0,$30,$15,$91,$28,$C8,$C4,$21,$90,$F9,$60
            .byte   $86,$2A,$A2,$D8,$A0,$14,$A5,$32,$29,$A0,$4C,$D5,$CC,$86,$2A,$48
            .byte   $98,$48,$38,$E5,$21,$AA,$98,$4A,$A8,$68,$45,$20,$6A,$B0,$03,$10
            .byte   $01,$C8,$68,$B0,$0B,$2C,$55,$C0,$91,$28,$2C,$54,$C0,$E8,$F0,$06
            .byte   $91,$28,$C8,$E8,$D0,$EF,$A6,$2A,$38,$60,$AD,$FB,$04,$30,$4D,$20
            .byte   $31,$CD,$2C,$1F,$C0,$10,$12,$20,$91,$CD,$90,$0D,$20,$89,$CA,$D0
            .byte   $3B,$2C,$1F,$C0,$30,$03,$20,$C4,$CD,$AD,$7B,$05,$18,$65,$20,$2C
            .byte   $1F,$C0,$30,$06,$C9,$28,$90,$02,$A9,$27,$8D,$7B,$05,$85,$24,$A5
            .byte   $25,$20,$BA,$CA,$2C,$1F,$C0,$10,$05,$20,$71,$CD,$F0,$03,$20,$6D
            .byte   $CD,$A9,$00,$2C,$1A,$C0,$30,$02,$A9,$14,$85,$22,$60,$AD,$FB,$04
            .byte   $09,$01,$D0,$05,$AD,$FB,$04,$29,$FE,$8D,$FB,$04,$60,$AD,$FB,$04
            .byte   $30,$1A,$20,$2E,$CD,$20,$80,$CD,$20,$64,$CD,$A9,$FD,$85,$39,$A9
            .byte   $1B,$85,$38,$60,$A9,$FD,$85,$37,$A9,$F0,$85,$36,$60,$A9,$28,$D0
            .byte   $02,$A9,$50,$85,$21,$A9,$18,$85,$23,$A9,$00,$85,$22,$85,$20,$60
            .byte   $2C,$1F,$C0,$10,$03,$20,$EF,$CC,$8D,$0E,$C0,$A9,$FF,$8D,$FB,$04
            .byte   $60,$8A,$48,$A2,$17,$8D,$01,$C0,$8A,$20,$BA,$CA,$A0,$27,$84,$2A
            .byte   $98,$4A,$B0,$03,$2C,$55,$C0,$A8,$B1,$28,$2C,$54,$C0,$A4,$2A,$91
            .byte   $28,$88,$10,$EA,$CA,$30,$04,$E4,$22,$B0,$DD,$8D,$00,$C0,$8D,$0C
            .byte   $C0,$4C,$F8,$CD,$8A,$48,$A2,$17,$8A,$20,$BA,$CA,$A0,$00,$8D,$01
            .byte   $C0,$B1,$28,$84,$2A,$48,$98,$4A,$B0,$03,$8D,$55,$C0,$A8,$68,$91
            .byte   $28,$8D,$54,$C0,$A4,$2A,$C8,$C0,$28,$90,$E6,$20,$B0,$CC,$CA,$30
            .byte   $04,$E4,$22,$B0,$D3,$8D,$0D,$C0,$20,$FE,$CD,$68,$AA,$60,$A5,$25
            .byte   $8D,$FB,$05,$20,$BA,$CA,$A5,$20,$2C,$1F,$C0,$10,$01,$4A,$18,$65
            .byte   $28,$85,$28,$60,$C9,$E1,$90,$06,$C9,$FB,$B0,$02,$29,$DF,$60,$AD
            .byte   $FB,$04,$29,$10,$D0,$11,$48,$98,$48,$AC,$7B,$05,$20,$44,$CE,$49
            .byte   $80,$20,$70,$CE,$68,$A8,$68,$60,$48,$24,$32,$30,$02,$29,$7F,$20
            .byte   $70,$CE,$68,$60,$B1,$28,$2C,$1F,$C0,$10,$19,$8D,$01,$C0,$84,$2A
            .byte   $98,$45,$20,$6A,$B0,$04,$AD,$55,$C0,$C8,$98,$4A,$A8,$B1,$28,$2C
            .byte   $54,$C0,$A4,$2A,$2C,$1E,$C0,$10,$06,$C9,$20,$B0,$02,$09,$40,$60
            .byte   $48,$29,$FF,$30,$16,$AD,$FB,$04,$6A,$68,$48,$90,$0E,$2C,$1E,$C0
            .byte   $10,$09,$49,$40,$2C,$AC,$CE,$F0,$02,$49,$40,$2C,$1F,$C0,$10,$1D
            .byte   $8D,$01,$C0,$48,$84,$2A,$98,$45,$20,$4A,$B0,$04,$AD,$55,$C0,$C8
            .byte   $98,$4A,$A8,$68,$91,$28,$AD,$54,$C0,$A4,$2A,$68,$60,$91,$28,$68
            .byte   $60,$48,$98,$48,$AC,$7B,$05,$20,$44,$CE,$8D,$7B,$06,$29,$80,$49
            .byte   $AB,$4C,$CD,$CE,$48,$98,$48,$AC,$7B,$05,$AD,$7B,$06,$20,$70,$CE
            .byte   $68,$A8,$68,$60,$20,$71,$CD,$A9,$FF,$85,$32,$AD,$FB,$04,$29,$04
            .byte   $F0,$02,$46,$32,$AD,$7B,$07,$85,$28,$AD,$FB,$07,$85,$29,$AD,$FB
            .byte   $05,$85,$25,$60,$4C,$8C,$CA,$10,$3D,$A9,$06,$CD,$B3,$FB,$F0,$36
            .byte   $A2,$03,$2C,$11,$C0,$30,$02,$A2,$0B,$8D,$B3,$FB,$2C,$80,$C0,$AD
            .byte   $B3,$FB,$C9,$06,$F0,$01,$E8,$2C,$81,$C0,$2C,$81,$C0,$A0,$00,$A9
            .byte   $F8,$85,$37,$84,$36,$B1,$36,$91,$36,$C8,$D0,$F9,$E6,$37,$D0,$F5
            .byte   $BD,$80,$C0,$BD,$80,$C0,$60,$00,$00,$00,$E9,$81,$4A,$D0,$14,$A4
            .byte   $3F,$A6,$3E,$D0,$01,$88,$CA,$8A,$18,$E5,$3A,$85,$3E,$10,$01,$C8
            .byte   $98,$E5,$3B,$D0,$40,$A4,$2F,$B9,$3D,$00,$91,$3A,$88,$10,$F8,$20
            .byte   $48,$F9,$20,$1A,$FC,$20,$1A,$FC,$4C,$E3,$FC,$A5,$3D,$20,$8E,$F8
            .byte   $AA,$BD,$00,$FA,$C5,$42,$D0,$13,$BD,$C0,$F9,$C5,$43,$D0,$0C,$A5
            .byte   $44,$A4,$2E,$C0,$9D,$F0,$B3,$C5,$2E,$F0,$CA,$C6,$3D,$D0,$DC,$E6
            .byte   $44,$C6,$35,$F0,$D6,$A4,$34,$98,$AA,$4C,$D2,$FC,$20,$C7,$FF,$AD
            .byte   $00,$02,$C9,$A0,$F0,$12,$C9,$8D,$D0,$01,$60,$20,$A7,$FF,$C9,$93
            .byte   $D0,$E5,$8A,$F0,$E2,$20,$78,$FE,$A9,$03,$85,$3D,$20,$13,$FF,$0A
            .byte   $E9,$BE,$C9,$C2,$90,$D1,$0A,$0A,$A2,$04,$0A,$26,$42,$26,$43,$CA
            .byte   $10,$F8,$C6,$3D,$F0,$F4,$10,$E4,$A2,$05,$20,$C8,$C4,$A5,$44,$0A
            .byte   $0A,$05,$35,$C9,$20,$B0,$06,$A6,$35,$F0,$02,$09,$80,$85,$44,$84
            .byte   $34,$B9,$00,$02,$C9,$BB,$F0,$04,$C9,$8D,$D0,$B4,$4C,$6B,$CF,$DF

;loads into B000
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$00,$33,$34,$35,$0E,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$80,$B3,$B4,$B5,$9E,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
            .byte   $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
            .byte   $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
            .byte   $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
            .byte   $30,$31,$00,$33,$34,$35,$0E,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
            .byte   $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
            .byte   $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
            .byte   $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
            .byte   $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
            .byte   $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
            .byte   $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
            .byte   $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
            .byte   $B0,$B1,$80,$B3,$B4,$B5,$9E,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
            .byte   $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
            .byte   $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
            .byte   $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
            .byte   $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF

; loads int B800
            .byte   $20,$2D,$C9,$A9,$16,$8D,$FA,$07,$A9,$01,$8D,$FA,$05,$A9,$00,$8D
            .byte   $7A,$05,$8D,$FA,$06,$8D,$7A,$07,$8D,$7A,$06,$AD,$A2,$C7,$AE,$A1
            .byte   $C7,$8D,$F1,$C0,$8D,$F2,$C0,$8E,$F3,$C0,$60,$AD,$FA,$07,$10,$FA
            .byte   $AD,$7A,$05,$29,$38,$F0,$03,$4C,$88,$C2,$A5,$27,$4C,$F6,$FD,$20
            .byte   $2D,$C9,$20,$DA,$C8,$0A,$4A,$4C,$90,$C9,$00,$00,$00,$20,$3F,$C8
            .byte   $AE,$F8,$07,$8D,$7A,$06,$60,$AD,$FA,$07,$4A,$4A,$A5,$27,$90,$02
            .byte   $29,$7F,$20,$E5,$C8,$90,$FB,$8D,$F0,$C0,$60,$20,$B5,$C9,$90,$15
            .byte   $AD,$F0,$C0,$09,$80,$C9,$8A,$D0,$09,$AD,$FA,$07,$29,$20,$D0,$04
            .byte   $A9,$8A,$38,$60,$18,$60,$AD,$FA,$07,$29,$08,$F0,$F8,$A5,$27,$0E
            .byte   $7A,$05,$49,$94,$D0,$03,$38,$B0,$05,$49,$06,$D0,$01,$18,$6E,$7A
            .byte   $05,$60,$AD,$7A,$05,$29,$07,$F0,$05,$20,$75,$C9,$38,$60,$A5,$27
            .byte   $29,$7F,$CD,$FA,$05,$D0,$05,$EE,$7A,$05,$38,$60,$AD,$FA,$07,$29
            .byte   $08,$F0,$0C,$20,$6B,$C8,$90,$07,$C9,$93,$F0,$05,$8D,$7A,$07,$18
            .byte   $60,$20,$DA,$C8,$C9,$91,$D0,$F9,$18,$60,$20,$6B,$C8,$B0,$05,$20
            .byte   $AD,$CA,$90,$F6,$60,$48,$AD,$F1,$C0,$49,$10,$29,$70,$18,$D0,$01
            .byte   $38,$68,$60,$C9,$30,$90,$2C,$C9,$3A,$B0,$28,$29,$0F,$48,$AD,$FA
            .byte   $04,$0A,$0A,$6D,$FA,$04,$0A,$8D,$FA,$04,$68,$6D,$FA,$04,$8D,$FA
            .byte   $04,$38,$60,$A0,$05,$D9,$FB,$C9,$F0,$F7,$88,$48,$B9,$FB,$C9,$0A
            .byte   $68,$90,$F2,$18,$60,$A0,$11,$D0,$EC,$A0,$0A,$D0,$E8,$8E,$F8,$07
            .byte   $A9,$00,$8D,$7A,$06,$60,$AD,$7A,$05,$10,$FA,$0E,$FA,$07,$4E,$FA
            .byte   $07,$A4,$24,$B1,$28,$85,$27,$A9,$FF,$91,$28,$AD,$7A,$05,$10,$1E
            .byte   $20,$6B,$C8,$90,$0C,$20,$99,$C9,$20,$86,$C8,$20,$30,$C8,$4C,$41
            .byte   $C9,$20,$BF,$CA,$90,$E1,$70,$D9,$20,$E1,$C9,$4C,$41,$C9,$20,$99
            .byte   $C9,$A9,$8D,$D0,$32,$A8,$A9,$CA,$48,$B9,$81,$C9,$48,$A5,$27,$29
            .byte   $7F,$60,$1E,$55,$78,$85,$91,$BF,$C4,$CE,$C1,$CD,$CD,$CF,$C3,$8D
            .byte   $A8,$98,$AC,$F8,$07,$AE,$7A,$06,$60,$48,$A4,$24,$A5,$27,$91,$28
            .byte   $68,$C9,$95,$D0,$02,$A5,$27,$85,$27,$60,$85,$27,$20,$2D,$C9,$20
            .byte   $DC,$C9,$4C,$90,$C9,$48,$A5,$35,$48,$AD,$F1,$C0,$85,$35,$4A,$4A
            .byte   $48,$4A,$4A,$68,$45,$35,$29,$08,$45,$35,$29,$0F,$B0,$02,$29,$08
            .byte   $F0,$02,$09,$20,$8D,$7A,$06,$68,$85,$35,$68,$60,$20,$A2,$C8,$B0
            .byte   $C8,$20,$2B,$C8,$20,$57,$C8,$A5,$27,$0A,$4A,$C9,$0D,$D0,$BA,$AD
            .byte   $FA,$07,$4A,$90,$B4,$A9,$8A,$85,$27,$D0,$E9,$80,$42,$54,$5A,$51
            .byte   $52,$80,$42,$53,$44,$50,$80,$45,$46,$4C,$4D,$58,$48,$00,$0F,$28
            .byte   $36,$30,$3E,$00,$4C,$7A,$5F,$6B,$00,$90,$99,$A6,$AF,$BA,$C5,$48
            .byte   $A9,$00,$8D,$FA,$04,$68,$C9,$0D,$D0,$02,$F0,$3F,$C9,$20,$B0,$04
            .byte   $8D,$FA,$05,$60,$20,$25,$C9,$90,$08,$98,$8D,$FA,$04,$A9,$04,$D0
            .byte   $2C,$20,$F3,$C8,$90,$04,$A9,$02,$D0,$23,$20,$13,$C9,$90,$36,$20
            .byte   $A2,$CA,$A9,$03,$D0,$17,$20,$F3,$C8,$B0,$2A,$C9,$0D,$F0,$0C,$20
            .byte   $29,$C9,$90,$21,$20,$A2,$CA,$A9,$03,$D0,$02,$A9,$00,$4D,$7A,$05
            .byte   $29,$07,$4D,$7A,$05,$8D,$7A,$05,$60,$C9,$0D,$F0,$EE,$C9,$20,$D0
            .byte   $04,$A9,$01,$D0,$E8,$60,$C9,$0D,$F0,$E1,$C9,$20,$D0,$F7,$A9,$05
            .byte   $D0,$DB,$C9,$0D,$F0,$D5,$C9,$45,$F0,$05,$C9,$44,$D0,$E3,$18,$AC
            .byte   $FA,$04,$A9,$CB,$48,$B9,$0D,$CA,$48,$AD,$FA,$04,$60,$18,$AD,$FA
            .byte   $07,$29,$04,$F0,$09,$AD,$00,$C0,$10,$04,$8D,$10,$C0,$38,$60,$E6
            .byte   $4E,$D0,$02,$E6,$4F,$20,$AD,$CA,$B8,$90,$F3,$20,$99,$C9,$29,$7F
            .byte   $CD,$FA,$05,$D0,$E8,$A0,$08,$B9,$87,$C9,$85,$27,$98,$48,$20,$30
            .byte   $C8,$68,$A8,$88,$10,$F1,$AD,$7A,$05,$29,$F8,$09,$01,$8D,$7A,$05
            .byte   $20,$B5,$CA,$10,$FB,$C9,$88,$F0,$DC,$85,$27,$20,$30,$C8,$20,$A2
            .byte   $C8,$AD,$7A,$05,$29,$07,$D0,$E8,$2C,$58,$FF,$A9,$8D,$4C,$A7,$C9
            .byte   $AD,$F2,$C0,$48,$09,$0C,$8D,$F2,$C0,$A0,$C8,$A9,$00,$69,$01,$D0
            .byte   $FC,$88,$D0,$F7,$68,$8D,$F2,$C0,$60,$0E,$7A,$05,$38,$6E,$7A,$05
            .byte   $60,$0E,$7A,$05,$18,$90,$F6,$0E,$FA,$05,$38,$6E,$FA,$05,$60,$8D
            .byte   $F1,$C0,$20,$93,$FE,$20,$89,$FE,$AE,$F8,$07,$D0,$E4,$29,$0F,$D0
            .byte   $03,$AD,$A2,$C7,$4D,$F3,$C0,$29,$0F,$4D,$F3,$C0,$8D,$F3,$C0,$60
            .byte   $6A,$6A,$6A,$6A,$4D,$F3,$C0,$29,$60,$4C,$59,$CB,$6A,$6A,$6A,$6A
            .byte   $4D,$F2,$C0,$29,$E0,$4D,$F2,$C0,$4C,$25,$CB,$0A,$0A,$0A,$29,$38
            .byte   $4D,$7A,$05,$29,$38,$4D,$7A,$05,$8D,$7A,$05,$A9,$00,$8D,$FA,$06
            .byte   $60,$08,$0E,$FA,$07,$28,$6E,$FA,$07,$60,$AD,$FA,$07,$29,$FB,$90
            .byte   $02,$09,$04,$8D,$FA,$07,$60,$08,$4E,$FA,$07,$28,$2E,$FA,$07,$60
            .byte   $AD,$FA,$07,$29,$DF,$90,$EC,$09,$20,$D0,$E8,$AD,$FA,$07,$29,$F7
            .byte   $90,$E1,$09,$08,$D0,$DD,$AD,$FA,$07,$29,$FD,$90,$D6,$09,$02,$D0
            .byte   $D2,$A9,$09,$8D,$79,$04,$AD,$A7,$C7,$AE,$A6,$C7,$20,$21,$C8,$AD
            .byte   $A3,$C7,$8D,$79,$05,$AD,$A4,$C7,$8D,$F9,$05,$A9,$01,$8D,$79,$07
            .byte   $A9,$00,$8D,$F9,$06,$8D,$F9,$04,$8D,$79,$06,$8D,$F9,$07,$60,$A5
            .byte   $27,$29,$7F,$C9,$20,$B0,$11,$C9,$0D,$F0,$05,$29,$1F,$8D,$79,$04
            .byte   $A9,$00,$8D,$F9,$07,$4C,$BB,$C1,$A9,$00,$8D,$F9,$04,$A5,$27,$29
            .byte   $7F,$C9,$30,$90,$16,$C9,$3A,$B0,$12,$A0,$0A,$29,$0F,$18,$6D,$F9
            .byte   $04,$88,$D0,$F9,$8D,$F9,$04,$A9,$80,$D0,$D7,$AC,$F9,$04,$C9,$4E
            .byte   $D0,$05,$8C,$F9,$05,$F0,$1D,$C9,$43,$D0,$07,$A9,$FF,$8D,$79,$06
            .byte   $D0,$12,$C9,$46,$D0,$05,$8C,$F9,$05,$F0,$09,$C9,$5A,$D0,$05,$A9
            .byte   $0D,$8D,$79,$04,$A9,$00,$8D,$F9,$04,$F0,$A5,$48,$A5,$24,$CD,$F9
            .byte   $04,$90,$0C,$F0,$0A,$A9,$20,$EE,$F9,$04,$20,$CB,$CC,$D0,$ED,$68
            .byte   $20,$CB,$CC,$EE,$F9,$04,$AD,$79,$05,$F0,$32,$AD,$F9,$04,$CD,$79
            .byte   $05,$90,$2A,$AD,$79,$06,$F0,$03,$20,$C4,$CC,$EE,$F9,$06,$AD,$F9
            .byte   $06,$CD,$F9,$05,$90,$10,$AD,$F9,$05,$F0,$0B,$20,$BE,$CC,$20,$BE
            .byte   $CC,$A9,$00,$8D,$F9,$06,$A9,$00,$85,$24,$8D,$F9,$04,$60,$20,$C4
            .byte   $CC,$20,$C4,$CC,$A9,$0D,$20,$CB,$CC,$A9,$0A,$48,$AD,$A5,$C7,$29
            .byte   $04,$F0,$04,$68,$4C,$DA,$CC,$68,$29,$7F,$4C,$62,$C8,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$20,$00,$00,$00,$03,$00,$00,$00,$00,$00,$00,$00,$08,$78,$20
            .byte   $CC,$C2,$4C,$15,$CD,$AD,$02,$C4,$A5,$42,$D0,$0B,$AE,$A8,$C7,$AC
            .byte   $A9,$C7,$28,$A9,$00,$18,$60,$C9,$03,$D0,$05,$28,$38,$A9,$27,$60
            .byte   $A0,$00,$AD,$11,$C0,$10,$02,$A0,$08,$8C,$7D,$04,$A5,$3E,$48,$A5
            .byte   $3F,$48,$A5,$42,$C9,$02,$F0,$0A,$A5,$46,$A6,$47,$20,$C5,$C7,$4C
            .byte   $55,$CD,$20,$F5,$C7,$20,$CD,$C5,$20,$06,$C5,$AC,$A0,$C7,$99,$AD
            .byte   $C0,$A0,$00,$90,$02,$A0,$27,$68,$85,$3F,$68,$85,$3E,$28,$98,$18
            .byte   $F0,$01,$38,$60,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$4F,$0D
            .byte   $2C,$58,$FF,$70,$0C,$38,$90,$18,$B8,$50,$06,$01,$31,$69,$82,$85
            .byte   $6F,$85,$27,$8A,$48,$98,$48,$8D,$FF,$CF,$A2,$C2,$8E,$F8,$07,$50
            .byte   $29,$A9,$EF,$20,$05,$C8,$AE,$F8,$07,$E4,$37,$D0,$15,$A9,$07,$C5
            .byte   $36,$F0,$0F,$85,$36,$20,$DC,$C9,$20,$36,$C9,$68,$A8,$68,$AA,$A5
            .byte   $27,$60,$E4,$39,$D0,$EF,$A9,$05,$85,$38,$90,$E9,$AD,$7A,$07,$10
            .byte   $05,$4E,$7A,$07,$D0,$0A,$20,$BF,$CA,$B0,$DD,$20,$6B,$C8,$90,$F6
            .byte   $20,$99,$C9,$20,$86,$C8,$18,$90,$CF,$20,$00,$C8,$A2,$00,$60,$4A
            .byte   $20,$2D,$C9,$90,$05,$20,$B5,$C9,$F0,$03,$20,$E5,$C8,$AD,$7A,$06
            .byte   $AA,$60,$4C,$3F,$C8,$4C,$AA,$C9,$A5,$36,$48,$A5,$37,$48,$A5,$38
            .byte   $48,$A5,$39,$48,$AD,$FA,$06,$85,$36,$AD,$7A,$05,$4A,$4A,$4A,$29
            .byte   $07,$09,$C0,$85,$37,$8A,$48,$A5,$27,$48,$09,$80,$20,$ED,$FD,$8D
            .byte   $FF,$CF,$68,$85,$27,$68,$8D,$F8,$07,$AA,$A5,$36,$8D,$FA,$06,$68
            .byte   $85,$39,$68,$85,$38,$68,$85,$37,$68,$85,$36,$60,$8D,$FF,$CF,$60
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
            .byte   $2C,$58,$FF,$70,$02,$38,$90,$18,$B8,$50,$06,$01,$31,$C2,$DE,$E1
            .byte   $D1,$85,$27,$8A,$48,$98,$48,$20,$CC,$C2,$A2,$C2,$8E,$F8,$07,$B0
            .byte   $07,$20,$D1,$CB,$A9,$05,$85,$36,$A5,$27,$29,$7F,$C9,$0D,$D0,$15
            .byte   $AD,$F9,$07,$D0,$10,$AD,$A5,$C7,$29,$08,$F0,$09,$A5,$27,$20,$CB
            .byte   $CC,$A9,$0A,$85,$27,$AD,$F9,$07,$30,$07,$29,$40,$F0,$06,$4C,$FF
            .byte   $CB,$4C,$1D,$CC,$AD,$79,$04,$C9,$0D,$F0,$5B,$45,$27,$29,$7F,$D0
            .byte   $0A,$AD,$F9,$07,$09,$40,$8D,$F9,$07,$D0,$48,$A5,$27,$29,$7F,$C9
            .byte   $0A,$D0,$0A,$A5,$27,$20,$CB,$CC,$20,$9B,$CC,$F0,$22,$C9,$0C,$D0
            .byte   $19,$AD,$F9,$05,$F0,$0A,$20,$98,$CC,$AD,$F9,$06,$D0,$F8,$F0,$0F
            .byte   $A9,$0C,$20,$CB,$CC,$20,$AE,$CC,$F0,$05,$A5,$27,$20,$6B,$CC,$AD
            .byte   $F9,$04,$F0,$0D,$AD,$F9,$04,$18,$ED,$79,$05,$E9,$F7,$90,$04,$69
            .byte   $1F,$85,$24,$4C,$BB,$C1,$A5,$27,$20,$DA,$CC,$68,$A8,$68,$AA,$A5
            .byte   $27,$60,$85,$27,$A9,$00,$48,$98,$48,$20,$CC,$C2,$20,$D1,$CB,$F0
            .byte   $EA,$C9,$00,$D0,$1C,$20,$CC,$C2,$20,$E5,$C8,$A2,$00,$60,$A2,$03
            .byte   $60,$85,$27,$A9,$00,$48,$98,$48,$8E,$F8,$07,$20,$CC,$C2,$4C,$45
            .byte   $C1,$18,$90,$E7,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

CODE0C00    = *

            
CODELEN     =       * + $1400 - CODEST